<?php

/*
 This program is distributed under the terms of the GNU General Public License.
 License. For details visit http://www.gnu.org/copyleft/gpl.html

 Copyright (C) 2001-2006  Kevin L. Papendick <kevinp@polarlava.com>
*/


/**
 * Localization Class
 *
 * This class provides local language text substitution.
 *
 * @package Localization
 * @author Kevin L. Papendick <kevinp@polarlava.com>
 * @copyright 2004 Kevin L. Papendick <kevinp@polarlava.com>
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @version V1.0 - 20041016
 */
class Localization {

    /**
     * The identifier of the language to be used.
     * @var string
     * @link http://www.gnu.org/software/gettext/manual/html_chapter/gettext_15.html#SEC221
     * @link http://www.gnu.org/software/gettext/manual/html_chapter/gettext_16.html#SEC222
     */
	var $language;

    /**
     * The language file directory.
     * @var string
     */
	var $language_dir;

    /**
     * The language filename.
     * @var string
     */
	var $language_file;

    /**
     * The translation array.
     * @var array
     */
	var $translation;



	/**
	 * Localization Object Constructor
	 *
	 * @param string $lang The language identifier token.
	 * @param string $dir The language file directory.
	 * @return boolean
	 */
	function Localization($lang = 'en_US', $lang_dir)
	{
		$this->setLanguage($lang);
		$this->setLanguageDir($lang_dir);
		$this->setLanguageFile();
		$this->setTranslation();
	}


//---------------------//
//--- SET FUNCTIONS ---//
//---------------------//

	/**
	 * Sets the identifier of the language to be used.
	 *
	 * @param string $a
	 * @return void
	 */
	function setLanguage($a)
	{
		$this->language = $a;
	}


	/**
	 * Sets the language file directory.
	 *
	 * @param string $a
	 * @return void
	 */
	function setLanguageDir($a)
	{
		$this->language_dir = $a;
	}


	/**
	 * Sets the language filename.
	 *
	 * @return boolean
	 */
	function setLanguageFile()
	{
		$stat = FALSE;

		$lang = $this->getLanguage();
		if (isset($lang) && strlen($lang) > 0) {
			$this->language_file = $lang . '.lang';
			$stat = TRUE;
		}

		return $stat;
	}


	/**
	 * Load the localization data from a file.
	 *
	 * @return boolean
	 */
	function setTranslation()
	{
		$stat = FALSE;
		$filename = $this->getLanguageDir() . $this->getLanguageFile();

		if ($file = file($filename)) {
			//--- SPL Support
			$ver = (int) substr(phpversion(), 0, 1);
			if (($ver >= 5)) {
				foreach (new ArrayObject($file) as $line) {
					list($key, $value) = preg_split('/\s+/', $line, 2);
					$this->translation[$key] = trim($value);
				}
			} else {
				foreach ($file as $line) {
					list($key, $value) = preg_split('/\s+/', $line, 2);
					$this->translation[$key] = trim($value);
				}
			}
			$stat = TRUE;
		}

		return $stat;
	}


//---------------------//
//--- GET FUNCTIONS ---//
//---------------------//


	/**
	 * Gets a localized text string.
	 *
	 * @return string $this->translation[$a]
	 */
	function get($a)
	{
		return (array_key_exists($a, $this->translation)) ? $this->translation[$a] : '***' . $a . '***';
	}


	/**
	 * Gets the language identifier.
	 *
	 * @return string $this->language
	 */
	function getLanguage()
	{
		return $this->language;
	}


	/**
	 * Gets the language directory.
	 *
	 * @return string $this->language_dir
	 */
	function getLanguageDir()
	{
		return $this->language_dir;
	}


	/**
	 * Gets the language filename.
	 *
	 * @return string $this->language_file
	 */
	function getLanguageFile()
	{
		return $this->language_file;
	}

} // end of class
?>