<?php

/*
 This program is distributed under the terms of the GNU General Public License.
 License. For details visit http://www.gnu.org/copyleft/gpl.html

 Copyright (C) 2001-2006  Kevin L. Papendick <kevinp@polarlava.com>
*/

//--- REVISIONS
/*
	V1.0 - Initial Release
	V1.1 - Change debug info reported in e-mail for getError() - klp 20031022
	V1.2 - Add resource identifier to mysql_*() functions in query() and setError().
			 This fixes an object cross-contamination problem when using a standard
			 connection with multiple objects to different databases. - klp 20040607
	V1.2.1 - Added detection to determine if magic quotes is enabled or not in the
				query() function.  If not, the query is run through the mysql_escape_string()
				function. - klp 20040709
	V1.3 - Redocumented per phpDocumentor.
			 Some moderate code cleanup.
			 Rolledback V1.2.1 changes as it should only be applied to the values not the
			 whole query.
			 Address known PHP V5 issue in getError().
			 Added support for SafeSQL via safeQuery().
			 - klp 20040819
	V1.3.1 - Detection of SafeSQL object in safeQuery() was inverted! - klp 20040902
	V1.3.2 - Added stripArray() method for use with safeQuery() - klp 20040903
	V1.4.0 - Added setFieldNames() and getFieldNames() - klp 20050219
	V1.4.1 - BUG FIX: query() method not setting an error and then blowing up
		     when calling setResultCnt() on bad SQL.  - klp 20050302
	V1.5.0 - Added getDbList() - klp 20050412
	V1.5.1 - Added setQuery() & getQuery() for debugging - klp 20050622
	V1.6.0 - Added error logging [setErrorLog(), getErrorLog(), logError()] and
			 query EXPLAIN logging [setExplainLog(), getExplainLog(), explainQuery(), logExplain()]. - klp 20050701
	V1.6.1 - BUG FIX: Notices being generated because $explain_log being checked without being set first.  Added
			 setting it to default in the constructor. - klp 20050817
	v1.6.2 - BUG FIX: stripArray() not handling nested arrays properly!  Corrected by detecting if the
			 value is an array and doing a recursive call to stripArray(). - klp 20051009
	v1.6.3 - BUG FIX: Correct error setting problem when DB connection fails in when calling connect() from the constructor - klp 20060105
	v1.6.4 - BUG FIX: setErrorLog(FALSE) in constructor so default value is set. - klp 20060107
	v1.6.5 - BUG FIX: getError() returning generic fail message even when there was no error when not in debug mode.
			 BUG FIX: query() not properly trapping and reporting DB selection error. - klp 20060115
	v1.6.6 - BUG FIX: Existing error not being reset on subsequent query() call.  Added setError(FALSE).
			 BUG FIX: Existing result set not being reset on subsequent query() call.  Added setError(FALSE).
			 BUG FIX: logError() from setError() logs a blank entry on error reset. [setError(FALSE)] -klp 20060121
	V1.7.0 - Added checking for empty error message in setError() and providing a debug_backtrace() instead when calling logError().
			 BUG FIX: Check for FALSE in setError() so error logging doesn't record error resets. -klp 20060809
*/


/**
 * MySqlDb Abstract Database Class
 *
 * This class provides a simple interface to MySQL databases.
 * Users are recommended to sub-class this class, but direct
 * instantiation may also be used.
 *
 * @package MySqlDb
 * @author Kevin L. Papendick <kevinp@polarlava.com>
 * @copyright 2001-2004 Kevin L. Papendick <kevinp@polarlava.com>
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @version V1.4.0 - 20050219
 */
class MySqlDb
{

   /**
     * The database connection object.
     * @var object
     */
	var $db;

    /**
     * The database host server name.
     * @var string
     */
	var $db_host;

    /**
     * The database connection port.
     * @var int
     */
	var $db_port;

    /**
     * The active database name.
     * @var string
     */
	var $db_name;

    /**
     * The database connection username.
     * @var string
     */
	var $db_user;

    /**
     * The database connection password.
     * @var string
     */
	var $db_pw;

    /**
     * The database connection type, either 'standard' or 'persistent'.
     * @var string
     */
	var $db_connect_type;

    /**
     * The database error text.
     * @var string
     */
	var $db_error;

    /**
     * A fully qualified path to the error log file.
     * @var string
     */
	var $error_log;

    /**
     * The database query results.
     * @var resource|boolean
     */
	var $result;

    /**
     * Array of field (column) names.
     * @var array
     */
	var $field_names;

    /**
     * The executed query.
     * @var string
     */
	var $query;

    /**
     * The number of rows affected or returned by a query.
     * @var int
     */
	var $result_cnt;

    /**
     * A single row from the results of a query.
     * @var array
     */
	var $row;

    /**
     * Enable/Disable debug mode.
     * @var boolean
     */
	var $debug;

    /**
     * Enable/Disable emailing of errors.
     * @var boolean
     */
	var $debug_mail;

    /**
     * Email 'To' address for error emailing.
     * @var string
     */
	var $debug_email_to;

    /**
     * Email 'From' address for error emailing.
     * @var string
     */
	var $debug_email_from;

    /**
     * Flags if SafeSQL is available.
     * @var boolean
 	  * @link http://www.phpinsider.com/php/code/SafeSQL/ SafeSQL query processer
    */
	var $safe_sql_avail;

    /**
     * Fully qualified path to SafeSQL class library.
     * @var string
 	  * @link http://www.phpinsider.com/php/code/SafeSQL/ SafeSQL query processer
    */
	var $safe_sql_lib;

   /**
     * The database connection object.
     * @var object
     */
	var $safeSqlObj;



	/**
	 * MySQL Database Object Constructor
	 *
	 * @param string $host The database host server name.
	 * @param string $name The database name.
	 * @param string $user The database connection username.
	 * @param string $passwd The database connection password.
	 * @param string $connect (optional) The database connection password.
	 * @param string $port (optional) The database connection port.
	 * @return boolean
	 */
	function MySqlDb($host, $name, $user, $passwd, $connect = '', $port = '')
	{
		//--- Set Connection Info
		$this->setDbHost($host);
		$this->setDbName($name);
		$this->setDbUser($user);
		$this->setDbPassword($passwd);

		if (!empty($connect)) {
			$this->setDbConnectionType($connect);
		} else {
			$this->setDbConnectionType('standard');
		}
		if (!empty($port)) {
			$this->setDbPort($port);
		} else {
			$this->setDbPort(3306);
		}

		//--- Defaults
		$this->row = array();
		$this->setDebug(FALSE);
		$this->setDebugMail(FALSE);
		$this->setSafeSql(FALSE);
		$this->setExplainLog(FALSE);
		$this->setErrorLog(FALSE);

		//--- Establish Connection
		if ($this->connect()) {
			return TRUE;
		} else {
			return FALSE;
		}

	}


	/**
	 * Connect to the MySQL database.
	 *
	 * @param string $host (optional) The database host server name.
	 * @param string $port (optional) The database connection port.
	 * @param string $user (optional) The database connection username.
	 * @param string $passwd (optional) The database connection password.
	 * @param string $connect (optional) The database connection password.
	 * @return boolean
	 */
	function connect($host = '', $port = '', $user = '', $passwd = '', $connect = '')
	{

		//--- Set Defaults
		$host = (empty($host)) ? $this->getDbHost() : $host;
		$port = (empty($port)) ? $this->getDbPort() : $port;
		$user = (empty($user)) ? $this->getDbUser() : $user;
		$passwd = (empty($passwd)) ? $this->getDbPassword() : $passwd;
		$connect = (empty($connect)) ? $this->getDbConnectionType() : $connect;

		//--- Establish Connection
		if ($connect == 'persistent') {
			if ($this->setDb(@mysql_pconnect("$host:$port", $user, $passwd))) {
				return TRUE;
			} else {
				$this->setError(mysql_error());
				return FALSE;
			}

		} else {
			if ($this->setDb(@mysql_connect("$host:$port", $user, $passwd))) {
				return TRUE;
			} else {
				$this->setError(mysql_error());
				return FALSE;
			}
		}

	}


	/**
	 * Disconnect from the MySQL database.
	 *
	 * @return boolean
	 */
	function disconnect()
	{
		if (@mysql_close($this->getDb())) {
			return TRUE;
		} else {
			$this->setError();
			return FALSE;
		}

	}


	/**
	 * EXPLAIN a database SELECT query.
	 *
	 * @return void
	 */
	function explainQuery()
	{
		if ($this->getExplainLog()) {
			mysql_select_db($this->getDbName(), $this->getDb());

			$query = 'EXPLAIN ' . $this->getQuery();

			if ($result = mysql_query($query, $this->getDb())) {
				$row = mysql_fetch_array($result, MYSQL_ASSOC);
				$explain = '';
				foreach ($row as $col => $val) {
					$explain .= $col . ': ' . $val . "\n";
				}
				$this->logExplain($this->getQuery());
				$this->logExplain($explain);
			} else {
				$this->logExplain($this->getQuery());
				$this->logExplain($this->getError());
			}
		}
	}


	/**
	 * Execute a database query.
	 *
	 * @param string $query The database query to execute.
	 * @return boolean
	 */
	function query($query)
	{
		if (!mysql_select_db($this->getDbName(), $this->getDb())) {
			$this->setError('DB');
			return FALSE;
		}

		//--- Reset
		$this->setError(FALSE);
		$this->setResult(FALSE);
		$this->setResultCnt(0);

		$this->setQuery($query);
		if ($result = mysql_query($query, $this->getDb())) {
			$this->setResult($result);
			//--- Result Count
			if (preg_match('/^\s*SELECT/i', $query)) {
				$this->setResultCnt(mysql_num_rows($this->getResult()));
			} else {
				$this->setResultCnt(mysql_affected_rows($this->getDb()));
			}
			$this->setFieldNames();

			if ($this->getExplainLog() && preg_match('/^\s*SELECT/i', $query)) {
				$this->explainQuery();
			}

			return TRUE;

		} else {
			// set error
			$this->setError();
			return FALSE;
		}
	}


	/**
	 * Process a database query using SafeSQL and then execute.
	 *
	 * @param string $query The database query to execute with placeholders.
	 * @param array $query_data Array of query placeholder data.
	 * @return boolean $stat
	 * @link http://www.phpinsider.com/php/code/SafeSQL/ SafeSQL query processer
	 */
	function safeQuery($query, $query_data)
	{
		$safe_query = '';
		if (get_magic_quotes_gpc()) {
			$this->stripArray($query_data);
		}

		if (!is_object($this->safeSqlObj)) {
			require_once($this->getSafeSqlLib());
			$this->safeSqlObj = new SafeSQL_MySQL();
		}

		if (is_object($this->safeSqlObj) && $this->getSafeSQL()) {
			$safe_query = $this->safeSqlObj->query($query, $query_data);
			$stat = $this->query($safe_query);
		} else {
			$stat = FALSE;
			$this->setSafeSQL(FALSE);
			$this->setError('Unable to create SafeSQL object.');
		}

		return $stat;
	}


	/**
	 * Apply stripslashes() to all elements of an array.
	 *
	 * This function will strip slashes from all elements of an array.
	 * This is primarily for use with the SafeSQL related safeQuery().
	 *
	 * @param array &$array Array reference of values to be stripped.
	 * @return void
	 */
	function stripArray(&$array)
	{
		for ($i = 0; $i < count($array); $i++) {
			if (is_array($array[$i])) {
				$this->stripArray($array[$i]);
			} else {
				$array[$i] = stripslashes($array[$i]);
			}
		}
	}


	/**
	 * Writes errors to a log file.
	 *
	 * @param string $error
	 * @return void
	 */
	function logError($error)
	{
		if ($this->getErrorLog()) {
			$fp = @fopen($this->getErrorLog(), 'a');
			@fputs($fp, '[' . date('Y-m-d H:i:s') . '] ');
			@fputs($fp, $error . "\n");
			@fclose($fp);
		}
	}


	/**
	 * Writes query EXPLAIN to a log file.
	 *
	 * @param string $data
	 * @return void
	 */
	function logExplain($data)
	{
		if ($this->getExplainLog()) {
			$fp = @fopen($this->getExplainLog(), 'a');
			@fputs($fp, '[' . date('Y-m-d H:i:s') . '] ');
			@fputs($fp, $data . "\n");
			@fclose($fp);
		}
	}



//---------------------//
//--- SET FUNCTIONS ---//
//---------------------//

	/**
	 * Sets the database object.
	 *
	 * @param object $a
	 * @return void
	 */
	function setDb($a)
	{
		$this->db = $a;
	}


	/**
	 * Sets the database connection type.
	 *
	 * @param string $a The database connection type, either 'standard' or 'persistent'
	 * @return void
	 */
	function setDbConnectionType($a)
	{
		$this->db_connect_type = $a;
	}


	/**
	 * Sets the database host server name.
	 *
	 * @param string $a
	 * @return void
	 */
	function setDbHost($a)
	{
		$this->db_host = $a;
	}


	/**
	 * Sets the database connection port.
	 *
	 * @param string $a
	 * @return void
	 */
	function setDbPort($a)
	{
		$this->db_port = $a;
	}


	/**
	 * Sets the active database name.
	 *
	 * @param string $a
	 * @return void
	 */
	function setDbName($a)
	{
		$this->db_name = $a;
	}


	/**
	 * Sets the database connection username.
	 *
	 * @param string $a
	 * @return void
	 */
	function setDbUser($a)
	{
		$this->db_user = $a;
	}


	/**
	 * Sets the database connection password.
	 *
	 * @param string $a
	 * @return void
	 */
	function setDbPassword($a)
	{
		$this->db_pw = $a;
	}


	/**
	 * Sets the database error text.
	 *
	 * @param string/boolean $error Text error message or FALSE to reset/clear.
	 * @return FALSE
	 */
	function setError($error = 'DB')
	{
		if ($error == 'DB') {
			$this->db_error = mysql_error($this->getDb());
		} else {
			$this->db_error = $error;
		}

		if ($error !== FALSE && $this->getErrorLog()) {
			$err = (empty($this->db_error)) ? print_r(debug_backtrace(), TRUE) : $this->db_error;
			$this->logError($err);
		}

		return FALSE;
	}


	/**
	 * Sets the executed database query.
	 *
	 * @param string $a
	 * @return bool TRUE
	 */
	function setQuery($a)
	{
		$this->query = $a;

		return TRUE;
	}


	/**
	 * Sets the database query results.
	 *
	 * @param resource|boolean $a
	 * @return bool TRUE
	 */
	function setResult($a)
	{
		$this->result = $a;

		return TRUE;
	}


	/**
	 * Sets the database result field (column) names.
	 *
	 * @return void
	 */
	function setFieldNames()
	{
		$this->field_names = array();
		if (is_resource($this->getResult())) {
			for ($i = 0; $i < mysql_num_fields($this->getResult()); $i++) {
				$this->field_names[] .= mysql_field_name($this->getResult(), $i);
			}
		}
	}


	/**
	 * Sets the number of rows affected or returned by a query.
	 *
	 * @param int $a
	 * @return void
	 */
	function setResultCnt($a)
	{
		$this->result_cnt = $a;
	}


	/**
	 * Enable error logging and sets location of error log.
	 *
	 * @param string $a A fully qualified path to the error log file.
	 * @return void
	 */
	function setErrorLog($a)
	{
		$this->error_log = $a;
	}


	/**
	 * Enable query EXPLAIN logging and sets location of EXPLAIN log.
	 *
	 * @param string $a A fully qualified path to the EXPLAIN log file.
	 * @return void
	 */
	function setExplainLog($a)
	{
		$this->explain_log = $a;
	}


	/**
	 * Enable/Disables debug mode.
	 *
	 * @param boolean $a
	 * @return void
	 */
	function setDebug($a)
	{
		$this->debug = $a;
	}


	/**
	 * Enable/Disables emailing of errors.
	 *
	 * @param boolean $a
	 * @return void
	 */
	function setDebugMail($a)
	{
		$this->debug_mail= $a;
	}


	/**
	 * Sets email 'To' address for error emailing.
	 *
	 * @param string $a
	 * @return void
	 */
	function setDebugEmailTo($a)
	{
		$this->debug_email_to = $a;
	}


	/**
	 * Sets email 'From' address for error emailing.
	 *
	 * @param string $a
	 * @return void
	 */
	function setDebugEmailFrom($a)
	{
		$this->debug_email_from = $a;
	}


	/**
	 * Flags if SafeSQL is available.
	 *
	 * @param boolean $a
	 * @return void
	 * @link http://www.phpinsider.com/php/code/SafeSQL/ SafeSQL query processer
	 */
	function setSafeSql($a)
	{
		$this->safe_sql = $a;
	}


	/**
	 * Sets the fully qualified path to SafeSQL class library.
	 *
	 * @param string $a
	 * @return void
	 * @link http://www.phpinsider.com/php/code/SafeSQL/ SafeSQL query processer
	 */
	function setSafeSqlLib($a)
	{
		$this->safe_sql_lib = $a;
		$this->setSafeSql(TRUE);
	}




//---------------------//
//--- GET FUNCTIONS ---//
//---------------------//

	/**
	 * Gets the database object.
	 *
	 * @return object $this->db
	 */
	function getDb()
	{
		return $this->db;
	}


	/**
	 * Gets the database connection type.
	 *
	 * @return string $this->db_connect_type
	 */
	function getDbConnectionType()
	{
		return $this->db_connect_type;
	}


	/**
	 * Gets the database host server name.
	 *
	 * @return string $this->db_host
	 */
	function getDbHost()
	{
		return $this->db_host;
	}


	/**
	 * Gets the database connection port.
	 *
	 * @return string $this->db_port
	 */
	function getDbPort()
	{
		return $this->db_port;
	}


	/**
	 * Gets the active database name.
	 *
	 * @return string $this->db_name
	 */
	function getDbName()
	{
		return $this->db_name;
	}


	/**
	 * Gets the database connection username.
	 *
	 * @return string $this->db_user
	 */
	function getDbUser()
	{
		return $this->db_user;
	}


	/**
	 * Gets the database connection password.
	 *
	 * @return string $this->db_pw
	 */
	function getDbPassword()
	{
		return $this->db_pw;
	}


	/**
	 * Gets the database error text and then clears it.  If enabled debug mail
	 * is also sent from this method.
	 *
	 * @see setDebugMail(), setDebugEmailTo(), setDebugEmailFrom()
	 * @return string $err
	 */
	function getError()
	{

		if ($this->getDebug()) {
			//--- Return raw error message
			$err = $this->db_error;
		} elseif (isset($this->db_error) && !empty($this->db_error)) {
			//--- Return nice generic failure message
			$err = 'A fatal database error has occurred, please try again later.';
		} else {
			//--- No Error
			unset($this->db_error);
			return NULL;
		}

		if ($this->getDebugMail()) {
			//--- Build Message
			$msg = 'The following database error occurred ' . date("l M j G:i:s T Y") . ':' . "\n\n";
			$msg .= 'Connection Information:' . "\n\n";
			$msg .= 'Host: ' . $this->getDbHost() . "\n";
			$msg .= 'Port: ' . $this->getDbPort() . "\n";
			$msg .= 'DB: ' . $this->getDbName() . "\n";
			$msg .= 'User: ' . $this->getDbUser() . "\n";
			$msg .= 'Connection: ' . $this->getDbConnectionType() . "\n\n";
			$msg .= 'URL: ' . $_SERVER['PHP_SELF'] . "\n";
			$msg .= 'File: ' . __FILE__ . "\n";
			$msg .= 'Line: ' . __LINE__ . "\n";
			if (version_compare(php_version(), '5.0.0', '>=')) {
				$msg .= 'Method: ' . __METHOD__ . "\n\n\n";
			} else {
				$msg .= 'Class: ' . __CLASS__ . "\n";
				$msg .= 'Function: ' . __FUNCTION__ . "\n\n\n";
			}
			$msg .= 'Error Message:' . "\n\n";
			$msg .= $this->db_error;

			//--- E-mail failure
			mail($this->getDebugEmailTo(), 'Database Error', $msg, 'From: ' . $this->getDebugEmailFrom() . "\r\n");
		}
		unset($this->db_error);

		return $err;
	}


	/**
	 * Gets the executed database query.
	 *
	 * @return string $this->query
	 */
	function getQuery()
	{
		return $this->query;
	}


	/**
	 * Gets the database query results.
	 *
	 * @return resource|boolean $this->result
	 */
	function getResult()
	{
		return $this->result;
	}


	/**
	 * Gets the result field (column) names.
	 *
	 * @return array $this->field_names
	 */
	function getFieldNames()
	{
		return $this->field_names;
	}


	/**
	 * Gets the number of rows affected or returned by a query.
	 *
	 * @return int $this->result_cnt
	 */
	function getResultCnt()
	{
		return $this->result_cnt;
	}


	/**
	 * Gets a list of available databases.
	 *
	 * @return array $db_list;
	 */
	function getDbList()
	{
		$db_list = array();
		$this->setResult(mysql_list_dbs($this->getDb()));
		while ($row = $this->getRow('NUM')) {
			$db_list[] = $row[0];
		}

		return $db_list;
	}


	/**
	 * Indicates if debug mode is enable or disabled.
	 *
	 * @return boolean $this->debug
	 */
	function getDebug()
	{
		return $this->debug;
	}


	/**
	 * Gets debug mail enable/disable state.
	 *
	 * @return boolean $this->debug_mail
	 */
	function getDebugMail()
	{
		return $this->debug_mail;
	}


	/**
	 * Gets email 'To' address for error emailing.
	 *
	 * @return string $this->debug_email_to
	 */
	function getDebugEmailTo()
	{
		return $this->debug_email_to;
	}


	/**
	 * Gets email 'From' address for error emailing.
	 *
	 * @return string $this->debug_email_from
	 */
	function getDebugEmailFrom()
	{
		return $this->debug_email_from;
	}


	/**
	 * A fully qualified path to the error log file.
	 *
	 * @return string $this->error_log
	 */
	function getErrorLog()
	{
		return $this->error_log;
	}


	/**
	 * A fully qualified path to the EXPLAIN log file.
	 *
	 * @return string $this->explain_log
	 */
	function getExplainLog()
	{
		return $this->explain_log;
	}


	/**
	 * Gets SafeSQL available state.
	 *
	 * @return string $this->safe_sql
	 * @link http://www.phpinsider.com/php/code/SafeSQL/ SafeSQL query processer
	 */
	function getSafeSql()
	{
		return $this->safe_sql;
	}


	/**
	 * Gets the fully qualified path to SafeSQL class library.
	 *
	 * @return string $this->safe_sql_lib
	 * @link http://www.phpinsider.com/php/code/SafeSQL/ SafeSQL query processer
	 */
	function getSafeSqlLib()
	{
		return $this->safe_sql_lib;
	}


	/**
	 * Gets a single row from the results of a query.
	 *
	 * @param string The result type desired, 'NUM' for numeric array,
	 * 'ASSOC' for associative array, or 'BOTH' for both.
	 * @return array|FALSE $this->row
	 * Typical Usage:
	 * <code>
	 * while ($row = $dbObj->getRow('ASSOC')) {
	 *	   print_r $row;
	 * }
	 * </code>
	 */
	function getRow($type = 'ASSOC')
	{
		//--- Set results type
		if ($type == 'ASSOC') {
			$type = MYSQL_ASSOC;
		} elseif ($type == 'NUM') {
			$type = MYSQL_NUM;
		} else {
			$type = MYSQL_BOTH;
		}

		if (is_resource($this->getResult())) {
			while ($this->row = mysql_fetch_array($this->getResult(), $type)) {
				return $this->row;
			}
		} else {
			return FALSE;
		}
	}
}
?>
