<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-2006  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require_once(LIB_PATH . 'PbDb_class.php');

	class PbBase
	{
		//--- Class Variables
		var $dbObj;

    	/**
    	 * Error message.
    	 * @var string
    	 */
		var $error;

		var $prefix;
		var $display_msg;
		var $id;
		var $spl_enabled;



		//-----------------------------------------------
		//--- CONSTRUCTOR
		//--- PURPOSE:
		//---   Create base blog class object
		//--- PARAMETERS: table prefix (optional)
		//--- RETURNS: nothing
		//-----------------------------------------------
		function PbBase($prefix = 'wl_')
		{

			//--- Set DB Table Prefix
			$this->setDbPrefix($prefix);

			//--- Instatiate DB Object
			$this->setDbObj();
			$safe_sql = (DB_CONNECT === 'mysqli') ? 'SafeSQLi.class.php' : 'SafeSQL.class.php';
			$this->dbObj->setSafeSqlLib(LIB_PATH . $safe_sql);
		}


		//-----------------------------------------------
		//-----------------------------------------------
		//--- SET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------

		//-----------------------------------------------
		//--- METHOD: setDbObj
		//--- PURPOSE: set database object
		//--- PARAMETERS: none
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setDbObj()
		{
			$this->dbObj = new PbDb();

			//--- SPL Support Available?
			$ver = (int) substr(phpversion(), 0, 1);
			$stat = ($ver >= 5) ? TRUE : FALSE;
			$this->setSpl($stat);
		}

		//-----------------------------------------------
		//--- METHOD: setDbPrefix
		//--- PURPOSE: set database prefix
		//--- PARAMETERS: prefix
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setDbPrefix($a)
		{
			$this->prefix = $a;
		}

		/**
		 * Sets the database error text.
		 *
		 * @param string/boolean $error Error message or FALSE to reset/clear.
		 */
		function setError($error)
		{
			$this->error = ($error === FALSE) ? NULL : $error;
		}

		//-----------------------------------------------
		//--- METHOD: setId
		//--- PURPOSE: set object ID
		//--- PARAMETERS: id
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setId($a)
		{
			$this->id = (int) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setDisplayMsg
		//--- PURPOSE: set display message
		//--- PARAMETERS: status message, pre-message, Footer message, message CSS class
		//--- RETURNS: XHTML display data
		//-----------------------------------------------
		function setDisplayMsg($msg, $pre_msg = 'FAIL_DISPLAY_MSG', $footer_msg = FALSE, $msg_class = 'pb_search_fail')
		{
			$msg = str_replace('  ', '&nbsp;&nbsp;', $msg);
			$this->display_msg = '<div class="pb_entry">';
			$this->display_msg .= '<p class="pb_body">' . $this->langObj->get($pre_msg) . ':<br /><br />';
			$this->display_msg .= '<span class="' . $msg_class . '">' . $msg . '</span>';
			if ($footer_msg) {
				$this->display_msg .= '<br /><br />' . $footer_msg . '</p>';
			}
			$this->display_msg .= '<br />-------------------------<br /><a href="' . BASE_URL . '" title="' . BLOG_NAME . ' - ' . $this->langObj->get('DISPLAY_RECENT_ENTRIES') . '">' . $this->langObj->get('DISPLAY_RECENT_ENTRIES') . '</a>';
			$this->display_msg .= '</div>';
		}

		/**
		 * Indicates that the PHP5 SPL is available
		 *
		 * @param boolean $a
		 *
		 */
		function setSpl($a)
		{
			$this->spl_enabled = $a;
		}



		//-----------------------------------------------
		//-----------------------------------------------
		//--- GET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: getDbPrefix
		//--- PURPOSE: get database prefix
		//--- PARAMETERS: none
		//--- RETURNS: database prefix
		//-----------------------------------------------
		function getDbPrefix()
		{
			return $this->prefix;
		}

		//-----------------------------------------------
		//--- METHOD: getDisplayMsg
		//--- PURPOSE: get display message output
		//--- PARAMETERS: none
		//--- RETURNS: XHTML display message data
		//-----------------------------------------------
		function getDisplayMsg()
		{
			return $this->display_msg;
		}

		/**
		 * Gets the error message and clears it.
		 *
		 * @return string $this->error
		 */
		function getError()
		{
 			$err = $this->error;
  			$this->setError(NULL);
			return $err;
		}

		//-----------------------------------------------
		//--- METHOD: getId
		//--- PURPOSE: get object ID
		//--- PARAMETERS: none
		//--- RETURNS: id
		//-----------------------------------------------
		function getId()
		{
			return (int) $this->id;
		}

		//-----------------------------------------------
		//--- METHOD: getUserIp
		//--- PURPOSE: Retrieve the users remote IP Address
		//--- PARAMETERS: none
		//--- RETURNS: IP Address
		//---          FALSE on failure
		//-----------------------------------------------
		function getUserIp()
		{
			$ip = FALSE;

			if (isset($_SERVER)) {
			   if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			       $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
			   } elseif(isset($_SERVER['HTTP_CLIENT_IP'])) {
			       $ip = $_SERVER['HTTP_CLIENT_IP'];
			   } elseif(isset($_SERVER['REMOTE_ADDR'])) {
			       $ip = $_SERVER['REMOTE_ADDR'];
			   }
			} else {
			   if (isset($_ENV['HTTP_X_FORWARDED_FOR']) && getenv('HTTP_X_FORWARDED_FOR')) {
			       $ip = getenv('HTTP_X_FORWARDED_FOR');
			   } elseif (isset($_ENV['HTTP_CLIENT_IP']) && getenv('HTTP_CLIENT_IP')) {
			       $ip = getenv('HTTP_CLIENT_IP');
			   } elseif (isset($_ENV['REMOTE_ADDR']) && getenv('REMOTE_ADDR')) {
			       $ip = getenv('REMOTE_ADDR');
			   }
			}


			//--- Multiple IP Address Handling
			$multi_ips = explode(',', $ip);
			if ($this->validateIp(trim($multi_ips[0])))
			{
				$ip = trim($multi_ips[0]);
			}

			return $ip;
		}

		//-----------------------------------------------
		//--- METHOD: getIpBlocked
		//--- PURPOSE: Check if remote IP is blocked
		//--- PARAMETERS: IP Address
		//--- RETURNS: TRUE on blocked
		//---          FALSE on not blocked
		//-----------------------------------------------
		function getIpBlocked($ip)
		{
			$stat = FALSE;

			if (defined('COMMENT_BLOCK_FILE') && file_exists(COMMENT_BLOCK_FILE)) {
				$list = file(COMMENT_BLOCK_FILE);
				foreach ($list as $line) {
					$line = trim($line);

					if ($line === $ip) {
						$stat = TRUE;
						break;
					}

					$ip_octets = explode('.', $ip);
					$octets = explode('.', $line);
					if ($octets[0] != '*' && $octets[0] != $ip_octets[0]) {
						 continue;
					} elseif ($octets[0] == '*') {
						$stat = TRUE;
						break;
					} elseif ($octets[1] != '*' && $octets[1] != $ip_octets[1]) {
						 continue;
					} elseif ($octets[1] == '*') {
						$stat = TRUE;
						break;
					} elseif ($octets[2] != '*' && $octets[2] != $ip_octets[2]) {
						 continue;
					} elseif ($octets[2] == '*') {
						$stat = TRUE;
						break;
					} elseif ($octets[3] != '*' && $octets[3] != $ip_octets[3]) {
						 continue;
					} elseif ($octets[3] == '*') {
						$stat = TRUE;
						break;
					}

				}
			}

			return $stat;
		}

		/**
		 * Indicates that the PHP5 SPL is available
		 *
		 * @return boolean
		 *
		 */
		function getSpl()
		{
			return $this->spl_enabled;
		}




		//-----------------------------------------------
		//-----------------------------------------------
		//--- OTHER METHODS -----------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: writeDb
		//--- PURPOSE: update the database
		//--- PARAMETERS: query
		//---					Safe query data (optional)
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function writeDb($query, $query_data = FALSE)
		{

			if (!is_object($this->dbObj)) {
				$this->setDbObj();
			}

			if (!is_array($query_data)) {
				$stat = $this->dbObj->query($query);
				if (!$stat) {
					echo $this->dbObj->getError();
				}
			} elseif (is_array($query_data)) {
				$stat = $this->dbObj->safeQuery($query, $query_data);
				if (!$stat) {
					echo $this->dbObj->getError();
				}
			} else {
				$stat =  FALSE;
			}

			return $stat;
		}

		/**
		 * Validate an IP address
		 *
		 * @param string $ip The IP address (xxx.xxx.xxx.xxx)
		 * @return boolean
		 */
		function validateIp($ip)
		{
			$octets = explode('.', $ip);
		    $stat = (empty($ip) || count($octets) !== 4 ||
				(!ctype_digit($octets[0]) || $octets[0] > 255 || $octets[0] < 0) ||
				(!ctype_digit($octets[1]) || $octets[1] > 255 || $octets[1] < 0) ||
				(!ctype_digit($octets[2]) || $octets[2] > 255 || $octets[2] < 0) ||
				(!ctype_digit($octets[3]) || $octets[3] > 255 || $octets[3] < 0)) ? FALSE : TRUE;

		    return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: buildLink
		//--- PURPOSE: build a hyperlink
		//--- PARAMETERS: link href
		//--- 				link title
		//--- 				link text
		//--- RETURNS: XHTML hyperlink
		//-----------------------------------------------
		function buildLink($href = BASE_URL, $title, $text)
		{
			return '<a href="' . $href . '" title="' . $title . '">' . $text . '</a>';
		}

	} //end of class
?>
