<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-2006  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require_once(LIB_PATH . 'PbCommentBase_class.php');

	class PbComment extends PbCommentBase
	{
		//--- Class Variables
		var $email;
		var $linebreak;
		var $display;
		var $comment;



		//-----------------------------------------------
		//--- CONSTRUCTOR
		//--- PURPOSE:
		//---   Create blog comment object
		//--- PARAMETERS: table prefix (optional)
		//---			  comment id (optional)
		//---			  RBL Services (optional)
		//--- RETURNS: nothing
		//-----------------------------------------------
		function PbComment($prefix = 'wl_', $id = '', $rbls = array())
		{

			$this->PbCommentBase($prefix, $rbls);

			if (is_int($id)) {
				$this->getCommentData($id);
			}

			$this->setRblServices($rbls);
			$this->setRblBlocked(FALSE);
		}


		//-----------------------------------------------
		//-----------------------------------------------
		//--- SET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: setEmail
		//--- PURPOSE: set coment poster email
		//--- PARAMETERS: poster email
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setEmail($a)
		{
			$this->email = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setComment
		//--- PURPOSE: set coment text
		//--- PARAMETERS: comment text
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setComment($a)
		{
			$a = strip_tags($a, COMMENT_TAGS);
			$this->comment = str_replace('  ', '&nbsp;&nbsp;', $a);

			//--- Convert New Lines -> <br />
			if ($this->getLineBreak()) {
				$text_array = explode("\r\n", $this->comment);
				for ($x = 0; $x < count($text_array); $x++) {
					if (strlen($text_array[$x]) == 0) {
						$text_array[$x] = '<br /><br />';
					}
				}
				$this->comment = implode('', $text_array);
			}
		}

		//-----------------------------------------------
		//--- METHOD: setLineBreak
		//--- PURPOSE: set linebreak flag
		//--- PARAMETERS: 1/0
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setLineBreak($a)
		{
			$this->linebreak = $a;
		}





		//-----------------------------------------------
		//-----------------------------------------------
		//--- GET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: getCommentData
		//--- PURPOSE: gets  Comment data
		//--- PARAMETERS: id (optional)
		//--- RETURNS: TRUE on success
		//---          FALSE of failure
		//-----------------------------------------------
		function getCommentData($id = '')
		{
			//--- Check for ID
			if (!isset($id)) {
				$id = $this->getId();
			}

			//--- Query
			$query = "SELECT
							cid,
							pid,
							eid,
							name,
							email,
							url,
							display,
							comment,
							trackback,
							timestamp,
							DATE_FORMAT(timestamp, '%s') AS timestamp_formatted,
							c.ip,
							r.whitelist
						 FROM
					 		{$this->prefix}comments AS c
					 	 LEFT JOIN
					 		{$this->prefix}rbl_blocked AS r
					 	 ON
					 	 	c.ip=r.ip
					 	 WHERE
					 	 	cid='%i'";
			$query_data = array(COMMENT_DATE_FORMAT, $id);

			//--- Issue Query
			if ($this->dbObj->safeQuery($query, $query_data)) {

				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {

					//--- Retrieve Results
					while ($this->dbObj->getRow()) {
						$this->setId($this->dbObj->row['cid']);
						$this->setPid($this->dbObj->row['pid']);
						$this->setEid($this->dbObj->row['eid']);
						$this->setName($this->dbObj->row['name']);
						$this->setEmail($this->dbObj->row['email']);
						$this->setUrl($this->dbObj->row['url']);
						$this->setDisplay($this->dbObj->row['display']);
						$this->setComment($this->dbObj->row['comment']);
						$this->setIsTrackback(($this->dbObj->row['trackback'] == '1' ? TRUE : FALSE));
						$this->setTimestamp($this->dbObj->row['timestamp']);
						$this->setTimestamp($this->dbObj->row['timestamp_formatted'], TRUE);
						$this->setIp($this->dbObj->row['ip']);
						$this->setIpWhitelisted($this->dbObj->row['whitelist']);
					}
					$this->setCommentExists(TRUE);

				} else {
					$this->setCommentExists(FALSE);
				}

				$stat = TRUE;

			} else {

				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}
				return $stat;
		}


		//-----------------------------------------------
		//--- METHOD: getEmail
		//--- PURPOSE: get poster email
		//--- PARAMETERS: none
		//--- RETURNS: poster email
		//-----------------------------------------------
		function getEmail()
		{
			return $this->email;
		}

		//-----------------------------------------------
		//--- METHOD: getComment
		//--- PURPOSE: get poster comment text
		//--- PARAMETERS: mode: HTML, DO_STUPEFY
		//--- RETURNS: poster comment
		//-----------------------------------------------
		function getComment($mode = 'HTML')
		{
			if ($mode === 'DO_STUPEFY') {
				return str_replace('&nbsp;&nbsp;', '  ', $this->comment); //strip
			} else {
				return $this->comment;
			}
		}

		//-----------------------------------------------
		//--- METHOD: getLineBreak
		//--- PURPOSE: get linebreak flag
		//--- PARAMETERS: none
		//--- RETURNS: linebreak flag
		//-----------------------------------------------
		function getLineBreak()
		{
			return (int) $this->linebreak;
		}




		//-----------------------------------------------
		//-----------------------------------------------
		//--- OTHER METHODS -----------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: updateComment
		//--- PURPOSE: update comment in the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function updateComment()
		{
			if (!$this->getId() > 0) {
				return FALSE;
			}

			$stat = FALSE;
			if (USE_COMMENTS) {
				$query = "UPDATE
						 		{$this->prefix}comments
							SET
								name='%s',
								email='%s',
								url='%s',
								comment='%s',
								display='%i',
								timestamp=timestamp
							WHERE
								cid='%i'";

				$query_data = array(
									$this->getName(),
									$this->getEmail(),
									$this->getUrl(),
									$this->getComment(),
									(string) $this->getDisplay(),
									$this->getId()
								  );

				$this->setCommentExists(TRUE);
				$stat = $this->dbObj->safeQuery($query, $query_data);
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: insertComment
		//--- PURPOSE: insert comment in the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function insertComment()
		{
			$stat = FALSE;
			$this->setDisplay('1');
			$save = TRUE;
			$use_rbl = (defined('USE_RBL') && USE_RBL) ? TRUE : FALSE;
			$rbl_referer_check = (defined('RBL_REFERER_CHECK') && RBL_REFERER_CHECK) ? TRUE : FALSE;
			$rbl_block_keyword = (defined('RBL_BLOCK_KEYWORD') && RBL_BLOCK_KEYWORD) ? TRUE : FALSE;
			$rbl_url_check = (defined('RBL_URL_CHECK') && RBL_URL_CHECK) ? TRUE : FALSE;
			$rbl_text_check = (defined('RBL_TEXT_CHECK') && RBL_TEXT_CHECK) ? TRUE : FALSE;
			$rbl_save_blocked = (defined('RBL_SAVE_BLOCKED') && RBL_SAVE_BLOCKED) ? TRUE : FALSE;

			$this->setIsTrackback(FALSE);
			$ip_blocked = $this->getIpBlocked($this->getUserIp());
			$rbl_blocked = FALSE;
			$url_blocked = FALSE;
			$text_blocked = FALSE;
			if ($use_rbl) {
				$this->setRblObj(DB_PREFIX, $this->getRblServices(), $rbl_referer_check, $rbl_block_keyword);
			}
			if ($use_rbl && !$ip_blocked) {
				$rbl_referer = (isset($_SERVER['HTTP_REFERER'])) ? strip_tags($_SERVER['HTTP_REFERER']) : NULL;
				$rbl_blocked = $this->rblObj->checkIp($this->getUserIp(), $rbl_referer);
				$this->setRblBlocked($rbl_blocked);

				//--- Commentor's URL
				if ($rbl_url_check && !$rbl_blocked) {
					$url_blocked = $this->rblObj->checkForKeywords($this->getUrl());
					$this->setRblBlocked($url_blocked);
				}

				//--- Comment Text Check
				if ($rbl_text_check && !$url_blocked && !$rbl_blocked) {
					$text_blocked = $this->rblObj->checkForKeywords($this->getComment());
					$this->setRblBlocked($text_blocked);
				}
			}
			if ($use_rbl) {
				$this->setDisplay(($rbl_blocked || $url_blocked || $text_blocked) ? '0' : '1');
				$save = $rbl_save_blocked;
			}

			//--- Force block/no display
			if ($use_rbl && $this->rblObj->getForceBlock()) {
				$this->setDisplay(($this->rblObj->getForceBlock()) ? '0' : '1');
				$save = FALSE;
			}

			if (USE_COMMENTS && $this->commentsEnabled($this->getEid()) &&	!$ip_blocked && $save) {
				$stat = $this->insertData();
				if ($stat) {
					if (DB_CONNECT === 'mysqli') {
						$this->setId(mysqli_insert_id($this->dbObj->getDb()));
					} else {
						$this->setId(mysql_insert_id());
					}
				}
				if ($stat && $this->getPid() == 0) {
					if (defined('COMMENT_MAIL_RESTRICT') && COMMENT_MAIL_RESTRICT) {
						if ($this->getDisplay() === '1') {
							$this->emailNotify($this->getName());
						}
					} else {
						$this->emailNotify($this->getName()); //comment
					}
				} elseif ($stat && $this->getDisplay() === '1') {
					// Reply notify only if displayed (not spam)
					$this->emailNotify($this->getName(), FALSE);
				}
			}

			return $stat;
		}

		/**
		 * Delete comments from the database
		 *
		 * @param array Optional array of comments ID's
		 * @return boolean
		 */
		function deleteComment($cids = FALSE)
		{
			if (!$this->getId() > 0 && !$cids) {
				return FALSE;
			}

			if (!is_array($cids)) {
				$cids = array();
				$cids = $this->findDeletes($this->getId());
				$cids[] = $this->getId();
			}
			$query_data = array($cids, $cids);
			$query = "DELETE FROM
					 		{$this->prefix}comments
						WHERE
							cid IN (%c)
						OR
							pid IN (%c)";

			$this->setCommentExists(FALSE);

			return $this->writeDb($query, $query_data);
		}

		/**
		 * Enable comments display in the database
		 *
		 * @param array Optional array of comments ID's
		 * @return boolean
		 */
		function displayComment($cids = FALSE)
		{
			if (!$this->getId() > 0 && !$cids) {
				return FALSE;
			}

			if (!is_array($cids)) {
				$cids = array();
				$cids[] = $this->getId();
			}
			$query_data = array($cids);
			$query = "UPDATE
					 		{$this->prefix}comments
						SET
							display='1',
							timestamp=timestamp
						WHERE
							cid IN (%c)";

			$this->setCommentExists(TRUE);

			return $this->writeDb($query, $query_data);
		}

		//-----------------------------------------------
		//--- METHOD: findDeletes
		//--- PURPOSE: find all decedents of comment to delete
		//--- PARAMETERS: comment id
		//--- RETURNS: array of comment id's
		//-----------------------------------------------
		function findDeletes($cid)
		{
			$cids = array();
			$query = "SELECT
							cid
						 FROM
					 		{$this->prefix}comments
						 WHERE
							pid='%i'";
			$query_data = array($cid);

			if ($this->dbObj->safeQuery($query, $query_data)) {
				if ($this->dbObj->getResultCnt() > 0) {
					while ($this->dbObj->getRow()) {
						$cids[] = $this->dbObj->row['cid'];
					}
					$tmp_cids = $cids; // necessary, because we're modifying $cids
					//--- SPL Support
					if ($this->getSpl()) {
						foreach (new ArrayObject($tmp_cids) as $tmp_cid) {
							$cids = array_merge($cids, $this->findDeletes($tmp_cid));
						}
					} else {
						foreach ($tmp_cids as $tmp_cid) {
							$cids = array_merge($cids, $this->findDeletes($tmp_cid));
						}
					}
				}
			} else {
				echo $this->dbObj->getError();
			}

			return $cids;
		}

	} //end of class
?>