<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-2006  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require_once(LIB_PATH . 'PbBase_class.php');
	require_once(LIB_PATH . 'PbComment_class.php');
	@require_once(LIB_PATH . 'XmlRpc_class.php'); //suppress until updated or replaced.  :(
	require_once(LIB_PATH . 'Localization_class.php');

	class PbEntry extends PbBase
	{
		//--- Class Variables
		var $topic_id;
		var $title;
		var $created;
		var $updated;
		var $linebreak;
		var $post_by;
		var $uid;
		var $display;
		var $allow_comments;
		var $body;
		var $more;
		var $entry_exists;

		/**
		 * Entry status message (ping, trackback results)
		 * @var
		 */
		var $entry_msg;

		var $more_exists;
		var $time_adjust_value;
		var $time_adjust_direction;
		var $time_adjust_create;
		var $time_adjust_update;
		var $comment_cnt;
		var $commentObj;
		var $ping;
		var $rbl_services;

		/**
		 * Trackback URL for an entry.
		 * @var
		 */
		var $tb_url;

		/**
		 * Trackback should be sent.
		 * @var
		 */
		var $tb_send;



		//-----------------------------------------------
		//--- CONSTRUCTOR
		//--- PURPOSE:
		//---   Create blog entry object
		//--- PARAMETERS: table prefix (optional)
		//---					entry id (optional)
		//--- RETURNS: nothing
		//-----------------------------------------------
		function PbEntry($prefix = 'wl_', $id = '')
		{
			$this->PbBase($prefix);
			$this->langObj = new Localization(I18N_LANG, './lang/');
			$this->comments = array();

			if (is_int($id)) {
				//--- Retrieve Entry Data
				$this->getEntryData($id);
			}
		}


		//-----------------------------------------------
		//-----------------------------------------------
		//--- SET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		/**
		 * Set entry status message (ping, trackback results)
		 *
		 * @protected
		 * @param string $a
		 */
		function setEntryMsg($a)
		{
			$this->entry_msg = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setTopicId
		//--- PURPOSE: sets topic ID
		//--- PARAMETERS: topic id
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setTopicId($a)
		{
			$this->topic_id = (int) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setTitle
		//--- PURPOSE: set entry title
		//--- PARAMETERS: entry title
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setTitle($a)
		{
			$this->title = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setCreated
		//--- PURPOSE: set entry creation time
		//--- PARAMETERS: entry creation time
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setCreated($a)
		{
			$this->created = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setUpdated
		//--- PURPOSE: set entry update time
		//--- PARAMETERS: entry update time
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setUpdated($a)
		{
			$this->updated = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPostBy
		//--- PURPOSE: set entry posted by
		//--- PARAMETERS: posted by name
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPostBy($a)
		{
			$this->post_by = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setUid
		//--- PURPOSE: set posted by user ID
		//--- PARAMETERS: user Id
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setUid($a)
		{
			$this->uid = (int) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setLineBreak
		//--- PURPOSE: set entry linebreak flag
		//--- PARAMETERS: 1/0
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setLineBreak($a)
		{
			$this->linebreak = (int) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setAllowComments
		//--- PURPOSE: set allow comments flag
		//--- PARAMETERS: 1/0
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setAllowComments($a)
		{
			$this->allow_comments = (int) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPing
		//--- PURPOSE: set ping flag
		//--- PARAMETERS: boolean
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPing($a)
		{
			$this->ping = (bool) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setDisplay
		//--- PURPOSE: set entry display flag
		//--- PARAMETERS: 1/0
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setDisplay($a)
		{
			$this->display = (int) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setBody
		//--- PURPOSE: set entry body text
		//--- PARAMETERS: entry body text
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setBody($a)
		{
			$this->body = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setMore
		//--- PURPOSE: set entry more text
		//--- PARAMETERS: entry more text
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setMore($a)
		{
			$this->more = $a;
			if (strlen($this->more) > 0) {
				$this->setMoreExists(TRUE);
			}
		}

		//-----------------------------------------------
		//--- METHOD: setEntryExists
		//--- PURPOSE: flag value indicates if entry
		//---				data exists for the given ID
		//--- PARAMETERS: TRUE/FALSE
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setEntryExists($a)
		{
			$this->entry_exists = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setMoreExists
		//--- PURPOSE: flag value indicates if more
		//---				data exists for the given ID
		//--- PARAMETERS: TRUE/FALSE
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setMoreExists($a)
		{
			$this->more_exists = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setAdjustTime
		//--- PURPOSE: sets create & update timestamps
		//--- PARAMETERS: update timestamp flag
		//---					create timestamp flag
		//---					time adjust direction (+/-)
		//---					adjust hours
		//---					adjust minutes
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setAdjustTime($update_timestamp, $create_timestamp, $time_adjust_direction, $time_adj_hour, $time_adj_min)
		{
			//--- Update Timestamp?
			$this->time_adjust_create = (int) $create_timestamp;
			$this->time_adjust_update = (int) $update_timestamp;

			$this->time_adjust_direction = $time_adjust_direction;
			$this->time_adjust_value = 0;
			if ($time_adj_hour > 0 || $time_adj_min > 0) {
				$this->time_adjust_value = (int) (($time_adj_hour * 3600) + ($time_adj_min * 60));
			}
		}

		//-----------------------------------------------
		//--- METHOD: setCommentCnt
		//--- PURPOSE: sets the number of comments that
		//---				are associated with this entry
		//--- PARAMETERS: id (optional)
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setCommentCnt($id = '')
		{
			//--- Check for ID
			if (empty($id)) {
				$id = $this->getId();
			}

			$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? " AND display='1'" : '';
			$query = "SELECT
							COUNT(*) AS count
						 FROM
					 		{$this->prefix}comments
					 	 WHERE
					 	 	eid='%i'
					 	 $display_flag";
			$query_data = array($id);
			if ($this->dbObj->safeQuery($query, $query_data)) {
				$this->dbObj->getRow();
				$this->comment_cnt = $this->dbObj->row['count'];
			} else {
				echo $this->dbObj->getError();
			}
		}

		/**
		 * Set a trackback URL for an entry.
		 *
		 * @param string $a
		 */
		function setTbUrl($a)
		{
			$this->tb_url = $a;
		}

		/**
		 * Set a whether a trackback for an entry should be sent.
		 *
		 * @param boolean $a
		 */
		function setTbSend($a)
		{
			$this->tb_send = $a;
		}

		/**
		 * RBL services to use for spam checking.
		 *
		 * @param array $a
		 */
		function setRblServices($a)
		{
			$this->rbl_services = $a;
		}





		//-----------------------------------------------
		//-----------------------------------------------
		//--- GET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: getEntryData
		//--- PURPOSE: gets  Entry data
		//--- PARAMETERS: id (optional)
		//--- RETURNS: TRUE on success
		//---          FALSE of failure
		//-----------------------------------------------
		function getEntryData($id = '')
		{
			//--- Check for ID
			if (empty($id)) {
				$id = $this->getId();
			}

			//--- Query
			$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? " AND display='1'" : '';
			$post_dated = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? ' AND updated<=UNIX_TIMESTAMP()' : '';
			$query = "SELECT
							eid,
							title,
							tid,
							created,
							updated,
							post_by,
							uid,
							display,
							comments,
							body,
							more,
							tb_url
						 FROM
					 		{$this->prefix}entries
					 	 WHERE
					 	 	eid='%i'
					 	 $post_dated
					 	 $display_flag";
			$query_data = array($id);

			if ($this->dbObj->safeQuery($query, $query_data)) {
				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {
					//--- Retrieve Results
					while ($this->dbObj->getRow()) {
						$this->setId($this->dbObj->row['eid']);
						$this->setTitle($this->dbObj->row['title']);
						$this->setTopicId($this->dbObj->row['tid']);
						$this->setCreated($this->dbObj->row['created']);
						$this->setUpdated($this->dbObj->row['updated']);
						$this->setPostBy($this->dbObj->row['post_by']);
						$this->setUid($this->dbObj->row['uid']);
						$this->setDisplay($this->dbObj->row['display']);
						$this->setAllowComments($this->dbObj->row['comments']);
						$this->setBody($this->dbObj->row['body']);
						$this->setMore($this->dbObj->row['more']);
						$this->setTbUrl($this->dbObj->row['tb_url']);
					}
					$this->setEntryExists(TRUE);

				} else {
					$this->setEntryExists(FALSE);
				}

				$stat = TRUE;

			} else {

				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}
				return $stat;
		}


		/**
		 * Get entry status message (ping, trackback results)
		 *
		 * @return string
		 */
		function getEntryMsg()
		{
			return $this->entry_msg;
		}

		//-----------------------------------------------
		//--- METHOD: getTopicId
		//--- PURPOSE: get topic ID
		//--- PARAMETERS: none
		//--- RETURNS: topic id
		//-----------------------------------------------
		function getTopicId()
		{
			return (int) $this->topic_id;
		}

		//-----------------------------------------------
		//--- METHOD: getTitle
		//--- PURPOSE: get entry title
		//--- PARAMETERS: none
		//--- RETURNS: entry title
		//-----------------------------------------------
		function getTitle()
		{
			return $this->title;
		}

		//-----------------------------------------------
		//--- METHOD: getCreated
		//--- PURPOSE: get entry creation time
		//--- PARAMETERS: TRUE/FALSE formatted (optional)
		//--- RETURNS: entry creation time
		//-----------------------------------------------
		function getCreated($formatted = FALSE)
		{
			if ($formatted) {
				$date = (defined('I18N_DATE_FORMAT')) ? strftime(I18N_DATE_FORMAT, $this->created + TIME_ADJUST) :	date(DATE_FORMAT, $this->created + TIME_ADJUST);
			} else {
				$date = $this->created;
			}
			return $date;
		}

		//-----------------------------------------------
		//--- METHOD: getUpdated
		//--- PURPOSE: get entry update time
		//--- PARAMETERS: TRUE/FALSE formatted (optional)
		//--- RETURNS: entry update time
		//-----------------------------------------------
		function getUpdated($formatted = FALSE)
		{
			if ($formatted) {
				$date = (defined('I18N_DATE_FORMAT')) ? strftime(I18N_DATE_FORMAT, $this->updated + TIME_ADJUST) :	date(DATE_FORMAT, $this->updated + TIME_ADJUST);
			} else {
				$date = $this->updated;
			}
			return $date;
		}

		//-----------------------------------------------
		//--- METHOD: getPostBy
		//--- PURPOSE: get posted by name
		//--- PARAMETERS: none
		//--- RETURNS: entry posted by
		//-----------------------------------------------
		function getPostBy()
		{
			return $this->post_by;
		}

		//-----------------------------------------------
		//--- METHOD: getUid
		//--- PURPOSE: get posted by user ID
		//--- PARAMETERS: none
		//--- RETURNS: user ID
		//-----------------------------------------------
		function getUid()
		{
			return (int) $this->uid;
		}

		//-----------------------------------------------
		//--- METHOD: getLineBreak
		//--- PURPOSE: get linebreak flag
		//--- PARAMETERS: none
		//--- RETURNS: entry linebreak flag
		//-----------------------------------------------
		function getLineBreak()
		{
			return (int) $this->linebreak;
		}

		//-----------------------------------------------
		//--- METHOD: getDisplay
		//--- PURPOSE: get display flag
		//--- PARAMETERS: none
		//--- RETURNS: entry display flag
		//-----------------------------------------------
		function getDisplay()
		{
			return (int) $this->display;
		}

		//-----------------------------------------------
		//--- METHOD: getAllowComments
		//--- PURPOSE: get allow comments flag
		//--- PARAMETERS: none
		//--- RETURNS: entry allow comments flag
		//-----------------------------------------------
		function getAllowComments()
		{
			return (int) $this->allow_comments;
		}

		//-----------------------------------------------
		//--- METHOD: getPing
		//--- PURPOSE: get ping flag
		//--- PARAMETERS: none
		//--- RETURNS: entry ping, Ping-O-Matic! flag
		//-----------------------------------------------
		function getPing()
		{
			return $this->ping;
		}

		//-----------------------------------------------
		//--- METHOD: getBody
		//--- PURPOSE: get entry body
		//--- PARAMETERS: mode: HTML, DO_STUPEFY, GIGGLE text
		//--- RETURNS: entry body text
		//-----------------------------------------------
		function getBody($mode = 'HTML')
		{
			if ($mode === 'DO_STUPEFY') {
				return $this->entryProc($this->body, 'DO_STUPEFY');
			} elseif ($mode === 'GIGGLE') {
				return giggle_autolink($this->body, -1, 1);
			} else {
				return $this->body;
			}
		}

		//-----------------------------------------------
		//--- METHOD: getMore
		//--- PURPOSE: get entry more
		//--- PARAMETERS: mode: HTML, DO_STUPEFY, GIGGLE text
		//--- RETURNS: entry more text
		//-----------------------------------------------
		function getMore($mode = 'HTML')
		{
			if ($mode === 'DO_STUPEFY') {
				return $this->entryProc($this->more, 'DO_STUPEFY');
			} elseif ($mode === 'GIGGLE') {
				return giggle_autolink($this->more, -1, 1);
			} else {
				return $this->more;
			}
		}

		//-----------------------------------------------
		//--- METHOD: getEntryExists
		//--- PURPOSE: flag value indicates if entry
		//---				data exists for the given ID
		//--- PARAMETERS: none
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function getEntryExists()
		{
			return $this->entry_exists;
		}

		//-----------------------------------------------
		//--- METHOD: getMoreExists
		//--- PURPOSE: flag value indicates if more
		//---				data exists for the given ID
		//--- PARAMETERS: none
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function getMoreExists()
		{
			return $this->more_exists;
		}

		//-----------------------------------------------
		//--- METHOD: getTimeValue
		//--- PURPOSE: get entry time adjustment
		//--- PARAMETERS: none
		//--- RETURNS: entry more text
		//-----------------------------------------------
		function getTimeValue()
		{
			return $this->more;
		}

		//-----------------------------------------------
		//--- METHOD: getCommentCnt
		//--- PURPOSE: gets the number of comments for
		//---				this entry.
		//--- PARAMETERS: none
		//--- RETURNS: entry comment count
		//-----------------------------------------------
		function getCommentCnt()
		{
			if (!isset($this->comment_cnt)) {
				$this->setCommentCnt();
			}

			return $this->comment_cnt;
		}

		//-----------------------------------------------
		//--- METHOD: getComment
		//--- PURPOSE: gets a comments object.
		//--- PARAMETERS: comment id
		//--- RETURNS: comment objects
		//-----------------------------------------------
		function getComment($cid)
		{
			$commentObj = new PbComment($this->getDbPrefix(), $cid);
			return $commentObj;
		}

		//-----------------------------------------------
		//--- METHOD: getComments
		//--- PURPOSE: gets the comments for an entry or comment.
		//--- PARAMETERS: entry id
		//---					parent id (optional)
		//--- RETURNS: array of comment objects
		//-----------------------------------------------
		function getComments($eid = FALSE, $pid = 0)
		{
			$sub_comment = ($eid != $pid) ? TRUE : FALSE;
			$eid = (!$eid) ? (int) $this->getId() : $eid;
			$comments = array();
			$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? " AND display='1'" : '';

			$query = "SELECT
							cid
						 FROM
						 	{$this->prefix}comments
						 WHERE
						 	eid='%i'
						 AND
						 	pid='%i'
						 $display_flag
						 ORDER BY timestamp ASC";
			$query_data = array($eid, $pid);

			if ($this->dbObj->safeQuery($query, $query_data)) {
				$i = 0;
				while ($this->dbObj->getRow()) {
					$cid = (int) $this->dbObj->row['cid'];
					$comments[$i] = $this->getComment($cid);
					$i++;
				}
			} else {
				echo $this->dbObj->getError();
			}

			if (!$sub_comment) {
				$this->comments = $comments;
			}

			return $comments;
		}

		/**
		 * Get a trackback URL for an entry.
		 *
		 * @return string
		 */
		function getTbUrl()
		{
			return $this->tb_url;
		}

		/**
		 * Get a whether a trackback for an entry should be sent.
		 *
		 * @return boolean
		 */
		function getTbSend()
		{
			return $this->tb_send;
		}

		/**
		 * Get RBL services to check for spam.
		 *
		 * @return array
		 */
		function getRblServices()
		{
			return $this->rbl_services;
		}



		//-----------------------------------------------
		//-----------------------------------------------
		//--- OTHER METHODS -----------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: deleteEntry
		//--- PURPOSE: delete entry data from the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function deleteEntry()
		{

			if (!($this->getId() >= 0)) {
				return FALSE;
			}

			$query = "DELETE FROM
							{$this->prefix}entries
						 WHERE
						 	eid='%i'";
			$query_data = array($this->getId());

			$stat = $this->writeDb($query, $query_data);
			$this->setEntryExists(!$stat);

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: updateEntry
		//--- PURPOSE: update entry data in the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function updateEntry()
		{
			if (!($this->getId() >= 0)) {
				return FALSE;
			}

			//--- Entry Processing
			$this->setBody($this->entryProc($this->getBody()));
			$this->setMore($this->entryProc($this->getMore()));

			//--- Create Timestamp
			$created = ($this->time_adjust_create) ? 'created=UNIX_TIMESTAMP()' : '';
			$created .= ($this->time_adjust_create) ? $this->time_adjust_direction . $this->time_adjust_value . ',' : '';
			$post_dated = FALSE;
			if ($this->time_adjust_direction == '+' && $this->time_adjust_value > 0) {
				$post_dated = TRUE;
			}

			//--- Update Timestamp
			if ($this->time_adjust_create || $this->time_adjust_update) {
				$updated = 'UNIX_TIMESTAMP()' . $this->time_adjust_direction . $this->time_adjust_value;
				if ($this->time_adjust_direction == '+' && $this->time_adjust_value > 0) {
					$post_dated = TRUE;
				}
			} else {
				$updated = 'updated';
			}

			$query = "UPDATE
							{$this->prefix}entries
						 SET
						 	title='%s',
						 	tid='%i',
						 	$created
						 	updated=$updated,
						 	display='%i',
						 	comments='%i',
						 	body='%s',
						 	more='%s',
						 	tb_url='%s'
						 WHERE
						 	eid='%i'";
			$query_data = array($this->getTitle(), $this->getTopicId(), $this->getDisplay(),
								$this->getAllowComments(), $this->getBody(), $this->getMore(),
								$this->getTbUrl(), $this->getId()
							   );

			$entry_msg = '';
			$stat = $this->writeDb($query, $query_data);
			if ($stat) {
				$this->setEntryExists($stat);
				if ((defined('USE_PINGER') && USE_PINGER) && $this->getDisplay() && !$post_dated && $this->getPing()) {
					$p_res = $this->pinger();
					$entry_msg .= 'Ping-O-Matic!:&nbsp;&nbsp;';
					$entry_msg .= ($p_res === FALSE) ? $this->langObj->get('DISPLAY_MSG_FAIL') : $this->langObj->get('DISPLAY_MSG_SUCCESS');
				}
				if ((defined('USE_TRACKBACK') && USE_TRACKBACK) && $stat && $this->getDisplay() && $this->getTbUrl() && $this->getTbSend()) {
					$tb_res = $this->sendTrackback();
					if (!empty($entry_msg)) $entry_msg .= '<br />';
					$entry_msg .= $this->langObj->get('TRACKBACK') . ':&nbsp;&nbsp;';
					$entry_msg .= (!empty($tb_res)) ? $tb_res : $this->langObj->get('DISPLAY_MSG_SUCCESS');
				}
			} else {
				$entry_msg .= $this->langObj->get('DISPLAY_MSG_FAIL');
				$entry_msg .= '<br /><br />' . $this->dbObj->getError();
			}
			if (empty($entry_msg)) {
				$entry_msg = $this->langObj->get('DISPLAY_MSG_SUCCESS');
			}
			$this->setEntryMsg($entry_msg);

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: insertEntry
		//--- PURPOSE: insert entry data in the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function insertEntry($pop3 = FALSE)
		{

			if (!($this->getId() >= 0)) {
				return FALSE;
			}

			if (!$pop3) {
				$this->setPostBy($_SESSION['post_by']);
				$this->setUid($_SESSION['uid']);
			}

			//--- Entry Processing
			$this->setBody($this->entryProc($this->getBody()));
			$this->setMore($this->entryProc($this->getMore()));

			//--- Create Timestamp
			$created = 'UNIX_TIMESTAMP()'. $this->time_adjust_direction . $this->time_adjust_value;
			$post_dated = FALSE;
			if ($this->time_adjust_direction == '+' && $this->time_adjust_value > 0) {
				$post_dated = TRUE;
			}

			$query = "INSERT INTO
							{$this->prefix}entries
							(
								title,
								tid,
								created,
								updated,
								post_by,
								uid,
								display,
								comments,
								body,
								more,
								tb_url
							)
						VALUES
							(
								'%s',
								'%i',
								$created,
								$created,
								'%s',
								'%i',
								'%i',
								'%i',
								'%s',
								'%s',
								'%s'
							)";
			$query_data = array($this->getTitle(), $this->getTopicId(), $this->getPostBy(),
									  $this->getUid(), $this->getDisplay(), $this->getAllowComments(),
									  $this->getBody(), $this->getMore(), $this->getTbUrl()
							  );

			$entry_msg = '';
			$stat = $this->writeDb($query, $query_data);
			if ($stat) {
				if (DB_CONNECT === 'mysqli') {
					$this->setId(mysqli_insert_id($this->dbObj->getDb()));
				} else {
					$this->setId(mysql_insert_id());
				}
				$this->setEntryExists($stat);
				if ((defined('USE_PINGER') && USE_PINGER) && $this->getDisplay() && !$post_dated  && $this->getPing()) {
					$p_res = $this->pinger();
					$entry_msg .= 'Ping-O-Matic!:&nbsp;&nbsp;';
					$entry_msg .= ($p_res === FALSE) ? $this->langObj->get('DISPLAY_MSG_FAIL') : $this->langObj->get('DISPLAY_MSG_SUCCESS');
				}
				if ((defined('USE_TRACKBACK') && USE_TRACKBACK) && $stat && $this->getDisplay() && $this->getTbUrl()) {
					$tb_res = $this->sendTrackback();
					if (!empty($entry_msg)) $entry_msg .= '<br />';
					$entry_msg .= $this->langObj->get('TRACKBACK') . ':&nbsp;&nbsp;';
					$entry_msg .= (!empty($tb_res)) ? $tb_res : $this->langObj->get('DISPLAY_MSG_SUCCESS');
				}
			} else {
				$entry_msg .= $this->langObj->get('DISPLAY_MSG_FAIL');
				$entry_msg .= '<br /><br />' . $this->dbObj->getError();
			}
			if (empty($entry_msg)) {
				$entry_msg = $this->langObj->get('DISPLAY_MSG_SUCCESS');
			}
			$this->setEntryMsg($entry_msg);

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: entryProc
		//--- PURPOSE: entry text manipulation processing
		//--- PARAMETERS: text (body or more)
		//---					mode, 'DO_STUPIFY'
		//--- RETURNS: formatted/processed text
		//-----------------------------------------------
		function entryProc($text, $mode = 'PROC')
		{
			if (strlen($text) > 0) {
				//--- Convert Double New Lines -> <br /><br />
				if ($this->getLineBreak()) {
					$text_array = explode("\r\n", $text);
					for ($x = 0; $x < count($text_array); $x++) {
						if (strlen($text_array[$x]) == 0) {
							$text_array[$x] = '<br /><br />';
						}
					}
					$text = implode("\n", $text_array);
				}

				//--- Auto-Link
				$pattern = '/\[([E|C|T]):(\d+)(:([\w\s\.?,\'"~!@#$%^&*\(\)_+\-:;|]+))*\]/';
				$matches = array();
				preg_match_all($pattern, $text, $matches, PREG_SET_ORDER);
				if (isset($matches[0]) && count($matches[0]) > 0) {
					//--- SPL Support
					if ($this->getSpl()) {
						foreach (new ArrayObject($matches) as $key => $sub_array) {
							$link_text = (isset($sub_array[4])) ? $sub_array[4] : 'ENTRY_NAME';
							$link = $this->autoLink($sub_array[1], $sub_array[2], $link_text);
							$text = str_replace($sub_array[0], $link, $text);
						}
					} else {
						foreach ($matches as $key => $sub_array) {
							$link_text = (isset($sub_array[4])) ? $sub_array[4] : 'ENTRY_NAME';
							$link = $this->autoLink($sub_array[1], $sub_array[2], $link_text);
							$text = str_replace($sub_array[0], $link, $text);
						}
					}
				}

				//--- SmartyPants-PHP
				// REF: http://monauraljerk.org/smartypants-php/
				// Requires PHP V4.3.3
				if (defined('USE_SMARTYPANTS') && USE_SMARTYPANTS) {
					require_once SMARTYPANTS_LIB_PATH . 'SmartyPants-PHP.inc';

					//--- Strip/Add Changes?
					if ($mode === 'DO_STUPEFY') {
						$text = SmartyPants($text, DO_STUPEFY);
					} else {
						$text = SmartyPants($text);
					}
				}

				//--- Convert: double spaces <-> entities
				if ($mode === 'DO_STUPEFY') {
					$text = str_replace('&nbsp;&nbsp;', '  ', $text);
					$trans_tbl = get_html_translation_table(HTML_ENTITIES, ENT_QUOTES);
					$trans_tbl = array_flip($trans_tbl);
					$ret = strtr($text, $trans_tbl);
					$text = preg_replace('/&#(\d+);/me', "chr('\\1')", $ret);
				} else {
					$text = str_replace('  ', '&nbsp;&nbsp;', $text);
					$pattern = '/href="(&)"/'; //TODO: href & to &amp;
				}

			}

			return $text;
		}

		//-----------------------------------------------
		//--- METHOD: pinger
		//--- PURPOSE: ping Ping-O-Matic!
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function pinger()
		{
			$current_er = error_reporting(0); //suppress until updated or replaced.  :(
			$stat = TRUE;
			if (defined('USE_PINGER') && USE_PINGER) {
				//define("XMLRPC_DEBUG", 1);
				$method = 'weblogUpdates.ping';
				$site = 'rpc.pingomatic.com';
				$location = '/RPC2';
				$params = array(XMLRPC_prepare(BLOG_NAME),XMLRPC_prepare(BASE_URL));
				list($stat, $result) = XMLRPC_request($site, $location, $method, $params);

				//--- Error
				/*
				XMLRPC_debug_print();
				if ($stat === FALSE) {
					foreach ($result as $key => $value) {
						echo(" $key => $value " . '<br/>');
					}
				}
				*/
			}
			error_reporting($current_er);

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: autoLink
		//--- PURPOSE: creates a link to another entry, comment or topic
		//--- PARAMETERS: type
		//---					id
		//---					text
		//--- RETURNS: XHTML link
		//-----------------------------------------------
		function autoLink($type, $id, $link_text)
		{
			$link = TRUE;
			$eid = $id;
			if ($type === 'C') {
				$query = "SELECT
								eid
							 FROM
						 		{$this->prefix}comments
						 	 WHERE
						 	 	cid='%i'";
				$query_data = array($id);

				if ($this->dbObj->safeQuery($query, $query_data)) {
					while ($this->dbObj->getRow()) {
						if ($this->dbObj->getResultCnt() > 0) {
							$eid = $this->dbObj->row['eid'];
						} else {
							$eid = -1;
						}
					}
				}
			} elseif ($type === 'T') {
				$query = "SELECT
								name
							 FROM
						 		{$this->prefix}topics
						 	 WHERE
						 	 	tid='%i'";
				$query_data = array($id);

				if ($this->dbObj->safeQuery($query, $query_data)) {
					while ($this->dbObj->getRow()) {
						if ($this->dbObj->getResultCnt() > 0) {
							$name = $this->dbObj->row['name'];
						}
					}
					$title = BLOG_NAME . ' ' . $this->langObj->get('TOPIC') . ': ' . $name;
					$link_text = ($link_text === 'ENTRY_NAME') ? $name : $link_text;
					$link = $this->buildLink(BASE_URL . '?tid=' . $id, $title, $link_text);
				} else {
					$link = $this->buildLink(BASE_URL, BLOG_NAME, BLOG_NAME);
				}
			} elseif ($type !== 'E') {
				$link = FALSE;
			}

			if ($type !== 'T') {
				$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? " AND display='1'" : '';
				$post_dated = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? ' AND updated<=UNIX_TIMESTAMP()' : '';
				$query = "SELECT
								title,
								updated
							 FROM
						 		{$this->prefix}entries
						 	 WHERE
						 	 	eid='%i'
						 	 $post_dated
						 	 $display_flag";
				$query_data = array($eid);

				if ($link && $this->dbObj->safeQuery($query, $query_data)) {
					$link = $this->buildLink(BASE_URL, BLOG_NAME, BLOG_NAME);
					while ($this->dbObj->getRow()) {
						if ($this->dbObj->getResultCnt() > 0) {
							$date = (defined('I18N_DATE_FORMAT')) ? strftime(I18N_DATE_FORMAT, $this->dbObj->row['updated'] + TIME_ADJUST) :	date(DATE_FORMAT, $this->dbObj->row['updated'] + TIME_ADJUST);
							if ($type === 'C') {
								$href = BASE_URL . '?eid=' . $eid . '#' . $id;
								$title = $date . ' - ' . BLOG_NAME . ': ' . $this->langObj->get('COMMENT_ON') . ' ' . $this->dbObj->row['title'];
								$link_text = ($link_text === 'ENTRY_NAME') ? strtolower($this->langObj->get('COMMENT_ON')) . ' ' . $this->dbObj->row['title'] : $link_text;
							} else {
								$href = BASE_URL . '?eid=' . $eid;
								$title = $date . ' - ' . BLOG_NAME . ': ' . $this->dbObj->row['title'];
								$link_text = ($link_text === 'ENTRY_NAME') ? $this->dbObj->row['title'] : $link_text;
							}
							$link = $this->buildLink($href, $title, $link_text);
						} else {
							$link = $this->buildLink(BASE_URL, BLOG_NAME, BLOG_NAME);
						}
					}
				}
			}

			return $link;
		}

		/**
		 * Build trackback auto-discovery RDF code.
		 *
		 * @return string $content XHTML comment trackback RDF
		 */
		function buildTrackbackRdf()
		{
			$url = BASE_URL . '?eid=' . $this->getId();
			$title = $this->getTitle();
			$ping = BASE_URL . 'trackback.php?eid=' . $this->getId();
			if ($this->getBody() > 256) {
				$desc = @htmlspecialchars(substr($this->getBody(), 0, 256)  . '...', ENT_QUOTES, I18N_CHARSET);
			} else {
				$desc = @htmlspecialchars($this->getBody(), ENT_QUOTES, I18N_CHARSET);
			}
			$by = $this->getPostBy();
			$date = date('Y-m-d\TH:i:sP', $this->getUpdated());
			$rdf = <<<TBRDF
				<!--
				<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
					xmlns:dc="http://purl.org/dc/elements/1.1/"
					xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">
				<rdf:Description
					rdf:about="$url"
					trackback:ping="$ping"
					dc:identifier="$url"
					dc:title="$title"
					dc:description="$desc"
					dc:creator="$by"
					dc:date="$date" />
				</rdf:RDF>
				-->

TBRDF;

			return $rdf;
		}


		/**
		 * Send a trackback to a remote site for this entry.
		 *
		 * @return string Trackback XML response message.
		 */
		function sendTrackback()
		{
		 	require(LIB_PATH . 'PbTrackback.php');
			$tb = new PbTrackback(DB_PREFIX, $this->getRblServices());
			$tb->setTrackbackOptions(TB_STRIP_TAGS, TB_EXCERPT_LENGTH);
			$excerpt = (strlen($this->getMore()) > 0) ? $this->getBody() . '<br /><br />' . $this->getMore() : $this->getBody();
			$tb->setTrackback($this->getId(), I18N_CHARSET, $this->getTbUrl(), $this->getTitle(), $excerpt, BLOG_NAME);
			$tb->sendTrackback();

			return $tb->getResult(TRUE);
		}


	} //end of class
?>
