<?php

//--- Includes
require_once(LIB_PATH . 'PbBase_class.php');

/**
 * Realtime Blacklist (RBL) Class
 *
 * This class does RBL IP address and HTTP referer keyword lookups and blocking.
 *
 * This work is partially derived from PHPrbl: PHP realtime black listing V0.4
 * Released under GNU GPL License version 2, see LICENSE for more info
 * @link http://phprbl.init1.nl || http://eol.init1.nl
 *
 * @author Kevin L. Papendick <kevinp@polarlava.com>
 * @copyright 2005 Eelco Wesemann <eelco@init1.nl>
 * @copyright 2006 Kevin L. Papendick <kevinp@polarlava.com>
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @version V1.0 - 20060812
 */
class PbRbl extends PbBase  {

    /**
     * Check referer URL's for bad words or strings?
     * @var boolean
     */
	var $check_url_keywords;

    /**
     * Indicates that a posts for this IP should be blocked. (Akismet Auto-Block)
     * @var boolean
     */
	var $force_block;

    /**
     * The IP address to check.
     * @var string
     */
	var $ip;

    /**
     * The number of times an IP address has been seen.
     * @var string
     */
	var $ip_visits;

    /**
     * The associative array of all blocked keyword information.
     * @var array
     */
	var $keywords_all;

    /**
     * Log IP address that gave a bad referer string?
     * @var boolean
     */
	var $keywords_autoblock;

    /**
     * Matched keywords in referer URL.
     * @var array
     */
	var $keyword_matches;

    /**
     * The associative array of all local DB IP information.
     * @var array
     */
	var $local_rbls;

    /**
     * The referring URL.
     * @var string
     */
	var $referer;

    /**
     * The IP address to check, reversed.
     * @var string
     * @private
     */
	var $reverse_ip;

    /**
     * A list of RBL services that found an IP listed.
     * @var array
     */
	var $rbl_listed;

    /**
     * A list of RBL services to check.
     * @var array
     */
	var $rbl_services;

    /**
     * The requested Comment or Trackback object.
     * @var object
     */
	var $reqObj;

    /**
     * Flag indicating IP address has been whitelisted.
     * @var boolean
     */
	var $whitelist;



	/**
	 * RBL Object Constructor
	 *
	 * @param string $prefix The table name prefix.
	 * @param array $services An array of RBL services to utilize.
	 * @param boolean $chk_keywd Indicates whether URL's should be checked for bad keywords
	 * @param boolen $keywd_blk Indicates if the IP address should be blocked when bad keywords are found.
	 * @param object $obj Comment or Trackback object
	 */
	function PbRbl($prefix = 'wl_', $services = array ('zen.spamhaus.org', 'rbl.init1.nl', 'dnsbl.ahbl.org', 'blocklist2.squawk.com'), $chk_keywd = FALSE, $keywd_blk = FALSE, $obj = FALSE)
	{
		$this->PbBase($prefix);
		$this->setRblServices($services);
		$this->setCheckUrlKeywords($chk_keywd);
		$this->setKeywordsAutoblock($keywd_blk);
		$this->setReqObj($obj);
		$this->setForceBlock(FALSE);
	}


//---------------------//
//--- SET FUNCTIONS ---//
//---------------------//

	/**
	 * Sets whether to check referer URL's for bad words or strings.
	 *
	 * @param boolean $a
	 * @return void
	 */
	function setCheckUrlKeywords($a = FALSE)
	{
		$this->check_url_keywords = $a;
	}


	/**
	 * Sets that a posts for this IP should be blocked. (Akismet Auto-Block)
	 *
	 * @param boolean $a
	 * @return void
	 */
	function setForceBlock($a = FALSE)
	{
		$this->force_block = $a;
	}


	/**
	 * Adds service(s) to the list of RBL's where an IP is listed.
	 *
	 * @param string $a A single service or a pipe delimited list of services.
	 * @return void
	 */
	function setRblListed($a)
	{
		if (!$a) {
			$this->rbl_listed = array();
		} elseif (strpos($a, '|') !== FALSE) {
			$this->rbl_listed = explode('|', $a);
		} else {
			$this->rbl_listed[] = $a;
		}
	}


	/**
	 * Sets IP address to check.
	 *
	 * @param string $a
	 * @return void
	 */
	function setIp($a = NULL)
	{
		$this->ip = $a;
		$this->setReverseIp();
	}


	/**
	 * Sets the number of times an IP address has been seen.
	 *
	 * @param int $a
	 * @return void
	 */
	function setIpVisits($a)
	{
		$this->ip_visits = (int) $a;
	}


	/**
	 * Sets whether to log IP address that gave a bad referer string.
	 *
	 * @param boolean $a
	 * @return void
	 */
	function setKeywordsAutoblock($a = FALSE)
	{
		$this->keywords_autoblock = $a;
	}


	/**
	 * Sets the list of RBL services to check an initialize the RBL listed array.
	 *
	 * @param array $a
	 * @return void
	 */
	function setRblServices($a = array ('zen.spamhaus.org', 'rbl.init1.nl', 'dnsbl.ahbl.org'))
	{
		$this->rbl_services = $a;
		$this->rbl_listed = array();
	}


	/**
	 * Sets the referring URL.
	 *
	 * @param string $a
	 * @return void
	 */
	function setReferer($a = NULL)
	{
		$this->referer = $a;
	}


	/**
	 * Sets IP address to check, reversed.
	 *
	 * @private
	 * @return void
	 */
	function setReverseIp()
	{
		$ip = $this->getIp();
		$this->reverse_ip = implode('.', array_reverse(explode('.', $ip)));
	}


	/**
	 * Sets the requested Comment or Trackback object.
	 *
	 * @private
	 * @param object $obj
	 * @return void
	 */
	function setReqObj($obj)
	{
		$this->reqObj = $obj;
	}


	/**
	 * Sets the IP address as being whitelisted.
	 *
	 * @param boolean $a
	 * @return void
	 */
	function setWhitelist($a = NULL)
	{
		$this->whitelist = $a;
	}


	/**
	 * Sets an associative array of all local DB IP information.
	 *
	 * @return void
	 */
	function setLocalRbls()
	{
		$this->local_rbls = array();

		$query = "SELECT
					id,
					ip,
					visits,
					lastseen,
					services,
					referer,
					whitelist
				  FROM
				  	{$this->prefix}rbl_blocked
				  ORDER BY
				  	ip";

		if ($this->dbObj->query($query)) {
			if ($this->dbObj->getResultCnt() > 0) {
				while ($row = $this->dbObj->getRow()) {
					$this->local_rbls[] = array(
											'id'	=> $row['id'],
											'ip'	=> $row['ip'],
											'lastseen'	=> $row['lastseen'],
											'visits'	=> $row['visits'],
											'services'	=> $row['services'],
											'referer'	=> $row['referer'],
											'whitelist'	=> $row['whitelist']
										  );
				}
			}
		} else {
			//--- Query Error
			echo $this->dbObj->getError();
			$stat = FALSE;
		}
	}


	/**
	 * Sets an associative array of all blocked keyword information.
	 *
	 * @return void
	 */
	function setAllBlockedKeywords()
	{
		$this->keywords_all = array();

			$query = "SELECT
						id,
						keyword,
						occurances,
						lastseen
					  FROM
					  	{$this->prefix}rbl_keywords
					  ORDER BY
					  	keyword";

		if ($this->dbObj->query($query)) {
			if ($this->dbObj->getResultCnt() > 0) {
				while ($row = $this->dbObj->getRow()) {
					$this->keywords_all[] = array(
											'id'			=> $row['id'],
											'keyword'		=> $row['keyword'],
											'occurances'	=> $row['occurances'],
											'lastseen'			=> $row['lastseen']
										  );
				}
			}
		} else {
			//--- Query Error
			echo $this->dbObj->getError();
			$stat = FALSE;
		}
	}




//---------------------//
//--- GET FUNCTIONS ---//
//---------------------//


	/**
	 * Gets whether to check referer URL's for bad words or strings.
	 *
	 * @return boolean $this->check_url_keywords
	 */
	function getCheckUrlKeywords()
	{
		return $this->check_url_keywords;
	}


	/**
	 * Gets that a posts for this IP should be blocked. (Akismet Auto-Block)
	 *
	 * @return string $this->force_block
	 */
	function getForceBlock()
	{
		return $this->force_block;
	}


	/**
	 * Gets the ip address that is being checked.
	 *
	 * @return string $this->ip
	 */
	function getIp()
	{
		return $this->ip;
	}


	/**
	 * Gets an associative array of all local DB IP information..
	 *
	 * @return array $this->local_rbls
	 */
	function getLocalRbls()
	{
		if ($this->local_rbls === NULL) {
			$this->setLocalRbls();
		}

		return $this->local_rbls;
	}


	/**
	 * Gets the number of times an IP address has been seen.
	 *
	 * @return int $this->rbl_cnt
	 */
	function getIpVisits()
	{
		return $this->ip_visits;
	}


	/**
	 * Gets whether to block IP address that gave a bad referer string.
	 *
	 * @return boolean $this->keywords_autoblock
	 */
	function getKeywordsAutoblock()
	{
		return $this->keywords_autoblock;
	}


	/**
	 * Gets all blocked keyword information.
	 *
	 * @return boolean $this->keywords_all
	 */
	function getAllBlockedKeywords()
	{
		if ($this->keywords_all === NULL) {
			$this->setAllBlockedKeywords();
		}

		return $this->keywords_all;
	}


	/**
	 * Gets the list of RBL services an IP was found in.
	 *
	 * @return array $this->rbl_listed
	 */
	function getRblListed()
	{
		return array_unique($this->rbl_listed);
	}


	/**
	 * Gets the list of RBL services to check.
	 *
	 * @return array $this->rbl_services
	 */
	function getRblServices()
	{
		return $this->rbl_services;
	}


	/**
	 * Gets the referring URL.
	 *
	 * @return string $this->referer
	 */
	function getReferer()
	{
		return $this->referer;
	}


	/**
	 * Gets the ip address that is being checked.
	 *
	 * @private
	 * @return string $this->reverse_ip
	 */
	function getReverseIp()
	{
		return $this->reverse_ip;
	}


	/**
	 * Gets the requested Comment or Trackback object.
	 *
	 * @private
	 * @return object $this->reqObj
	 */
	function getReqObj()
	{
		return $this->reqObj;
	}


	/**
	 * Gets the whitelist value for the IP Address.
	 *
	 * @return boolean $this->whitelist
	 */
	function getWhitelist()
	{
		return $this->whitelist;
	}





//----------------------//
//--- MISC FUNCTIONS ---//
//----------------------//


	/**
	 * Checks for listing with Akismet spam service.
	 *
	 * @link http://akismet.com/
	 * @link http://www.achingbrain.net/stuff/akismet/
	 * @return boolean TRUE if flagged as spam.
	 **/
	function checkAkismet()
	{
		if (!defined('RBL_AKISMET_KEY') && (!$this->getSpl() || RBL_AKISMET_KEY === FALSE)) { return FALSE; };
		$akismet_auto_del = (defined('RBL_AKISMET_AUTODEL') && RBL_AKISMET_AUTODEL === TRUE) ? TRUE : FALSE;

		$obj = $this->getReqObj();
		$url = (strpos($obj->getUrl(), 'http://') === FALSE) ? 'http://' . $obj->getUrl() : $obj->getUrl();
		if ($obj->getIsTrackback()) {
			$name = $obj->getTbBlogName();
			$email = '';
			$comment = $obj->getTbExcerptRaw();
		} else {
			$name = $obj->getName();
			$email = $obj->getEmail();
			$comment = $obj->getComment();
		}

		require_once(LIB_PATH . 'PHP5Akismet/Akismet.class.php');
		$akismet = @new Akismet(BASE_URL, RBL_AKISMET_KEY);
		$akismet->setCommentAuthor($name);
		$akismet->setCommentAuthorEmail($email);
		$akismet->setCommentAuthorURL($url);
		$akismet->setCommentContent($comment);
		$akismet->setUserIP($this->getIp()); //override with our method of IP detection
		$akismet->setPermalink(BASE_URL . '?eid=' . $obj->getEid());

		$stat = FALSE;
		if (@$akismet->isCommentSpam()) {
			$this->setRblListed('akismet');
			if (!$akismet_auto_del) {
				$this->updateLocalRblDb();
			} else {
				$this->setForceBlock(TRUE);
			}
			$stat = TRUE;
		}

		return $stat;
	}

	/**
	 * Checks for IP listing with the RBL services.
	 *
	 * @param string $ip The IP address to check.
	 * @param string $referer The referring URL.
	 * @return boolean TRUE if found in RBL.
	 */
	function checkIp($ip, $referer = NULL)
	{
		$stat = FALSE;

		$this->setIp($ip);
		$this->setReferer($referer);
		$this->setIpVisits(0);
		$this->setRblListed(FALSE);

		if ($this->checkLocalRbl()) {
			$this->updateLocalRblDb();
			$stat = TRUE;
		} else {
			$stat = $this->checkAkismet();
			if (!$stat) {
				$stat = $this->checkReferer();
				if (!$stat) {
					$stat = $this->checkRemoteRbl();
				}
			}
		}

		return (!$this->getWhitelist() && $stat);
	}


	/**
	 * Checks for IP listing in the local database.
	 *
	 * @param string $ip The IP address to check.
	 * @return boolean TRUE if found in local RBL.
	 */
	function checkLocalRbl($ip = NULL)
	{
		$stat = FALSE;

		if (is_null($ip)) {
			$ip = $this->getIp();
		}

		$query = "SELECT
					visits,
					services,
					whitelist
				  FROM
				  	{$this->prefix}rbl_blocked
				  WHERE
				  	ip='%s'
				  LIMIT 1";
		$query_data = array($ip);

		if ($this->dbObj->safeQuery($query, $query_data)) {
			if ($this->dbObj->getResultCnt() > 0) {
				$row = $this->dbObj->getRow();
				$this->setIpVisits($row['visits']);
				$this->setRblListed($row['services']);
				$this->setWhitelist($row['whitelist']);
				$stat = TRUE;
			}
		} else {
			//--- Query Error
			echo $this->dbObj->getError();
			$stat = FALSE;
		}

			return $stat;
	}


	/**
	 * A simple wrapper of checkForKeywords() that checks for keywords in the referer URL.
	 *
	 * @param string $referer The referer URL to check.
	 * @return boolean TRUE if keywords found in URL.
	 * @see checkForKeywords()
	 */
	function checkReferer($referer = NULL)
	{
		if (!$this->getCheckUrlKeywords()) {
			return FALSE;
		}

		if (is_null($referer)) {
			$referer = $this->getReferer();
		}

		return $this->checkForKeywords($referer);
	}


	/**
	 * Checks for keywords in the provided string.
	 *
	 * @param string $text The text to check.
	 * @param boolean $update Indicate if keyword count should automatically be updated on find.
	 * @return boolean TRUE if keywords found.
	 */
	function checkForKeywords($text = NULL, $update = TRUE)
	{
		$stat = FALSE;

		if (!empty($text) && !$this->getWhitelist()) {
			$text = strtolower($text);
			$word_matches = FALSE;

			$query = "SELECT
						keyword,
						occurances
					  FROM
					  	{$this->prefix}rbl_keywords";

			if ($this->dbObj->query($query)) {
				while ($this->dbObj->getRow()) {
					$keyword = strtolower($this->dbObj->row['keyword']);
					$occurances = $this->dbObj->row['occurances'];
					$pattern = '/' . preg_quote($keyword, '/') . '/';
					$matches = array();
					if (preg_match($pattern, preg_quote($text, '/'), $matches, PREG_OFFSET_CAPTURE)) {
						if ($update) {
							$word_matches = TRUE;
							$this->addFoundKeyword($keyword);
							$stat = $this->updateKeywordCnt($keyword);
						} else {
							$stat = TRUE;
						}
					}
				}

				if ($this->getKeywordsAutoblock() && $update && $word_matches) {
					$stat = $this->updateLocalRblDb();
					$this->setRblListed('local_rbl');
				}
			} else {
				//--- Query Error
				echo $this->dbObj->getError();
			}
		}

		return $stat;
	}


	/**
	 * Checks for IP listing in the local database.
	 *
	 * @param string $ip The IP address to check.
	 * @return boolean TRUE if found in local RBL.
	 */
	function checkRemoteRbl($ip = NULL)
	{
		$debug = FALSE;
		$stat = FALSE;
		$pattern = '/127.0.0.?/';

		if (is_null($ip)) {
			$ip = $this->getReverseIp();
		}

		if (!is_array($this->getRblServices())) {
			return $stat;
		}

		//--- SPL Support
		if ($this->getSpl()) {
			foreach (new ArrayObject($this->getRblServices()) as $rbl_service) {
				$lookup_rbl_ip = $this->getReverseIp() . '.' . $rbl_service;
				$do_lookup = gethostbyname($lookup_rbl_ip);
				if ($debug) {
					echo '<br />IP: ' . $ip . '<br />';
					echo 'result: ' . $do_lookup . '<br />';
				}
				if (preg_match($pattern, $do_lookup)) {
					$this->setRblListed($rbl_service);
					$stat = TRUE;
					if ($debug) {
						echo 'listed: ' . $rbl_service . '<br />';
					}
				}
			}
		} else {
			foreach ($this->getRblServices() as $rbl_service) {
				$lookup_rbl_ip = $this->getReverseIp() . '.' . $rbl_service;
				$do_lookup = gethostbyname($lookup_rbl_ip);
				if ($debug) {
					echo '<br />IP: ' . $ip . '<br />';
					echo 'result: ' . $do_lookup . '<br />';
				}
				if (preg_match($pattern, $do_lookup)) {
					$this->setRblListed($rbl_service);
					$stat = TRUE;
					if ($debug) {
						echo 'listed: ' . $rbl_service . '<br />';
					}
				}
			}
		}

		if ($stat) {
			$this->updateLocalRblDb();
		}

		return $stat;
	}


	/**
	 * Update/Add IP listing in the local database.
	 *
	 * @return boolean Success/Failure of database action.
	 */
	function updateLocalRblDb()
	{
		$stat = FALSE;
		$ip = $this->getIp();
		$services = implode('|', $this->getRblListed());
		$referer = $this->getReferer();

		if (in_array('local_rbl', $this->getRblListed())) {
			$this->setIpVisits($this->getIpVisits() + 1);
			$query = "UPDATE
						{$this->prefix}rbl_blocked
					  SET
					  	lastseen=NOW(),
					  	services='%s',
					  	visits='%i',
					  	referer='%s'
					  WHERE
					  	ip='%s'";
		$query_data = array($services, $this->getIpVisits(), $referer, $ip);
		} else {
			$services .= (!empty($services)) ? '|local_rbl' : 'local_rbl';
			$query = "INSERT INTO
						{$this->prefix}rbl_blocked
					  (
					  	ip,
					  	visits,
					  	lastseen,
					  	services,
					  	referer
					  )
					  VALUES
					  (
					  	'%s',
					  	'1',
					  	NOW(),
					  	'%s',
					  	'%s'
					  )";
		$query_data = array($ip, $services, $referer);
		}

		if ($this->dbObj->safeQuery($query, $query_data)) {
			$stat = TRUE;
		} else {
			//--- Query Error
			echo $this->dbObj->getError();
		}

		return $stat;
	}

	/**
	 * Add keyword found in referer URL.
	 *
	 * @param $keyword
	 */
	function addFoundKeyword($keyword)
	{
		if (!isset($this->keyword_matches)) {
			$this->keyword_matches = array();
		}

		array_push($this->keyword_matches, $keyword);

		return;
	}

	/**
	 * Update occurance count for keyword found in referer URL.
	 *
	 * @param $keyword
	 *
	 * @return boolean
	 */
	function updateKeywordCnt($keyword)
	{
		$stat = FALSE;
		$query = "UPDATE
						{$this->prefix}rbl_keywords
					  SET
					  	lastseen=NOW(),
					  	occurances=occurances+1
					  WHERE
					  	keyword='%s'";
		$query_data = array($keyword);

		if ($this->dbObj->safeQuery($query, $query_data)) {
			$stat = TRUE;
		} else {
			//--- Query Error
			echo $this->dbObj->getError();
		}

		return $stat;
	}

	/**
	 * Update whitelist value for an IP.
	 *
	 * @param boolean $wl TRUE = whitelist, FALSE = blacklist
	 * @param string $ip The IP address to add or change OR the ID to change.
	 * @return boolean
	 */
	function updateWhitelist($wl, $ip)
	{
		$stat = FALSE;

		if (empty($ip)) {
			return $stat;
		}

		$whitelist = ($wl) ? '1' : '0';
		if (is_int($ip)) {
			$query = "UPDATE
							{$this->prefix}rbl_blocked
						  SET
						  	whitelist='%s'
						  WHERE
						  	id='%s'";
			$query_data = array($whitelist, $ip);
		} elseif ($this->checkLocalRbl($ip)) {
			$query = "UPDATE
							{$this->prefix}rbl_blocked
						  SET
						  	whitelist='%s'
						  WHERE
						  	ip='%s'";
			$query_data = array($whitelist, $ip);
		} else {
			$query = "INSERT INTO
						{$this->prefix}rbl_blocked
					  (
					  	ip,
					  	visits,
					  	lastseen,
					  	services,
					  	referer
					  )
					  VALUES
					  (
					  	'%s',
					  	'1',
					  	NOW(),
					  	'local_rbl',
					  	''
					  )";
			$query_data = array($ip);
		}

		if ($this->dbObj->safeQuery($query, $query_data)) {
			$stat = TRUE;
		} else {
		//--- Query Error
		echo $this->dbObj->getError();
		}

		return $stat;
	}

	/**
	 * Delete an RBL Entry
	 *
	 * @param int $id The entry ID to be deleted
	 * @return boolean
	 */
	function delEntry($id)
	{
		$query = "DELETE FROM
					{$this->prefix}rbl_blocked
				  WHERE
				  	id='%s'";
		$query_data = array($id);

		if ($this->dbObj->safeQuery($query, $query_data)) {
			$stat = TRUE;
		} else {
			$stat = FALSE;
			echo $this->dbObj->getError();
		}

		return $stat;
	}

	/**
	 * Delete an RBL Keyword
	 *
	 * @param int $id The entry ID to be deleted
	 * @return boolean
	 */
	function delKeyword($id)
	{
		$query = "DELETE FROM
					{$this->prefix}rbl_keywords
				  WHERE
					id='%s'";
		$query_data = array($id);

		if ($this->dbObj->safeQuery($query, $query_data)) {
			$stat = TRUE;
		} else {
			$stat = FALSE;
			echo $this->dbObj->getError();
		}

		return $stat;
	}

	/**
	 * Add an RBL Keyword
	 *
	 * @param int $word The keyword to be added
	 * @return boolean
	 */
	function addKeyword($word)
	{
		$stat = TRUE;
		if (!$this->checkForKeywords($word, FALSE)) {
			$query = "REPLACE INTO
						{$this->prefix}rbl_keywords
					  	(keyword, occurances, lastseen)
					  VALUES
					  	('%s', 0, NOW())";
			$query_data = array($word);

			if ($this->dbObj->safeQuery($query, $query_data)) {
				$stat = TRUE;
			} else {
			$stat = FALSE;
			echo $this->dbObj->getError();
			}
		}

		return $stat;
	}
} // end of class
?>
