<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-2006  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require_once(LIB_PATH . 'PbCommentBase_class.php');

	/**
	 * Trackback Class
	 *
	 * This class provides entry and comment trackback support.
	 *
	 * @package PolarBlog
	 * @author Kevin L. Papendick <kevinp@polarlava.com>
	 * @copyright 2006 Kevin L. Papendick <kevinp@polarlava.com>
	 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
	 * @version V1.0
	 * @link http://www.sixapart.com/pronet/docs/trackback_spec
     * @link http://www.npj.ru/artreal/zprj/xtrackback-eng
	 */
	class PbTrackback extends PbCommentBase
	{

	    /**
	     * The XML status message to be sent in response to trackback ping.
	     * @var string
	     */
		var $response;

	    /**
	     * The status message received in response to trackback ping.
	     * @var string
	     */
		var $result;

		/**
		 * The trackback character set
		 * @var string
		 */
		var $tb_charset;

		/**
		 * The trackback title
		 * @var string
		 */
		var $tb_title;

		/**
		 * The trackback excerpt
		 * @var string
		 */
		var $tb_excerpt;

		/**
		 * The raw trackback data
		 * @var string
		 */
		var $tb_excerpt_raw;

		/**
		 * The trackback excerpt length
		 * @var int
		 */
		var $tb_excerpt_length;

		/**
		 * The trackback blog name
		 * @var string
		 */
		var $tb_name;

		/**
		 * Whether the trackback fields should have HTML tags stripped.
		 * @var string
		 */
		var $tb_strip_tags;


		/**
		 * Trackback Constructor
		 *
		 * @param string $prefix The database table prefix.
		 * @param array $rbls List of RBL services to be polled.
		 * @param int $cid The trackback (comment) ID.
		 */
		function PbTrackback($prefix = 'wl_', $rbls = array(), $cid = -1)
		{
			$this->PbCommentBase($prefix, $rbls, $cid);
			$this->setRblServices($rbls);
			$this->setRblBlocked(FALSE);
			$this->setTrackbackOptions(FALSE, -1);
			$this->setId($cid);
		}


	//--------------------//
	//--- SET METHODS ----//
	//--------------------//

		/**
		 * Sets the trackback XML response data.
		 *
		 * @access private
		 * @param string $fail Fail message or nothing on success.
		 */
		function setResponse($fail = FALSE)
		{
			if ($fail) {
				$this->response = '<?xml version="1.0" encoding="' . $this->getTbCharset() . '"?'.">\n";
				$this->response .= "<response>\n";
				$this->response .= "<error>1</error>\n";
				$this->response .= '<message>' . $fail . "</message>\n";
				$this->response .= '</response>';
			} else {
				$this->response = '<?xml version="1.0" encoding="' . $this->getTbCharset() . '"?'.">\n";
				$this->response .= "<response>\n";
				$this->response .= "<error>0</error>\n";
				$this->response .= '</response>';
			}
		}

		/**
		 * Sets the trackback response result.
		 *
		 * @access private
		 * @param string $a Result of trackback ping attempt.
		 */
		function setResult($a)
		{
			$this->result = $a;
		}

		/**
		 * Sets the parameters for a trackback.
		 *
		 * @param int $eid The entry id
		 * @param string $charset The trackback charset
		 * @param string $url The trackback URL
		 * @param string $title The trackback title
		 * @param string $url The trackback excerpt
		 * @param string $url The trackback blog name
		 */
		function setTrackback($eid, $charset, $url, $title = '', $excerpt = '', $name = '')
		{
			$this->setEid($eid);
			$this->setPid(0); //always zero, top-level comment
			$this->setTbCharset($charset);
			$this->setTbUrl($url);
			$this->setTbTitle($title);
			$this->setTbExcerpt($excerpt);
			$this->setTbBlogName($name);
		}

		/**
		 * Sets the options for a trackback.
		 *
		 * @param boolean $strip Strip tags from excerpt?
		 * @param int $excerpt_length The trackback blog name
		 * @param boolean $strip Strip HTML tags from trackback fields?
		 */
		function setTrackbackOptions($strip = TRUE, $excerpt_length = -1)
		{
			$this->setTbExcerptLength($excerpt_length);
			$this->setTbStripTags($strip);
		}

		/**
		 * Set the trackback character set
		 *
		 * @protected
		 * @param string $a
		 */
		function setTbCharset($a)
		{
			$this->tb_charset = $a;
		}

		/**
		 * Set the trackback title.
		 *
		 * @protected
		 * @param string $a
		 */
		function setTbTitle($a)
		{
			$this->tb_title = ($this->getTbStripTags()) ? strip_tags($a) : $a;
		}

		/**
		 * Set the trackback content excerpt.
		 *
		 * @protected
		 * @param string $a
		 */
		function setTbExcerpt($a)
		{
			$this->setTbExcerptRaw($a);
			$a = ($this->getTbStripTags()) ? strip_tags($a) : $a;
			strlen($a);
			if ($this->getTbExcerptLength() != -1 && strlen($a) > $this->getTbExcerptLength()) {
				$this->tb_excerpt = substr($a, 0, $this->getTbExcerptLength()) . '&hellip;';
			} else {
				$this->tb_excerpt = $a;
			}
		}

		/**
		 * Set the raw trackback content.
		 *
		 * @protected
		 * @param string $a
		 */
		function setTbExcerptRaw($a)
		{
			$this->tb_excerpt_raw = $a;
		}

		/**
		 * Set the trackback content excerpt length.
		 *
		 * @param int $a
		 */
		function setTbExcerptLength($a)
		{
			$this->tb_excerpt_length = ($this->getTbStripTags()) ? strip_tags($a) : $a;
		}

		/**
		 * Set the trackback blog name.
		 *
		 * @protected
		 * @param string $a
		 */
		function setTbBlogName($a)
		{
			$this->tb_name = ($this->getTbStripTags()) ? strip_tags($a) : $a;
		}

		/**
		 * Sets whether the trackback fields should have HTML tags stripped.
		 *
		 * @param boolean $a
		 */
		function setTbStripTags($a)
		{
			$this->tb_strip_tags = $a;
		}

		/**
		 * Set Trackback URL
		 *
		 * @protected
		 * @param string $a
		 */
		function setTbUrl($a)
		{
			$this->setUrl(($this->getTbStripTags()) ? strip_tags($a) : $a);
		}


	//--------------------//
	//--- GET METHODS ----//
	//--------------------//

		/**
		 * Gets the trackback XML response data.
		 *
		 * @return string $this->response
		 */
		function getResponse()
		{
			return $this->response;
		}

		/**
		 * Gets the trackback response result data.
		 *
		 * @param boolean $parsed Return just error message.
		 * @return string $this->result
		 */
		function getResult($parsed = FALSE)
		{
			if ($parsed) {
				return $this->parseResult();
			} else {
				return $this->result;
			}
		}

		/**
		 * Gets the trackback character set.
		 *
		 * @return string $this->tb_charset
		 */
		function getTbCharset()
		{
			return $this->tb_charset;
		}

		/**
		 * Gets the trackback title.
		 *
		 * @return string $this->tb_title
		 */
		function getTbTitle()
		{
			return $this->tb_title;
		}

		/**
		 * Gets the trackback content excerpt.
		 *
		 * @return string $this->tb_excerpt
		 */
		function getTbExcerpt()
		{
			return $this->tb_excerpt;
		}

		/**
		 * Gets the raw trackback content.
		 *
		 * @return string $this->tb_excerpt_raw
		 */
		function getTbExcerptRaw()
		{
			return $this->tb_excerpt_raw;
		}

		/**
		 * Gets the trackback content excerpt length.
		 *
		 * @return string $this->tb_excerpt_length
		 */
		function getTbExcerptLength()
		{
			return $this->tb_excerpt_length;
		}

		/**
		 * Gets the trackback blog name.
		 *
		 * @return string $this->tb_name
		 */
		function getTbBlogName()
		{
			return $this->tb_name;
		}

		/**
		 * Gets whether the trackback fields should have HTML tags stripped.
		 *
		 * @return string $this->tb_strip_tags
		 */
		function getTbStripTags()
		{
			return $this->tb_strip_tags;
		}

		/**
		 * Get trackback URL.  This is just a wrapper for getUrl().
		 *
		 * @see getUrl()
		 * @return string
		 */
		function getTbUrl()
		{
			return $this->getUrl();
		}


	//----------------------//
	//--- OTHER METHODS ----//
	//----------------------//

		/**
		 * Send a trackback ping to a remote site
		 *
		 */
		function sendTrackback()
		{
			$errno = FALSE;
			$errstr = FALSE;

		    $url = parse_url($this->getTbUrl());
		    $url['scheme'] = (array_key_exists('scheme', $url)) ? $url['scheme'] : '';
		    $url['host'] = (array_key_exists('host', $url)) ? $url['host'] : '';
		    $url['port'] = (array_key_exists('port', $url)) ? $url['port'] : 80;
		    $url['port'] = ($url['scheme'] === 'https' && $url['port'] == 80) ? 443 : $url['port'];
		    $url['path'] = (array_key_exists('path', $url)) ? $url['path'] : '';
		    $url['query'] = (array_key_exists('query', $url)) ? '?' . $url['query'] : '';
		    $url['fragment'] = (array_key_exists('fragment', $url)) ? '#' . $url['fragment'] : '';

		    $fp = fsockopen($url['host'], $url['port'], $errno, $errstr, 10);
		    if ($fp) {
		        $post_data = 'url=' . rawurlencode(BASE_URL . '?eid=' . $this->getEid());
		        $post_data .= '&title=' . rawurlencode($this->getTbTitle());
		        $post_data .= '&excerpt=' . rawurlencode($this->getTbExcerpt());
		        $post_data .= '&blog_name=' . rawurlencode($this->getTbBlogName());

		        $header= 'POST ' . $url['path']  . $url['query'] . $url['fragment'] . " HTTP/1.1\r\n";
		        $header .= 'Host: ' . $url['host'] . "\r\n";
		        $header .= "User-Agent: PolarBlog Trackback Agent 1.0\r\n";
		        $header .= 'Content-type: application/x-www-form-urlencoded; charset=' . $this->getTbCharset() . "\r\n";
		        $header .= 'Content-length: ' . strlen($post_data) . "\r\n";
		        $header .= "Connection: close\r\n\r\n";

		        fwrite($fp, $header . $post_data);
				$res = '';
		        while(!feof($fp)) {
		            $res .= fgets($fp, 128);
		        }
		        fclose($fp);

		    } else {
		    	$res = "<message>DB: ($errno) $errstr</message>";
		    }
			$this->setResult($res);

			return;
		}

		/**
		 * Receive a trackback ping from a remote site
		 *
		 * @return boolean Success status
		 */
		function receiveTrackback()
		{
			$stat = FALSE;
			$this->setDisplay('1');
			$save = TRUE;
			$use_rbl = (defined('USE_RBL') && USE_RBL) ? TRUE : FALSE;
			$rbl_referer_check = (defined('RBL_REFERER_CHECK') && RBL_REFERER_CHECK) ? TRUE : FALSE;
			$rbl_block_keyword = (defined('RBL_BLOCK_KEYWORD') && RBL_BLOCK_KEYWORD) ? TRUE : FALSE;
			$rbl_url_check = (defined('RBL_URL_CHECK') && RBL_URL_CHECK) ? TRUE : FALSE;
			$rbl_text_check = (defined('RBL_TEXT_CHECK') && RBL_TEXT_CHECK) ? TRUE : FALSE;
			$rbl_save_blocked = (defined('RBL_SAVE_BLOCKED') && RBL_SAVE_BLOCKED) ? TRUE : FALSE;
			$tb_msg = FALSE;

			$this->setIsTrackback(TRUE);
			if (!$this->validateTrackback()) {
				return $stat;
			}

			$ip_blocked = $this->getIpBlocked($this->getUserIp());
			$rbl_blocked = FALSE;
			$url_blocked = FALSE;
			$text_blocked = FALSE;

			if ($ip_blocked) {
				$tb_msg = $this->langObj->get('TB_IP_BLOCK');
			}

			if ($use_rbl) {
				$this->setRblObj(DB_PREFIX, $this->getRblServices(), $rbl_referer_check, $rbl_block_keyword);
			}
			if ($use_rbl && !$ip_blocked) {
				$rbl_referer = (isset($_SERVER['HTTP_REFERER'])) ? strip_tags($_SERVER['HTTP_REFERER']) : NULL;
				$rbl_blocked = $this->rblObj->checkIp($this->getUserIp(), $rbl_referer);
				$this->setRblBlocked($rbl_blocked);
				if ($rbl_blocked) {
					$tb_msg = $this->langObj->get('TB_RBL_IP');
					$tb_msg .= ($rbl_save_blocked) ? '  ' . $this->langObj->get('TB_SAVE_REVIEW') : '';
				}

				//--- TBer's URL
				if ($rbl_url_check && !$rbl_blocked) {
					$url_blocked = $this->rblObj->checkForKeywords($this->getUrl());
					$this->setRblBlocked($url_blocked);
					if ($url_blocked) {
						$tb_msg = $this->langObj->get('TB_RBL_URL');
						$tb_msg .= ($url_blocked) ? '  ' . $this->langObj->get('TB_SAVE_REVIEW') : '';
					}
				}

				//--- Text Check
				if ($rbl_text_check && !$url_blocked && !$rbl_blocked) {
					$text_blocked = $this->rblObj->checkForKeywords($this->getTbExcerpt());
					$this->setRblBlocked($text_blocked);
					if ($text_blocked) {
						$tb_msg = $this->langObj->get('TB_RBL_EXCERPT');
						$tb_msg .= ($text_blocked) ? '  ' . $this->langObj->get('TB_SAVE_REVIEW') : '';
					}
				}
			}
			if ($use_rbl) {
				$this->setDisplay(($rbl_blocked || $url_blocked || $text_blocked) ? '0' : '1');
				$save = $rbl_save_blocked;
			}

			//--- Force block/no display
			if ($use_rbl && $this->rblObj->getForceBlock()) {
				$tb_msg = $this->langObj->get('TB_RBL_IP');
				$this->setDisplay(($this->rblObj->getForceBlock()) ? '0' : '1');
				$save = FALSE;
			}

			if (USE_TRACKBACK && USE_COMMENTS && $this->commentsEnabled($this->getEid()) && !$ip_blocked && $save) {
				$stat = $this->insertData(TRUE);
				if ($stat) {
					if (DB_CONNECT === 'mysqli') {
						$this->setId(mysqli_insert_id($this->dbObj->getDb()));
					} else {
						$this->setId(mysql_insert_id());
					}
					if (defined('COMMENT_MAIL_RESTRICT') && COMMENT_MAIL_RESTRICT) {
						if ($this->getDisplay() === '1') {
							$this->emailNotify($this->getTbBlogName(), TRUE);
						}
					} else {
						$this->emailNotify($this->getTbBlogName(), TRUE);
					}
				} else {
					$tb_msg = $this->langObj->get('TB_DB_ERROR');
				}
				$this->setCommentExists($stat);
			}

			$this->setResponse($tb_msg);
			return $stat;
		}

		/**
		 * Validate trackback data before saving
		 *
		 * @return boolean
		 */
		function validateTrackback()
		{
			$stat = TRUE;
			$msg = array();
			$url = $this->getTbUrl();
			$excerpt = $this->getTbExcerpt();

			if ((defined('USE_COMMENTS') && !USE_COMMENTS) ||
				!defined('USE_TRACKBACK') || (defined('USE_TRACKBACK') && !USE_TRACKBACK) ||
				!$this->commentsEnabled($this->getEid())) {
				$this->setResponse($this->langObj->get('TB_DISABLED'));
				return FALSE;
			}

			if ($stat && $this->getEid() == -1) {
				$stat = FALSE;
				$msg[] = $this->langObj->get('ENTRY_ID');
			}
			if ($stat && empty($url)) {
				$stat = FALSE;
				$msg[] = $this->langObj->get('URL');
			}
			$this->setResponse($this->langObj->get('TB_INVALID_DATA') . ': ' . implode(', ', $msg));

			return $stat;
		}

		/**
		 * Toggle trackback display
		 *
		 * @param boolean $on Display on
		 * @return boolean
		 */
		function toggleDisplay($on)
		{
			$on = ($on) ? '1' : '0';

			$query = "UPDATE {$this->prefix}comments
						SET
							display='%s',
							timestamp=timestamp
						WHERE
							cid='%i'";
			$query_data = array($on, $this->getId());
			return $this->dbObj->safeQuery($query, $query_data);
			echo '<pre>';
			print_r($this->dbObj->getQuery());
			echo '</pre>';
			exit;

		}

		/**
		 * Parse out the trackback result message from the full XML.
		 *
		 * @return string
		 */
		function parseResult()
		{
			$msg = '';
			$res = explode("[\r]\n", $this->getResult());
			//--- SPL Support
			if ($this->getSpl()) {
				foreach (new ArrayObject($res) as $line) {
					preg_match('/<message>(.*)<\/message>/', $line, $match);
					if (!empty($match[1])) {
						$msg = $match[1];
						continue;
					}
					preg_match('/<title>(.*)<\/title>/i', $line, $match);
					if (!empty($match[1])) {
						$msg = $match[1];
						continue;
					}
				}
			} else {
				foreach ($res as $line) {
					preg_match('/<message>(.*)<\/message>/', $line, $match);
					if (!empty($match[1])) {
						$msg = $match[1];
						continue;
					}
					preg_match('/<title>(.*)<\/title>/i', $line, $match);
					if (!empty($match[1])) {
						$msg = $match[1];
						continue;
					}
				}
			}

			return $msg;
		}

	} //end of class
?>