<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-2005  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require_once(LIB_PATH . 'PbBase_class.php');

	class PbUser extends PbBase
	{
		//--- Class Variables
		var $user;
		var $password;
		var $email;
		var $hint;
		var $pop3;
		var $pop3_alt_email;
		var $post_by;
		var $post_entry;
		var $post_comment;
		var $admin;
		var $timestamp;
		var $user_exists;



		//-----------------------------------------------
		//--- CONSTRUCTOR
		//--- PURPOSE:
		//---   Create blog user object
		//--- PARAMETERS: table prefix (optional)
		//---					user id (optional)
		//--- RETURNS: nothing
		//-----------------------------------------------
		function PbUser($prefix = 'wl_', $id = '')
		{

			$this->PbBase($prefix);

			if (is_int($id)) {
				//--- Retrieve Data
				$this->getUserData($id);
			}

		}


		//-----------------------------------------------
		//-----------------------------------------------
		//--- SET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: setUserExists
		//--- PURPOSE: flag value indicates if user
		//---				data exists for the given ID
		//--- PARAMETERS: TRUE/FALSE
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setUserExists($a)
		{
			$this->user_exists = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setUser
		//--- PURPOSE: set user name
		//--- PARAMETERS: user name
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setUser($a)
		{
			$this->user = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPassword
		//--- PURPOSE: set user password
		//--- PARAMETERS: password
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPassword($a)
		{
			$this->password = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setEmail
		//--- PURPOSE: set user email
		//--- PARAMETERS: email address
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setEmail($a)
		{
			$this->email = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setHint
		//--- PURPOSE: set user password hint
		//--- PARAMETERS: hint phrase
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setHint($a)
		{
			$this->hint = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPop3
		//--- PURPOSE: flag allow posting entries via POP3
		//--- PARAMETERS: name
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPop3($a)
		{
			$this->pop3 = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPop3AltEmail
		//--- PURPOSE: CSV of other email addresss with
		//---          POP3 access to post messages
		//--- PARAMETERS: name
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPop3AltEmail($a)
		{
			$this->pop3_alt_email = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPostBy
		//--- PURPOSE: set entry posted by name
		//--- PARAMETERS: name
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPostBy($a)
		{
			$this->post_by = $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPostEntry
		//--- PURPOSE: flag allow posting entries
		//--- PARAMETERS: TRUE/FALSE
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPostEntry($a)
		{
			$this->post_entry = (bool) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setPostComment
		//--- PURPOSE: flag allow posting comments
		//--- PARAMETERS: TRUE/FALSE
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setPostComment($a)
		{
			$this->post_comment = (bool) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setAdmin
		//--- PURPOSE: flag allow admin access
		//--- PARAMETERS: TRUE/FALSE
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setAdmin($a)
		{
			$this->admin = (bool) $a;
		}

		//-----------------------------------------------
		//--- METHOD: setTimestamp
		//--- PURPOSE: user creation timestamp
		//--- PARAMETERS: timestamp
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setTimestamp($a)
		{
			$this->timestamp = $a;
		}







		//-----------------------------------------------
		//-----------------------------------------------
		//--- GET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: getUserData
		//--- PURPOSE: gets User data
		//--- PARAMETERS: id (optional)
		//--- RETURNS: TRUE on success
		//---          FALSE of failure
		//-----------------------------------------------
		function getUserData($id = '')
		{
			//--- Check for ID
			if (!isset($id)) {
				$id = $this->getId();
			} else {
				$this->setId($id);
			}

			//--- Query
			$query = "SELECT
							user,
							email,
							hint,
							post_by,
							post_entry,
							post_comment,
							admin,
							DATE_FORMAT(timestamp, '%s') AS timestamp,
							pop3,
							pop3_alt_email
						 FROM
					 		{$this->prefix}users
					 	 WHERE
					 	 	id='%i'";
			$query_data = array(COMMENT_DATE_FORMAT, $id);

			//--- Issue Query
			if ($this->dbObj->safeQuery($query, $query_data)) {
				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {
					//--- Retrieve Results
					while ($this->dbObj->getRow()) {
						$this->setUser($this->dbObj->row['user']);
						$this->setEmail($this->dbObj->row['email']);
						$this->setHint($this->dbObj->row['hint']);
						$this->setPostBy($this->dbObj->row['post_by']);
						$this->setPostEntry($this->dbObj->row['post_entry']);
						$this->setPostComment($this->dbObj->row['post_comment']);
						$this->setAdmin($this->dbObj->row['admin']);
						$this->setTimestamp($this->dbObj->row['timestamp']);
						$this->setPop3($this->dbObj->row['pop3']);
						$this->setPop3AltEmail($this->dbObj->row['pop3_alt_email']);
					}
					$this->setUserExists(TRUE);
				} else {
					$this->setUserExists(FALSE);
				}
				$stat = TRUE;
			} else {
				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			return $stat;
		}


		//-----------------------------------------------
		//--- METHOD: getUser
		//--- PURPOSE: get user name
		//--- PARAMETERS: none
		//--- RETURNS: user name
		//-----------------------------------------------
		function getUser()
		{
			return $this->user;
		}

		//-----------------------------------------------
		//--- METHOD: getPassword
		//--- PURPOSE: get user password
		//--- PARAMETERS: none
		//--- RETURNS: user password
		//-----------------------------------------------
		function getPassword()
		{
			return $this->password;
		}

		//-----------------------------------------------
		//--- METHOD: getEmail
		//--- PURPOSE: get user email address
		//--- PARAMETERS: none
		//--- RETURNS: email address
		//-----------------------------------------------
		function getEmail()
		{
			return $this->email;
		}

		//-----------------------------------------------
		//--- METHOD: getHint
		//--- PURPOSE: get user password hint
		//--- PARAMETERS: none
		//--- RETURNS: password hint
		//-----------------------------------------------
		function getHint()
		{
			return $this->hint;
		}

		//-----------------------------------------------
		//--- METHOD: getPop3
		//--- PURPOSE: get entry posting by POP3 flag
		//--- PARAMETERS: none
		//--- RETURNS: flag
		//-----------------------------------------------
		function getPop3()
		{
			return $this->pop3;
		}

		//-----------------------------------------------
		//--- METHOD: getPop3AltEmail
		//--- PURPOSE: CSV of other email addresss with
		//---          POP3 access to post messages
		//--- PARAMETERS: none
		//--- RETURNS: CSV string
		//-----------------------------------------------
		function getPop3AltEmail()
		{
			return $this->pop3_alt_email;
		}

		//-----------------------------------------------
		//--- METHOD: getPostBy
		//--- PURPOSE: get user entry posted by name
		//--- PARAMETERS: none
		//--- RETURNS: posted by name
		//-----------------------------------------------
		function getPostBy()
		{
			return $this->post_by;
		}

		//-----------------------------------------------
		//--- METHOD: getPostEntry
		//--- PURPOSE: get entry posting flag
		//--- PARAMETERS: none
		//--- RETURNS: flag
		//-----------------------------------------------
		function getPostEntry()
		{
			return $this->post_entry;
		}

		//-----------------------------------------------
		//--- METHOD: getPostComment
		//--- PURPOSE: get entry comment flag
		//--- PARAMETERS: none
		//--- RETURNS: flag
		//-----------------------------------------------
		function getPostComment()
		{
			return $this->post_comment;
		}

		//-----------------------------------------------
		//--- METHOD: getAdmin
		//--- PURPOSE: get admin flag
		//--- PARAMETERS: none
		//--- RETURNS: flag
		//-----------------------------------------------
		function getAdmin()
		{
			return $this->admin;
		}

		//-----------------------------------------------
		//--- METHOD: getTimestamp
		//--- PURPOSE: get creation timestamp
		//--- PARAMETERS: none
		//--- RETURNS: timestamp
		//-----------------------------------------------
		function getTimestamp()
		{
			return $this->timestamp;
		}

		//-----------------------------------------------
		//--- METHOD: getUserExists
		//--- PURPOSE: flag value indicates if user
		//---				data exists for the given ID
		//--- PARAMETERS: none
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function getUserExists()
		{
			return $this->user_exists;
		}




		//-----------------------------------------------
		//-----------------------------------------------
		//--- OTHER METHODS -----------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: updateUser
		//--- PURPOSE: update user in the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function updateUser()
		{
			if (!$this->getId() > 0) {
				return FALSE;
			}

			$post_entry = ($this->getPostEntry()) ? 1 : 0;
			$post_comment = ($this->getPostComment()) ? 1 : 0;
			$admin = ($this->getAdmin()) ? 1 : 0;
			$password = (strlen($this->getPassword()) > 0) ? "password=MD5(MD5('" . $this->getPassword() . "'))," : '';

			$query = "UPDATE
					 		{$this->prefix}users
						SET
							user='%s',
							$password
							email='%s',
							hint='%s',
							post_by='%s',
							post_entry='%i',
							post_comment='%i',
							admin='%i',
							pop3='%i',
							pop3_alt_email='%s',
							timestamp=timestamp
						WHERE
							id='%i'";
			$query_data = array($this->getUser(), $this->getEmail(), $this->getHint(),
									  $this->getPostBy(), $post_entry, $post_comment,
									  $admin, $this->getPop3(), $this->getPop3AltEmail(), $this->getId()
							  );

			$this->setUserExists(TRUE);

			return $this->writeDb($query, $query_data);

		}

		/**
		 * Update the user password and hint string.
		 *
		 * @return boolean Success status
		 */
		function updateUserPassword()
		{
			$stat = FALSE;

			if ($this->getId() > 0 && strlen($this->getPassword()) > 0) {
				$query = "UPDATE
						 		{$this->prefix}users
							SET
								password=MD5(MD5('%s')),
								hint='%s',
								timestamp=timestamp
							WHERE
								id='%i'";
				$query_data = array($this->getPassword(), $this->getHint(), $this->getId());
				$stat = $this->writeDb($query, $query_data);
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: insertUser
		//--- PURPOSE: insert user in the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function insertUser()
		{

			$post_entry = ($this->getPostEntry()) ? 1 : 0;
			$post_comment = ($this->getPostComment()) ? 1 : 0;
			$admin = ($this->getAdmin()) ? 1 : 0;
			$pop3 = ($this->getPop3()) ? 1 : 0;

			$query = "INSERT INTO
							{$this->prefix}users
							(
								user,
								password,
								email,
								hint,
								post_by,
								post_entry,
								post_comment,
								admin,
								timestamp,
								pop3,
								pop3_alt_email
							)
						VALUES
							(
							 '%s',
							 MD5(MD5('" . $this->getPassword() . "')),
							 '%s',
							 '%s',
							 '%s',
							 '%i',
							 '%i',
							 '%i',
							 NOW(),
							 '%i',
							 '%s'
							)";
			$query_data = array($this->getUser(), $this->getEmail(), $this->getHint(),
								$this->getPostBy(), $post_entry, $post_comment, $admin,
								$pop3, $this->getPop3AltEmail()
							  );

			$stat = $this->writeDb($query, $query_data);
			if ($stat) {
				if (DB_CONNECT === 'mysqli') {
					$this->setId(mysqli_insert_id($this->dbObj->getDb()));
				} else {
					$this->setId(mysql_insert_id());
				}
				$this->setUserExists($stat);
			}
			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: deleteUser
		//--- PURPOSE: update entry data in the database
		//--- PARAMETERS: none
		//--- RETURNS: TRUE on success
		//---          FALSE on failure
		//-----------------------------------------------
		function deleteUser()
		{
			if (!$this->getId() > 0) {
				return FALSE;
			}

			$query = "DELETE FROM
					 		{$this->prefix}users
						WHERE
							id='%i'";
			$query_data = array($this->getId());

			$stat = $this->writeDb($query, $query_data);
			$this->setUserExists($stat);
			return $stat;
		}

	} //end of class
?>