<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-2008  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require_once(LIB_PATH . 'PbBase_class.php');
	require_once(LIB_PATH . 'PbUser_class.php');
	require_once(LIB_PATH . 'PbTopic_class.php');
	require_once(LIB_PATH . 'PbEntry_class.php');
	require_once(LIB_PATH . 'Localization_class.php');
	require_once(LIB_PATH . 'Localization_class.php');
	require_once(LIB_PATH . 'PbCalendar_class.php');
	if (defined('USE_GIGGLE') && USE_GIGGLE) {
		require_once(GIGGLE_LIB_PATH . 'phpgiggle.inc.php');
	}

	class PolarBlog extends PbBase
	{
		//--- Class Variables
		/**
		 * Comments that have display disabled.
		 * @var
		 */
		var $comments_no_display;

		var $entries;
		var $entry_cnt;
		var $users;
		var $topics;
		var $display;
		var $accessory_display;
		var $display_cnt;
		var $index_mode;
		var $separator_date;
		var $comment_indent;
		/**
		 * New Entry ID
		 * @var int
		 */
		var $new_eid;

		var $rblObj;
		var $rbl_services;



		//-----------------------------------------------
		//--- CONSTRUCTOR
		//--- PURPOSE:
		//---   Create blog entry object
		//--- PARAMETERS: table prefix
		//---			  RBL Services
		//--- RETURNS: nothing
		//-----------------------------------------------
		function PolarBlog($prefix = 'wl_', $rbl_services = array())
		{
			$this->PbBase($prefix);
			$this->setRblServices($rbl_services);
			$this->entries = array();
			$this->topics = array();
			$this->setTopics();
			$this->langObj = new Localization(I18N_LANG, './lang/');
		}


		//-----------------------------------------------
		//-----------------------------------------------
		//--- SET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: setCommentIndent
		//--- PURPOSE: set the comment indent value
		//--- PARAMETERS: indent in pixels
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setCommentIndent($a)
		{
			$this->comment_indent = $a;
		}

		/**
		 * Retrieve information on comments that have the display disabled.
		 *
		 * @return array
		 */
		function setCommentsNoDisplay()
		{
			$this->comments_no_display = array();
			$query = "SELECT
							cid,
							eid,
							name,
							SUBSTRING(comment,1,30) as excerpt,
							trackback,
							timestamp,
							ip
						 FROM
					 		{$this->prefix}comments
					 	 WHERE
					 	 	display='0'
					 	 ORDER BY
					 	 	eid,
					 	 	cid,
					 	 	timestamp";
			if ($this->dbObj->query($query)) {
				$stat = TRUE;
				if ($this->dbObj->getResultCnt() > 0) {
					//--- Retrieve Results
					while ($this->dbObj->getRow()) {
						$eid = (int) $this->dbObj->row['eid'];
						$this->comments_no_display[] = $this->dbObj->row;
					}
				}
			} else {
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setDisplayCnt
		//--- PURPOSE: set entries to display
		//--- PARAMETERS: count (optional)
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setDisplayCnt($cnt = -1)
		{
			if ($cnt == -1) {
				$this->display_cnt = DEFAULT_DISPLAY_CNT;
			} else {
				$this->display_cnt = $cnt;
			}
		}

		//-----------------------------------------------
		//--- METHOD: setDisplay
		//--- PURPOSE: set output display
		//--- PARAMETERS: start id
		//--- 				entry id (optional);
		//--- 				entry date (optional);
		//--- 				topic id (optional);
		//--- 				search string (optional);
		//--- 				footer_enabled (optional);
		//--- 				comment mode (optional);
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function setDisplay($start = 0, $eid = FALSE, $date = FALSE, $tid = FALSE, $search = FALSE, $footer_enabled = TRUE, $comment_mode = FALSE)
		{
			$this->setDisplayCnt();
			$this->index_mode = FALSE;

			//--- Display Single Entry
			$full_entry = FALSE;
			if ($tid !== FALSE  && is_int($tid)) {
				//--- Topic Specific Entries
				$stat = $this->setEntries($start, FALSE, $tid);
			} elseif (is_int($eid)) {
				$this->entries[0] = new PbEntry($this->getDbPrefix(), $eid);
				$full_entry = TRUE;
				$stat = ($this->entries[0] === 'Object') ? TRUE : FALSE;
			} elseif (is_int($date)) {
				//--- Date Specific Entries
				$stat = $this->setEntries($date, TRUE);
			} elseif (strlen($search) > 0) {
				//--- Search Entries
				$stat = $this->searchEntries($search);
				$stat = (isset($this->entries[0]) && $this->entries[0] === 'Object') ? TRUE : FALSE;
			} else {
				//--- Default: Display Last x Entries
				if ($start == 0) {
					$stat = $this->setEntries();
				} else {
					//--- Begin at offset
					$stat = $this->setEntries($start);
				}
			}

			//--- Build Display
			if (count($this->entries) > 0) {
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($this->entries) as $entry) {
						if ($entry->getEntryExists()) {
							if (defined('USE_TIMER') && USE_TIMER) {
								require_once(LIB_PATH . 'Timer.php');
								$timer = new Timer();
								$etime_total = (!isset($etime_total)) ? 0 : $etime_total;
							}
							$this->display .= $this->buildEntry($entry, $full_entry, $footer_enabled);
							if (defined('USE_TIMER') && USE_TIMER) {
								$proc_time = $timer->getPageTime();
								$etime_total += $timer->getPageTime('raw');
								$this->display .= '<!-- Entry Time: ' . $proc_time . " -->\n";
								$this->display .= '<!-- Cumulative Entry Time: ' . round($etime_total * 1000, 3) . " msecs -->\n";
							}

							//--- Include Comments
							if ($full_entry && !$comment_mode) {
								$this->display .= '<div><a name="comments" id="comments"></a></div>';
								$this->display .= $this->buildComments($entry);
							}
						} elseif (count($this->entries) && $entry->getId()) {
							$this->display = '<div class="pb_entry">';
							$this->display .= '<p class="pb_body">' . $this->langObj->get('NO_ENTRY_DATA') . " $eid.<br /></p>";
							$this->display .= '</div>';
						} else {
							$this->display = '<div class="pb_entry">';
							$this->display .= '<p class="pb_body">' . $this->langObj->get('NO_ENTRY_ID') . " $eid.<br /></p>";
							$this->display .= '</div>';
						}
					}
				} else {
					foreach ($this->entries as $entry) {
						if ($entry->getEntryExists()) {
							if (defined('USE_TIMER') && USE_TIMER) {
								require_once(LIB_PATH . 'Timer.php');
								$timer = new Timer();
								$etime_total = (!isset($etime_total)) ? 0 : $etime_total;
							}
							$this->display .= $this->buildEntry($entry, $full_entry, $footer_enabled);
							if (defined('USE_TIMER') && USE_TIMER) {
								$proc_time = $timer->getPageTime();
								$etime_total += $timer->getPageTime('raw');
								$this->display .= '<!-- Entry Time: ' . $proc_time . " -->\n";
								$this->display .= '<!-- Cumulative Entry Time: ' . round($etime_total * 1000, 3) . " msecs -->\n";
							}

							//--- Include Comments
							if ($full_entry && !$comment_mode) {
								$this->display .= '<div><a name="comments" id="comments"></a></div>';
								$this->display .= $this->buildComments($entry);
							}
						} elseif (count($this->entries) && $entry->getId()) {
							$this->display = '<div class="pb_entry">';
							$this->display .= '<p class="pb_body">' . $this->langObj->get('NO_ENTRY_DATA') . " $eid.<br /></p>";
							$this->display .= '</div>';
						} else {
							$this->display = '<div class="pb_entry">';
							$this->display .= '<p class="pb_body">' . $this->langObj->get('NO_ENTRY_ID') . " $eid.<br /></p>";
							$this->display .= '</div>';
						}
					}
				}
			} elseif (count($this->entries) == 0 && strlen($search) > 0) {
				$this->display = '<div class="pb_entry">';
				$this->display .= '<p class="pb_body">' . $this->langObj->get('NO_SEARCH_RESULT') . ':<br /><br />';
				$this->display .= '<span class="pb_search_fail">' . htmlentities(stripslashes($search), ENT_QUOTES, I18N_CHARSET) . '</span>';
				$this->display .= '<br /><br />' . $this->langObj->get('SEARCH_AGAIN') . '</p>';
				$this->display .= '</div>';
			} else {
				$this->display = '<div class="pb_entry">';
				$this->display .= '<p class="pb_body">' . $this->langObj->get('NO_ENTRIES') . '<br /></p>';
				$this->display .= '</div>';
			}
			if ($footer_enabled) {
			$this->display .= $this->buildNavBar($start, $search, $tid);
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setAccessoryDisplay
		//--- PURPOSE: set calendar & header output display
		//--- PARAMETERS: calendar month (optional)
		//--- 				calendar year (optional);
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setAccessoryDisplay($month, $year)
		{

			if (defined('CSS') && USE_NAV) {
				$nav_file = file(NAV_FILE);
				$this->accessory_display .= '<div id="Nav">';
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($nav_file) as $line) {
						$this->accessory_display .= $line;
					}
				} else {
					foreach ($nav_file as $line) {
						$this->accessory_display .= $line;
					}
				}
				$this->accessory_display .= '</div>';
			}

			if (defined('TOP_HEADER')) {
				$this->accessory_display .= '<div id="PbTopHeader" class="pb_nav-bar">';
				$this->accessory_display .= TOP_HEADER;
				$this->accessory_display .= '</div>';
			}

			if (defined('SUB_HEADER')) {
				$this->accessory_display .= '<div id="PbSubHeader" class="pb_sh_h1">';
				$this->accessory_display .= SUB_HEADER;
				$this->accessory_display .= '</div>';
			}
			if (defined('BLOG_TAGLINE')) {
				$this->accessory_display .= '<div class="pb_tagline">';
				$this->accessory_display .= BLOG_TAGLINE;
				$this->accessory_display .= '</div>';
			}
			if (defined('USE_TIMER') && USE_TIMER) {
				require_once(LIB_PATH . 'Timer.php');
				$timer = new Timer();
			}
			$this->accessory_display .= '<div id="PbCalendar" class="pb_nav-bar">';
			$this->accessory_display .= $this->buildCalendar($year, $month);
			$this->accessory_display .= '<hr />';

			if (isset($_GET['eid']) || isset($_GET['index'])) {
				$this->accessory_display .= $this->buildLink(BASE_URL, $this->langObj->get('ENTRY_LINK'), '[' . $this->langObj->get('ENTRY_LINK'). ']');
			}
			if (isset($_GET['eid'])) {
				$this->accessory_display .= '<br />';
			}
			if (!$this->index_mode) {
				$this->accessory_display .= $this->buildLink('?index=1', $this->langObj->get('ARCHIVE_LINK'), '[' . $this->langObj->get('ARCHIVE_LINK') . ']');
			}

			$this->accessory_display .= '<br />';
			$this->accessory_display .= '<hr />';

			$this->accessory_display .= $this->setAdminMenu();
			$this->accessory_display .= $this->setRecentLinks();
			$this->accessory_display .= $this->setTopicLinks();
			$this->accessory_display .= $this->setOtherBlogs();
			$this->accessory_display .= $this->setTagLinks();

			$this->accessory_display .= "</div>\n";
			if (defined('USE_TIMER') && USE_TIMER) {
				$proc_time = $timer->getPageTime();
				$this->accessory_display .= '<!-- Calendar Time: ' . $proc_time . " -->\n";
			}
		}

		//-----------------------------------------------
		//--- METHOD: setAdminMenu
		//--- PURPOSE: set administrative links
		//--- PARAMETERS: none
		//--- RETURNS: XHTML string
		//-----------------------------------------------
		function setAdminMenu()
		{
			$links = '';
			if (defined('USE_LOGIN_LINK') && USE_LOGIN_LINK && !(isset($_SESSION['login']) && $_SESSION['login'])) {
				$links .= $this->langObj->get('ADMINISTRATION');
				$links .= '<br />';
				$links .= $this->buildLink('?login=1', $this->langObj->get('PB_LOGIN'), '[' . $this->langObj->get('LOGIN') . ']');
				$links .= '<hr />';
			} elseif (isset($_SESSION['login']) && $_SESSION['login']) {
				$links .= $this->langObj->get('ADMINISTRATION');
				$links .= '<br />';
				if (isset($_SESSION['post_entry']) && $_SESSION['post_entry']) {
					$links .= '&nbsp;' . $this->buildLink('?admin=add', $this->langObj->get('ADD_NEW_ENTRY'), '[' . $this->langObj->get('ADD_ENTRY') .']') . '<br />';
				}
				$links .= '&nbsp;' . $this->buildLink('?change_pw=1', $this->langObj->get('CHANGE_PASSWORD'), '[' . $this->langObj->get('CHANGE_PASSWORD') . ']') . '<br />';
				if (isset($_SESSION['admin']) && $_SESSION['admin']) {
					$links .= '&nbsp;' . $this->buildLink('?admin=topics', $this->langObj->get('EDIT_AVAIL_TOPIC'), '[' . $this->langObj->get('EDIT_TOPICS') .']') . '<br />';
					$links .= '&nbsp;' . $this->buildLink('?admin=users', $this->langObj->get('EDIT_USERS'), '[' . $this->langObj->get('EDIT_USERS') . ']') . '<br />';
					if (defined('USE_RBL') && USE_RBL) {
						$links .= '&nbsp;' . $this->buildLink('?admin=rbl_admin', $this->langObj->get('RBL_ADMIN'), '[' . $this->langObj->get('RBL_ADMIN') . ']') . '<br />';
					}
				}
				$links .= '&nbsp;' . $this->buildLink('?logout=1', $this->langObj->get('LOGOUT'), '[' . $this->langObj->get('LOGOUT') . ']') . '<br />';
				$links .= '<hr />';
			}

			return $links;
		}

		//-----------------------------------------------
		//--- METHOD: setTagLinks
		//--- PURPOSE: set tag list links
		//--- PARAMETERS: none
		//--- RETURNS: XHTML string
		//-----------------------------------------------
		function setTagLinks()
		{
			global $config;

			if (!defined('USE_TAG_GROUPS') || USE_TAG_GROUPS == FALSE) { return; }

			$links = '';
			$cnt = 0;
			$group_cnt = count($config['tags']);
			//--- SPL Support
			if ($this->getSpl()) {
				foreach (new ArrayObject($config['tags']) as $group) {
					$cnt++;
					foreach(new ArrayObject($group) as $tag) {
						$img = '<img class="pb_no_border" src="' . $tag['image'] . '" alt="' . $tag['title'] . '" />';
						$links .= $this->buildLink($tag['url'], $tag['title'], $img);
						$links .= '<br />';
					}
					if ($cnt < $group_cnt) {
						$links .= '<hr />';
					}
				}
			} else {
				foreach ($config['tags'] as $group) {
					$cnt++;
					foreach($group as $tag) {
						$img = '<img class="pb_no_border" src="' . $tag['image'] . '" alt="' . $tag['title'] . '" />';
						$links .= $this->buildLink($tag['url'], $tag['title'], $img);
						$links .= '<br />';
					}
					if ($cnt < $group_cnt) {
						$links .= '<hr />';
					}
				}
			}

			return $links;
		}

		//-----------------------------------------------
		//--- METHOD: setOtherBlogs
		//--- PURPOSE: set links to other blogs
		//--- PARAMETERS: none
		//--- RETURNS: XHTML string
		//-----------------------------------------------
		function setOtherBlogs()
		{
			global $config;

			if (!defined('USE_OTHER_BLOGS') || USE_OTHER_BLOGS == FALSE) { return; }

			if (defined('OTHER_BLOGS_TITLE') && OTHER_BLOGS_TITLE) {
				$links = OTHER_BLOGS_TITLE;
				$links .= '<ul>';
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($config['blogs']) as $blog) {
						$links .= '<li> ' . $this->buildLink($blog['url'], $blog['title'] . ' - ' . $blog['desc'], $blog['title']) . '</li>';
					}
				} else {
					foreach ($config['blogs'] as $blog) {
						$links .= '<li> ' . $this->buildLink($blog['url'], $blog['title'] . ' - ' . $blog['desc'], $blog['title']) . '</li>';
					}
				}
				$links .= '</ul>';
			} else {
				$links = '';
				$current_section = '';
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($config['blogs']) as $section => $blogs) {
						if ($section != $current_section) {
							$links .= $section;
							$current_section = $section;
						}

						$links .= '<ul>';
						foreach (new ArrayObject($blogs) as $blog) {
							$links .= '<li> ' . $this->buildLink($blog['url'], $blog['title'] . ' - ' . $blog['desc'], $blog['title']) . '</li>';
						}
						$links .= '</ul>';
					}
				} else {
					foreach ($config['blogs'] as $section => $blogs) {
						if ($section != $current_section) {
							$links .= $section;
							$current_section = $section;
						}

						$links .= '<ul>';
						foreach ($blogs as $blog) {
							$links .= '<li> ' . $this->buildLink($blog['url'], $blog['title'] . ' - ' . $blog['desc'], $blog['title']) . '</li>';
						}
						$links .= '</ul>';
					}
				}
			}
			$links .= '<hr />';

			return $links;
		}

		//-----------------------------------------------
		//--- METHOD: setTopicLinks
		//--- PURPOSE: set links to blog topics
		//--- PARAMETERS: none
		//--- RETURNS: XHTML string
		//-----------------------------------------------
		function setTopicLinks()
		{
			if (!defined('USE_TOPIC_LINKS') || USE_TOPIC_LINKS == FALSE) { return; }

			$rss_url = '';
			$links = $this->langObj->get('TOPICS');
			$links .= '<ul>';
			//--- SPL Support
			if ($this->getSpl()) {
				foreach (new ArrayObject($this->topics) as $topicObj) {
					if (DEFINED('USE_RSS_TOPICS') && USE_RSS_TOPICS) {
						$rss_txt = $this->langObj->get('RSS_FEED') . ': ' . $topicObj->getName();
						$rss_img = '<img class="no_border" src="' . BASE_URL . 'gfx/rss_sm.gif" alt="' . $rss_txt . '"' . ' />';
						$rss_url = $this->buildLink(BASE_URL . 'rss.php?tid=' . $topicObj->getId(), $rss_txt, $rss_img) . '&nbsp;';
					}
					$links .= '<li>' . $rss_url . $this->buildLink(BASE_URL . '?tid=' . $topicObj->getId(), $this->langObj->get('TOPIC') . ': ' . $topicObj->getName(), $topicObj->getName()) . '</li>';
				}
			} else {
				foreach ($this->topics as $topicObj) {
					if (DEFINED('USE_RSS_TOPICS') && USE_RSS_TOPICS) {
						$rss_txt = $this->langObj->get('RSS_FEED') . ': ' . $topicObj->getName();
						$rss_img = '<img class="no_border" src="' . BASE_URL . 'gfx/rss_sm.gif" alt="' . $rss_txt . '"' . ' />';
						$rss_url = $this->buildLink(BASE_URL . 'rss.php?tid=' . $topicObj->getId(), $rss_txt, $rss_img) . '&nbsp;';
					}
					$links .= '<li>' . $rss_url  . $this->buildLink(BASE_URL . '?tid=' . $topicObj->getId(), $this->langObj->get('TOPIC') . ': ' . $topicObj->getName(), $topicObj->getName()) . '</li>';
				}
			}
			$links .= '</ul>';
			$links .= '<hr />';
			unset($this->topics);

			return $links;
		}

		//-----------------------------------------------
		//--- METHOD: setRecentLinks
		//--- PURPOSE: set recent story links
		//--- PARAMETERS: none
		//--- RETURNS: XHTML string
		//-----------------------------------------------
		function setRecentLinks()
		{
			if (!defined('DEFAULT_RECENT_CNT') || DEFAULT_RECENT_CNT == 0) { return; }

			$entries = array();
			$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? " WHERE display='1'" : '';
			$post_dated = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? 'AND updated<=UNIX_TIMESTAMP()' : '';
			$limit = DEFAULT_RECENT_CNT;
			$query = "SELECT
							eid
						 FROM
					 		{$this->prefix}entries
					 	 $display_flag
					 	 $post_dated
					 	 ORDER BY
					 	 	updated
					 	 DESC
					 	 LIMIT $limit";

			//--- Issue Query
			if ($this->dbObj->query($query)) {

				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {
					//--- Retrieve Results
					$i = 0;
					while ($this->dbObj->getRow()) {
						$eid = (int) $this->dbObj->row['eid'];
						$entries[$i] = new PbEntry($this->getDbPrefix(), $eid);
						$i++;
					}
				}

				$stat = TRUE;

			} else {
				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			$links = '';
			if ($stat) {
				$links .= $this->langObj->get('RECENT_ENTRIES');
				$links .= '<ul>';
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($entries) as $entry) {
						if (isset($this->topics[$entry->getTopicId()])) {
							$topicObj = $this->topics[$entry->getTopicId()];
							$topic = $topicObj->getName();
							$links .= '<li> ' . $this->buildLink('?eid=' . $entry->getId(), $entry->getUpdated(TRUE) . ' - ' . $topic . ': ' . $entry->getTitle(), $entry->getTitle()) . '</li>';
						}
					}
				} else {
					foreach ($entries as $entry) {
						if (isset($this->topics[$entry->getTopicId()])) {
							$topicObj = $this->topics[$entry->getTopicId()];
							$topic = $topicObj->getName();
							$links .= '<li> ' . $this->buildLink('?eid=' . $entry->getId(), $entry->getUpdated(TRUE) . ' - ' . $topic . ': ' . $entry->getTitle(), $entry->getTitle()) . '</li>';
						}
					}
				}
				$links .= '</ul>';
				$links .= '<hr />';
				unset($entries);
			}

			return $links;
		}

		//-----------------------------------------------
		//--- METHOD: setIndexDisplay
		//--- PURPOSE: set archive index output display
		//--- PARAMETERS: index start id
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function setIndexDisplay($start = 0)
		{
			if (DB_CONNECT === 'mysqli') {
				//Crappy coding has led to the exhaustion of available connections.
				//Scale back the display to avoid this problem.
				$t_cnt = count($this->topics) + 20;
				if ($t_cnt + DEFAULT_ARCHIVE_CNT >= 100) {
					$this->setDisplayCnt(DEFAULT_ARCHIVE_CNT - $t_cnt);
				} else {
					$this->setDisplayCnt(DEFAULT_ARCHIVE_CNT - $t_cnt);
				}
			} else {
				$this->setDisplayCnt(DEFAULT_ARCHIVE_CNT);
			}
			$this->index_mode = TRUE;

			//--- Display Index
			$stat = $this->setEntries($start);
			//--- Build Display
			/* PAGING
			if (count($this->entries) > 0 && strlen($search) > 0) {
				for ($i = 0; $i < $this->getEntryCnt(); $i++) {
					$this->display .= $this->buildEntry($this->entries[$i], $full_entry);
				}
			} else
			*/

			if (count($this->entries) > 0) {
				$search = FALSE; //clear search results from nav bar so display recent selected works after a search.
				$this->display .= $this->buildIndex();
			} else {
				$this->display = '<div class="pb_entry">';
				$this->display .= '<p class="pb_body">' . $this->langObj->get('NO_ENTRIES') . '</p>';
				$this->display .= '</div>';
			}
			$this->display .= $this->buildNavBar($start, FALSE);

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setEditDisplay
		//--- PURPOSE: set edit/add output display
		//--- PARAMETERS: entry id
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------

		function setEditDisplay($eid)
		{

			//--- Edit or Add?
			$this->entries[0] = new PbEntry($this->getDbPrefix(), $eid);
			$stat = ($this->entries[0] === 'Object') ? TRUE : FALSE;

			//--- Set fields
			$dupe_id = md5(uniqid(rand(), true));
			$_SESSION['dupe_id'] = $dupe_id;
			$title = ($eid) ? $this->entries[0]->getTitle() : '';
			$topic_id = ($eid) ? $this->entries[0]->getTopicId() : 0;
//			$body = ($eid) ? htmlentities($this->entries[0]->getBody('DO_STUPEFY'), ENT_QUOTES, I18N_CHARSET) : '';
//			$more = ($eid && $this->entries[0]->getMoreExists()) ? htmlentities($this->entries[0]->getMore('DO_STUPEFY'), ENT_QUOTES, I18N_CHARSET) : '';
			$body = ($eid) ? $this->entries[0]->getBody('DO_STUPEFY') : '';
			$more = ($eid && $this->entries[0]->getMoreExists()) ? $this->entries[0]->getMore('DO_STUPEFY') : '';
			$ping = ((defined('USE_PINGER') && USE_PINGER && !$eid)) ? ' checked="checked"' : '';
			$trackback = (defined('USE_TRACKBACK') && USE_TRACKBACK) ? $this->entries[0]->getTbUrl() : '';
			$tb_resend = ($eid) ? ' checked="checked"' : '';

			if ($eid && $this->entries[0]->getDisplay()) {
				$display_entry = ' checked="checked"';
			} elseif ($eid && !$this->entries[0]->getDisplay()) {
				$display_entry = '';
			} elseif (defined('DISPLAY_ENTRY_DEFAULT') && DISPLAY_ENTRY_DEFAULT) {
				$display_entry = ' checked="checked"';
			} else {
				$display_entry = '';
			}

			if ($eid && $this->entries[0]->getAllowComments()) {
				$allow_comments = ' checked="checked"';
			} elseif ($eid && !$this->entries[0]->getAllowComments()) {
				$allow_comments = '';
			} else {
				$allow_comments = ' checked="checked"';
			}

			//--- Set Headers
			if (defined('TOP_HEADER')) {
				$this->accessory_display .= '<div id="PbTopHeader" class="pb_nav-bar">';
				$this->accessory_display .= TOP_HEADER;
				$this->accessory_display .= '</div>';
			}
			$this->accessory_display .= '<div id="PbSubHeader" class="pb_sh_h1">';
			$this->accessory_display .= $this->langObj->get('PB_ENTRY');
			$this->accessory_display .= '</div>';

			//--- Set Display
			$this->display = '<div class="pb_edit">';
			$this->display .= '<form action="' . BASE_URL . '" method="post" name="entry_form">' . "\n";
			$this->display .= "\n\t" . '<p class="pb_created"><span class="pb_input_text">' . $this->langObj->get('TOPIC') . ':</span> ';
			// Topic
			if (isset($this->topics) && count($this->topics) > 0) {
				$this->display .= "\t\t<select name=\"tid\">\n";
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($this->topics) as $id => $obj) {
						$sel = ($topic_id == $id) ? ' selected="selected"' : '';
						$this->display .= "\t\t <option value=\"$id\"$sel>" . $obj->getName() . "</option>\n";
					}
				} else {
					foreach ($this->topics as $id => $obj) {
						$sel = ($topic_id == $id) ? ' selected="selected"' : '';
						$this->display .= "\t\t <option value=\"$id\"$sel>" . $obj->getName() . "</option>\n";
					}
				}
				$this->display .= "\t\t</select>\n";
			} else {
				$this->display .= $this->langObj->get('ADD_ONE_TOPIC') . "\n";
			}
			$this->display .= "<br />\n";

			// Title
			$this->display .= "\n\t" . '<span class="pb_input_text">' . $this->langObj->get('TITLE') . ':</span>&nbsp;&nbsp;';
			$this->display .= "\t\t <input type=\"text\" name=\"title\" value=\"$title\" size=\"50\" />\n";
			$this->display .= "<br />\n";
			// Body
			$this->display .= "\t" . '<span class="pb_input_text">' . $this->langObj->get('BODY') . ":</span><br />\n";
			$this->display .= "\t<textarea name=\"body\" rows=\"" . EDIT_BODY_ROWS . '" cols="' . EDIT_COLS . "\">$body</textarea></p>\n";
			// More
			$this->display .= "\t" . '<p class="pb_created"><span class="pb_input_text">' . $this->langObj->get('MORE') . ":</span><br />\n";
			$this->display .= "\t" . '<textarea name="more" rows="' . EDIT_MORE_ROWS . '" cols="' . EDIT_COLS . "\">$more</textarea>\n<br />\n";
			// Trackback
			if (defined('USE_TRACKBACK') && USE_TRACKBACK) {
				$this->display .= "\n\t" . '<span class="pb_input_text">' . $this->langObj->get('TRACKBACK') . ':</span>&nbsp;&nbsp;';
				$this->display .= "\t\t <input type=\"text\" name=\"tb_url\" value=\"$trackback\" size=\"80\" maxlength=\"100\" />\n";
				if ($tb_resend) {
					$this->display .= "\t\t" . '&nbsp;&nbsp;<input type="checkbox" name="tb_resend" value="1" /><span class="pb_input_text">Resend</span>' . "\n";
				}
				$this->display .= "<br />\n";
			}

			// Timestamp & Time Adjust
			if ($eid) {
				//--- Edit Mode
				$button_text = $this->langObj->get('UPDATE_ENTRY');
				$edit_mode = 'update';
				$this->display .= "\t\t" . '<span class="pb_input_text">' . $this->langObj->get('TIMESTAMP') . ':</span>&nbsp;<input type="checkbox" name="create_timestamp" value="1" /><span class="pb_input_text">' . $this->langObj->get('RESET_CREATE') . "</span>\n";
				$this->display .= "\n\t" . '&nbsp;||&nbsp;<input type="checkbox" name="update_timestamp" value="1" /><span class="pb_input_text">' . $this->langObj->get('SET_UPDATE') . "</span>\n";
				$this->display .= "\n\t" . '&nbsp;||&nbsp;<span class="pb_input_text"> ' . $this->langObj->get('ADJUST') . ":</span> \n";
			} else {
				//--- Add Mode
				$button_text = $this->langObj->get('ADD_ENTRY');
				$edit_mode = 'insert';
				$this->display .= "<input type=\"hidden\" name=\"create_timestamp\" value=\"1\" />\n";
				$this->display .= "\n\t" . '<span class="pb_input_text">' . $this->langObj->get('TIMESTAMP_ADJUST') . ":</span> \n";
			}
			$this->display .= "\t\t<input type=\"radio\" name=\"time_adj_dir\" value=\"+\" />+\n";
			$this->display .= "\t\t<input type=\"radio\" name=\"time_adj_dir\" value=\"-\" checked=\"checked\" />-\n";
			$this->display .= "\t\t<select name=\"time_adj_hour\">\n";
			for ($i = 0; $i <= 168; $i++) {
				$this->display .= "\t\t <option value=\"$i\">$i</option>\n";
			}
			$this->display .= "\t\t</select>" . $this->langObj->get('HRS_ABRV') . "\n";
			$this->display .= "\t\t<select name=\"time_adj_min\">\n";
			for ($i = 0; $i <= 59; $i++) {
				$this->display .= "\t\t <option value=\"$i\">$i</option>\n";
			}
			$this->display .= "\t\t</select>" . $this->langObj->get('MINS_ABRV') . "\n";
			$this->display .= "<br />\n";

			//--- Line Breaks, Display & Save
			$this->display .= "\t\t" . '&nbsp;&nbsp;<input type="checkbox" name="linebreak" value="1" checked="checked" /><span class="pb_input_text">' . $this->langObj->get('AUTO_BREAK') . "</span>\n";
			$this->display .= "\t\t" . '&nbsp;&nbsp;<input type="checkbox" name="display" value="1"' . $display_entry . ' /><span class="pb_input_text">' . $this->langObj->get('DISPLAY_ENTRY') . "</span>\n";
			$this->display .= "\t\t" . '&nbsp;&nbsp;<input type="checkbox" name="allow_comments" value="1"' . $allow_comments . ' /><span class="pb_input_text">' . $this->langObj->get('ALLOW_COMMENTS') . "</span>\n";
			if (defined('USE_PINGER') && USE_PINGER) {
				$this->display .= "\t\t" . '&nbsp;&nbsp;<input type="checkbox" name="ping" value="1"' . $ping . ' /><span class="pb_input_text">Ping-O-Matic!' . "</span>\n";
			}
			$this->display .= "<br />\n";

			//--- Buttons
			if (isset($this->topics) && count($this->topics) > 0) {
				$preview_width = PREVIEW_WIDTH;
				$preview_height = PREVIEW_HEIGHT;
				$this->display .= "\t" . '&nbsp;&nbsp;<input type="button" name="preview" value="' . $this->langObj->get('PREVIEW') . "\" onclick=\"entryPreview('entry_preview.php', $preview_width, $preview_height)\" />\n";
				$this->display .= "\t&nbsp;&nbsp;<input type=\"submit\" name=\"edit\" value=\"$button_text\" />\n";
			} else {
				$this->display .= "\t" . '&nbsp;&nbsp;' . $this->langObj->get('NO_TOPIC_NO_POST') . "\n";
			}
			if (defined('USE_SPELLER') && USE_SPELLER) {
				$this->display .= "\t" . '&nbsp;&nbsp;<input type="button" name="spell_check" value="' . $this->langObj->get('CHECK_SPELLING') . '" onClick="openSpellChecker();" />' . "\n";
			}
			$this->display .= "\t" . '&nbsp;&nbsp;<input type="submit" name="cancel" value="' . $this->langObj->get('CANCEL') . '" />' . "\n";
			$this->display .= "\t<input type=\"hidden\" name=\"dupe_id\" value=\"$dupe_id\" />\n";
			$this->display .= "\t<input type=\"hidden\" name=\"eid\" value=\"$eid\" />\n";
			$this->display .= "\t<input type=\"hidden\" name=\"edit_mode\" value=\"$edit_mode\" />\n";
			$this->display .= "<br />\n";
			$this->display .= ' </p></form>' . "\n";
			$this->display .= '</div>' . "\n";

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setEditUserDisplay
		//--- PURPOSE: set user edit output display
		//--- PARAMETERS: user id
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------

		function setEditUserDisplay($id)
		{
			$this->setUsers();

			//--- Set Headers
			if (defined('TOP_HEADER')) {
				$this->accessory_display .= '<div id="PbTopHeader" class="pb_nav-bar">';
				$this->accessory_display .= TOP_HEADER;
				$this->accessory_display .= '</div>';
			}
			$this->accessory_display .= '<div id="PbSubHeader" class="pb_sh_h1">';
			$this->accessory_display .= $this->langObj->get('PB_USER_ADMIN');
			$this->accessory_display .= '</div>';

			//--- Set Display
			$this->display = '<div class="pb_edit">';
			$this->display .= '<form action="' . BASE_URL . '" method="post">' . "\n";

			if ($id) {
				//--- Add/Edit User
				$edit_mode = ($id == -1) ? 'user_insert' : 'user_update';
				$userObj = new PbUser($this->getDbPrefix(), $id);

				$this->display .= "\t<table class=\"pb_created\">\n";
				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('USER') . ":</td>\n";
				$this->display .= "\t\t" . ' <td><input type="text" name="user" size="30" value="' . $userObj->getUser() . "\" /></td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('PASSWORD') . ":</td>\n";
				$this->display .= "\t\t" . ' <td><input type="password" size="30" name="password" value="" /></td>' . "\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('PASSWORD_HINT') . ":</td>\n";
				$this->display .= "\t\t" . ' <td><input type="text" name="hint" size="50" value="' . $userObj->getHint() . "\" /></td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('POSTED_BY') . ":</td>\n";
				$this->display .= "\t\t" . ' <td><input type="text" name="post_by" size="30" value="' . $userObj->getPostBy() . "\" /></td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('EMAIL') . ":</td>\n";
				$this->display .= "\t\t" . ' <td><input type="text" name="email" size="50" value="' . $userObj->getEmail() . "\" /></td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('POP3_ALT_EMAIL') . ":</td>\n";
				$this->display .= "\t\t" . ' <td><input type="text" name="pop3_alt_email" size="50" value="' . $userObj->getPop3AltEmail() . "\" /></td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('ALLOW_POSTS') . "</td>\n";
				$chk_y = ($userObj->getPostEntry()) ? ' checked="checked"' : '';
				$chk_n = (!$userObj->getPostEntry()) ? ' checked="checked"' : '';
				$this->display .= "\t\t <td>\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"post_entry\" value=\"1\"$chk_y />" . $this->langObj->get('YES') . "\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"post_entry\" value=\"0\"$chk_n />" . $this->langObj->get('NO') . "\n";
				$this->display .= "\t\t </td>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('ALLOW_POP3') . "</td>\n";
				$chk_y = ($userObj->getPop3()) ? ' checked="checked"' : '';
				$chk_n = (!$userObj->getPop3()) ? ' checked="checked"' : '';
				$this->display .= "\t\t <td>\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"pop3\" value=\"1\"$chk_y />" . $this->langObj->get('YES') . "\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"pop3\" value=\"0\"$chk_n />" . $this->langObj->get('NO') . "\n";
				$this->display .= "\t\t </td>\n";

				$this->display .= "\t\t</tr>\n";
				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('ALLOW_USER_COMMENTS') . "</td>\n";
				$chk_y = ($userObj->getPostComment()) ? ' checked="checked"' : '';
				$chk_n = (!$userObj->getPostComment()) ? ' checked="checked"' : '';
				$this->display .= "\t\t <td>\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"post_comment\" value=\"1\"$chk_y />" . $this->langObj->get('YES') . "\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"post_comment\" value=\"0\"$chk_n />" . $this->langObj->get('NO') . "\n";
				$this->display .= "\t\t </td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('ADMINISTRATOR') . "</td>\n";
				$chk_y = ($userObj->getAdmin()) ? ' checked="checked"' : '';
				$chk_n = (!$userObj->getAdmin()) ? ' checked="checked"' : '';
				$this->display .= "\t\t <td>\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"user_admin\" value=\"1\"$chk_y />" . $this->langObj->get('YES') . "\n";
				$this->display .= "\t\t \t<input type=\"radio\" name=\"user_admin\" value=\"0\"$chk_n />" . $this->langObj->get('NO') . "\n";
				$this->display .= "\t\t </td>\n";
				$this->display .= "\t\t</tr>\n";

				$add_update_button = $this->langObj->get('CREATE_USER');
				if ($edit_mode === 'user_update') {
					$this->display .= "\t\t<tr>\n";
					$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('CREATED') . ":</td>\n";
					$this->display .= "\t\t <td>" . $userObj->getTimestamp() . "</td>\n";
					$this->display .= "\t\t</tr>\n";
					$add_update_button = $this->langObj->get('UPDATE_USER');
				}

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t <td>&nbsp;</td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t <td colspan=\"2\">\n";
				$this->display .= "\t\t \t" . '<input type="submit" name="edit" value="' . $add_update_button . "\" />\n";
				$this->display .= "\t\t \t" . '<input type="submit" name="user_delete" value="' . $this->langObj->get('DELETE_USER') . "\" onclick=\"return confirm('" . $this->langObj->get('CONFIRM_DELETE_USER') . "')\" />\n";
				$this->display .= "\t\t \t" . '<input type="submit" name="cancel" value="' . $this->langObj->get('CANCEL') . "\" />\n";
				$this->display .= "\t\t \t<input type=\"hidden\" name=\"id\" value=\"$id\" />\n";
				$this->display .= "\t\t \t<input type=\"hidden\" name=\"edit_mode\" value=\"$edit_mode\" />\n";
				$this->display .= "\t\t \t<input type=\"hidden\" name=\"cancel_mode\" value=\"users\" />\n";
				$this->display .= "\t\t </td>\n";
				$this->display .= "\t\t</tr>\n";

			} else {
				//--- User Selector
				$this->display .= "\t<table class=\"pb_created\">\n";
				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('USER') . "</td>\n";
				$this->display .= "\t\t <td>\n";
				$this->display .= "\t\t \t<select name=\"id\">\n";
				$this->display .= "\t\t \t " . '<option value="-1">' . $this->langObj->get('OPTION_NEW_USER') . "</option>\n";
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($this->users) as $id => $obj) {
						$this->display .= "\t\t \t <option value=\"$id\">" . $obj->getUser() . "</option>\n";
					}
				} else {
					foreach ($this->users as $id => $obj) {
						$this->display .= "\t\t \t <option value=\"$id\">" . $obj->getUser() . "</option>\n";
					}
				}
				$this->display .= "\t\t \t</select>\n";
				$this->display .= "\t\t </td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t <td>&nbsp;</td>\n";
				$this->display .= "\t\t</tr>\n";

				$this->display .= "\t\t<tr>\n";
				$this->display .= "\t\t <td colspan=\"2\">\n";
				$this->display .= "\t\t \t" . '<input type="submit" name="edit" value="' . $this->langObj->get('EDIT_USER') . "\" />\n";
				$this->display .= "\t\t \t" . '<input type="submit" name="cancel" value="' . $this->langObj->get('CANCEL') . "\" />\n";
				$this->display .= "\t\t \t<input type=\"hidden\" name=\"edit_mode\" value=\"user_edit\" />\n";
				$this->display .= "\t\t </td>\n";
				$this->display .= "\t\t</tr>\n";
			}

			$this->display .= "\t</table></form>\n";
			$this->display .= '</div>' . "\n";

		}

		//-----------------------------------------------
		//--- METHOD: setEditTopicsDisplay
		//--- PURPOSE: set topic edit output display
		//--- PARAMETERS: topic id
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------

		function setEditTopicsDisplay($tid)
		{
			if (count($this->topics) === 0) {
				$this->setTopics();
			}

			//--- Set Headers
			if (defined('TOP_HEADER')) {
				$this->accessory_display .= '<div id="PbTopHeader" class="pb_nav-bar">';
				$this->accessory_display .= TOP_HEADER;
				$this->accessory_display .= '</div>';
			}
			$this->accessory_display .= '<div id="PbSubHeader" class="pb_sh_h1">';
			$this->accessory_display .= $this->langObj->get('PB_TOPIC_ADMIN');
			$this->accessory_display .= '</div>';

			//--- Set Display
			$this->display = '<div class="pb_edit">';
			$this->display .= '<form action="' . BASE_URL . '" method="post">' . "\n";

			if ($tid) {
				//--- Add/Edit Topic
				$edit_mode = ($tid == -1) ? 'topic_insert' : 'topic_update';
				$topicObj = new PbTopic($this->getDbPrefix(), $tid);
				$this->display .= "\n\t" . '<p class="pb_created"><span class="pb_input_text">' . $this->langObj->get('TOPIC') . ':</span> ';
				$this->display .= "\t\t" . '<input type="text" name="name" value="' . $topicObj->getName() . "\" />\n";
				$this->display .= "\t<br />\n";
				$this->display .= "\n\t" . '<span class="pb_input_text">' . $this->langObj->get('ICON') . ':</span>&nbsp;&nbsp;';
				$this->display .= "\t\t" . '<input type="text" name="icon" value="' . $topicObj->getIcon() . "\" />\n";
				if ($edit_mode !== 'topic_insert') {
					$this->display .= "\t<br />\n";
					$icon = $this->buildTopicIcon($topicObj->getIcon(), $topicObj->getName(), FALSE);
					$this->display .= "\t\t&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$icon\n";
				}
				$this->display .= "\t<br /><br />\n";
				$this->display .= "\t" . '<input type="submit" name="edit" value="' . $this->langObj->get('SAVE_TOPIC') . "\" />\n";
				if ($edit_mode !== 'topic_insert' && count($this->topics) > 1) {
					$this->display .= "\t" . '<input type="submit" name="topic_delete" value="' . $this->langObj->get('DELETE_TOPIC') . "\" onclick=\"return confirm('" . $this->langObj->get('CONFIRM_DELETE_TOPIC') . "')\" />\n";
				}
				$this->display .= "\t\t \t" . '<input type="submit" name="cancel" value="' . $this->langObj->get('CANCEL') . "\" />\n";
				$this->display .= "\t<input type=\"hidden\" name=\"tid\" value=\"$tid\" />\n";
				$this->display .= "\t<input type=\"hidden\" name=\"edit_mode\" value=\"$edit_mode\" />\n";
				$this->display .= "\t<input type=\"hidden\" name=\"cancel_mode\" value=\"topics\" />\n";

			} else {
				//--- Topic Selector
				$this->display .= "\n\t" . '<p class="pb_created"><span class="pb_input_text">' . $this->langObj->get('TOPIC') . ':</span> ';
				$this->display .= "\t\t<select name=\"tid\">\n";
				$this->display .= "\t\t" . ' <option value="-1">' . $this->langObj->get('OPTION_NEW_TOPIC') . "</option>\n";
				if (isset($this->topics) && count($this->topics) > 0) {
					//--- SPL Support
					if ($this->getSpl()) {
						foreach (new ArrayObject($this->topics) as $id => $obj) {
							$this->display .= "\t\t <option value=\"$id\">" . $obj->getName() . "</option>\n";
						}
					} else {
						foreach ($this->topics as $id => $obj) {
							$this->display .= "\t\t <option value=\"$id\">" . $obj->getName() . "</option>\n";
						}
					}
				}
				$this->display .= "\t\t</select>\n";
				$this->display .= "\t<br /><br />\n";
				$this->display .= "\t" . '<input type="submit" name="edit" value="'. $this->langObj->get('EDIT_TOPIC') . "\" />\n";
				$this->display .= "\t\t \t" . '<input type="submit" name="cancel" value="' . $this->langObj->get('CANCEL') . "\" />\n";
				$this->display .= "\t<input type=\"hidden\" name=\"edit_mode\" value=\"topic_edit\" />\n";
			}

			$this->display .= "\t</p></form>\n";
			$this->display .= '</div>' . "\n";
		}

		//-----------------------------------------------
		//--- METHOD: setAddCommentDisplay
		//--- PURPOSE: set add/reply comment display
		//--- PARAMETERS: entry id
		//---					parent id
		//---					comment id (optional)
		//---					reply mode (optional)
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------

		function setAddCommentDisplay($eid, $pid, $cid = '', $reply_mode = FALSE)
		{
			//--- Setup
			$base_url = BASE_URL;
			$comment = '';
			$comment_rows = COMMENT_ROWS;
			$comment_cols = COMMENT_COLS;
			$comment_tags = str_replace('<', '&lt;', COMMENT_TAGS);
			$comment_tags = str_replace('>', '&gt;', $comment_tags);
			$edit_mode = (empty($cid)) ? 'comment_insert' : 'comment_update';
			$speller = '';
			if (defined('USE_SPELLER') && USE_SPELLER) {
				$speller = '<input type="button" name="spell_check" value="Check Spelling" onClick="openSpellChecker();" />';
			}
			$name = '';
			$email = '';
			$url = '';
			if (isset($_COOKIE['pb_comment'])) {
				$name = (isset($_COOKIE['pb_comment']['name']) && ctype_print($_COOKIE['pb_comment']['name'])) ? strip_tags($_COOKIE['pb_comment']['name']) : '';
				$pattern = '/^((?:(?:(?:[a-zA-Z0-9][\.\-\+_]?)*)[a-zA-Z0-9])+)\@((?:(?:(?:[a-zA-Z0-9][\.\-_]?){0,62})[a-zA-Z0-9])+)\.([a-zA-Z0-9]{2,6})$/';
				$email = (isset($_COOKIE['pb_comment']['email']) && preg_match($pattern, $_COOKIE['pb_comment']['email'])) ? $_COOKIE['pb_comment']['email'] : '';
				$url = (isset($_COOKIE['pb_comment']['url']) && preg_match('/^[\w:\/.-]+$/', $_COOKIE['pb_comment']['url'])) ? $_COOKIE['pb_comment']['url'] : '';
			}
			$dupe_id = md5(uniqid(rand(), true));
			$_SESSION['dupe_id'] = $dupe_id;


			//--- Set Display
			$header = $this->langObj->get('COMMENT');
			if ($reply_mode) {
				$header = $this->langObj->get('REPLY');
				$entryObj = new PbEntry($this->getDbPrefix(), $eid);
				$commentObj = $entryObj->getComment($pid);
				$this->display .= $this->buildComment($eid, $commentObj, FALSE);
			}
			$name_txt = $this->langObj->get('NAME');
			$email_txt = $this->langObj->get('EMAIL');
			$website_txt = $this->langObj->get('WEBSITE');
			$comment_txt = $this->langObj->get('COMMENT');
			$allowed_tags = $this->langObj->get('ALLOWED_TAGS');
			$auto_break = $this->langObj->get('AUTO_BREAK');
			$required = $this->langObj->get('REQUIRED');
			$notification = $this->langObj->get('NOTIFICATION_ONLY');
			$optional = $this->langObj->get('OPTIONAL');
			$remember = $this->langObj->get('REMEMBER_ME');
			$cancel = $this->langObj->get('CANCEL');
			$submit = $this->langObj->get('SUBMIT');
			$no_name = $this->langObj->get('COMMENT_NO_NAME');
			$no_comment = $this->langObj->get('COMMENT_NO_TEXT');
			$this->display .= <<<REPLY
				<a name="comment" id="comment"></a>
				<a name="reply" id="reply"></a>
				<div class="pb_sh_h1">$header</div>
				<div class="pb_edit">
				<form action="$base_url" method="post">
				<table>
					<tr>
						<td class="pb_input_text">$name_txt:</td>
						<td><input type="text" name="name" value="$name" /> ($required)</td>
					</tr>
					<tr>
						<td class="pb_input_text">$email_txt:</td>
						<td><input type="text" name="email" value="$email" /> ($notification)</td>
					</tr>
					<tr>
						<td class="pb_input_text">$website_txt:</td>
						<td><input type="text" name="url" value="$url" /> ($optional)</td>
					</tr>
					<tr>
						<td class="pb_input_text" colspan="2"><input type="checkbox" name="remember_me" value="1" checked="checked" /> $remember</td>
					</tr>
					<tr>
						<td class="pb_input_text">$comment_txt:</td>
						<td>&nbsp;</td>
					</tr>
				</table>
				<p>
					<textarea name="comment" rows="$comment_rows" cols="$comment_cols">$comment</textarea>
					<br />
					<span class="pb_input_text">$allowed_tags:</span> $comment_tags
					<br />
					&nbsp;&nbsp;<input type="checkbox" name="linebreak" value="1" checked="checked" /><span class="pb_input_text">$auto_break</span>
					<br />
					&nbsp;&nbsp;<input type="submit" name="edit" value="$submit" onclick="return ValidateComment(this.form,'$no_name','$no_comment');" />
					$speller
					<input type="submit" name="cancel" value="$cancel" />
					<input type="hidden" name="dupe_id" value="$dupe_id" />
					<input type="hidden" name="eid" value="$eid" />
					<input type="hidden" name="pid" value="$pid" />
					<input type="hidden" name="cid" value="$cid" />
					<input type="hidden" name="edit_mode" value="$edit_mode" />
				</p>
			</form>
			</div>
REPLY;

		}

		//-----------------------------------------------
		//--- METHOD: setEditCommentDisplay
		//--- PURPOSE: set edit comment display
		//--- PARAMETERS: comment object reference
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------

		function setEditCommentDisplay($commentObj)
		{
			//--- Setup
			$base_url = BASE_URL;
			$comment_rows = COMMENT_ROWS;
			$comment_cols = COMMENT_COLS;
			$comment_tags = str_replace('<', '&lt;', COMMENT_TAGS);
			$comment_tags = str_replace('>', '&gt;', $comment_tags);
			$speller = '';
			if (defined('USE_SPELLER') && USE_SPELLER) {
				$speller = '<input type="button" name="spell_check" value="Check Spelling" onClick="openSpellChecker();" />';
			}
			$name = $commentObj->getName();
			$email = $commentObj->getEmail();
			$url = $commentObj->getUrl();
			$cid = $commentObj->getId();
			$eid = $commentObj->getEid();
			$pid = $commentObj->getPid();
			$comment = $commentObj->getComment();
			if ($commentObj->getDisplay()) {
				$comment = str_replace('<br />', "\n", $comment);
				$display_comment = ' checked="checked"';
			} else {
				$display_comment = '';
			}

			//--- Set Display
			$header = $this->langObj->get('EDIT_COMMENT');
			$name_txt = $this->langObj->get('NAME');
			$email_txt = $this->langObj->get('EMAIL');
			$website_txt = $this->langObj->get('WEBSITE');
			$comment_txt = $this->langObj->get('COMMENT');
			$allowed_tags = $this->langObj->get('ALLOWED_TAGS');
			$auto_break = $this->langObj->get('AUTO_BREAK');
			$required = $this->langObj->get('REQUIRED');
			$notification = $this->langObj->get('NOTIFICATION_ONLY');
			$optional = $this->langObj->get('OPTIONAL');
			$cancel = $this->langObj->get('CANCEL');
			$submit = $this->langObj->get('SUBMIT');
			$this->display .= <<<REPLY
				<a name="comment" id="comment"></a>
				<div class="pb_sh_h1"></div>
				<div class="pb_edit">
				<form action="$base_url" method="post">
				<table>
					<tr>
						<td class="pb_input_text">$name_txt:</td>
						<td><input type="text" name="name" value="$name" /> ($required)</td>
					</tr>
					<tr>
						<td class="pb_input_text">$email_txt:</td>
						<td><input type="text" name="email" value="$email" /> ($notification)</td>
					</tr>
					<tr>
						<td class="pb_input_text">$website_txt:</td>
						<td><input type="text" name="url" value="$url" /> ($optional)</td>
					</tr>
					<tr>
						<td class="pb_input_text">$comment_txt:</td>
						<td>&nbsp;</td>
					</tr>
				</table>
				<p>
					<textarea name="comment" rows="$comment_rows" cols="$comment_cols">$comment</textarea>
					<br />
					<span class="pb_input_text">$allowed_tags:</span> $comment_tags
					<br />
					&nbsp;&nbsp;<input type="checkbox" name="linebreak" value="1" checked="checked" /><span class="pb_input_text">$auto_break</span>
					&nbsp;&nbsp;<input type="checkbox" name="display" value="1"$display_comment /><span class="pb_input_text">Display Entry</span>
					<br />
					&nbsp;&nbsp;<input type="submit" name="edit" value="$submit" onclick="return ValidateComment(this.form);" />
					$speller
					<input type="submit" name="cancel" value="$cancel" />
					<input type="hidden" name="eid" value="$eid" />
					<input type="hidden" name="pid" value="$pid" />
					<input type="hidden" name="cid" value="$cid" />
					<input type="hidden" name="edit_mode" value="comment_update" />
				</p>
			</form>
			</div>
REPLY;

		}

		/**
		 * Builds array of entries to be worked with.
		 *
		 * @param int|string $start Entry ID starting point or date offset
		 * @param boolean $is_date Flag the $start is a date (lame!  This shouldn't be necessary.)
		 * @param int $tid Topic ID
		 * @param boolean $rss Flag that this is a call for building and RSS feed.
		 */
		function setEntries($start = NULL, $is_date = FALSE, $tid = FALSE, $rss = FALSE)
		{

			//--- Set Total Entry Count
			$this->setEntryCnt();

			//--- Query - This Date
			if ($is_date) {
				$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? " AND display='1'" : '';
				$post_dated = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? ' AND updated<=UNIX_TIMESTAMP()' : '';
				$query = "SELECT
								eid
							 FROM
						 		{$this->prefix}entries
						 	 WHERE
						 	 	DATE_FORMAT(FROM_UNIXTIME(updated), '%Y%m%d')='%s'
						 	 $display_flag
						 	 $post_dated
						 	 ORDER BY
						 	 	updated";
				$query_data = array($start);
			} else {
				//--- Query - Start Position
				$where = '';
				if (!$rss && (isset($_SESSION['login']) && $_SESSION['login'])) {
					if ($tid) {
						$where = "WHERE tid='$tid'";
					}
				} else {
					$where = "WHERE display='1' AND updated<=UNIX_TIMESTAMP()";
					if ($tid) {
						$where .= " AND tid='$tid'";
					}
				}

				$limit = ($start) ? 'LIMIT ' . $start . ',' . $this->display_cnt : 'LIMIT ' . $this->display_cnt;
				$query = "SELECT
								eid
							 FROM
						 		{$this->prefix}entries
						 	$where
						 	 ORDER BY
						 	 	updated
						 	 DESC
						 	 $limit";
				$query_data = array();
			}

			//--- Issue Query
			if ($this->dbObj->safeQuery($query, $query_data)) {
				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {
					$i = 0;
					while ($this->dbObj->getRow()) {
						$eid = (int) $this->dbObj->row['eid'];
						$this->entries[$i] = new PbEntry($this->getDbPrefix(), $eid);
						$i++;
					}
				}
				$stat = TRUE;

			} else {
				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setUsers
		//--- PURPOSE: set users in object
		//--- PARAMETERS: none
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setUsers()
		{
			//--- Reset
			unset($this->users);

			//--- Query
			$query = "SELECT
							id
						 FROM
					 		{$this->prefix}users
					 	 ORDER BY
					 	 	user";

			//--- Issue Query
			if ($this->dbObj->query($query)) {

				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {

					//--- Retrieve Results
					while ($this->dbObj->getRow()) {
						$id = (int) $this->dbObj->row['id'];
						$this->users[$id] = new PbUser($this->getDbPrefix(), $id);
					}
				}

				$stat = TRUE;

			} else {

				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setTopics
		//--- PURPOSE: set topic objects
		//--- PARAMETERS: none
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setTopics()
		{
			//--- Reset
			unset($this->topics);
			$this->topics = array();

			//--- Query
			$query = "SELECT
							tid
						 FROM
					 		{$this->prefix}topics FORCE INDEX (name_idx)
					 	 ORDER BY
					 	 	name";

			//--- Issue Query
			if ($this->dbObj->query($query)) {
				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {

					//--- Retrieve Results
					while ($this->dbObj->getRow()) {
						$tid = (int) $this->dbObj->row['tid'];
						$this->topics[$tid] = new PbTopic($this->getDbPrefix(), $tid);
					}
				}

				$stat = TRUE;

			} else {

				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setEntryCnt
		//--- PURPOSE: set total entry count in object
		//--- PARAMETERS: none
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function setEntryCnt()
		{

			//--- Query
			$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? "WHERE display='1'" : '';
			$post_dated = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? ' AND updated<='. time() : '';
			$query = "SELECT
							COUNT(*) AS count
						 FROM
					 		{$this->prefix}entries
					 	 $display_flag
					 	 $post_dated";

			//--- Issue Query
			if ($this->dbObj->query($query)) {

				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {

					//--- Retrieve Results
					$this->dbObj->getRow();
					$this->entry_cnt = (int) $this->dbObj->row['count'];

				}

				$stat = TRUE;

			} else {

				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			return $stat;
		}

		//-----------------------------------------------
		//--- METHOD: setLoginDisplay
		//--- PURPOSE: sets login display screen
		//--- PARAMETERS: username (optional)
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setLoginDisplay($username)
		{
			$this->display = '<div class="pb_entry">' . "\n";
			$this->display .= '<form action="' . BASE_URL . '" method="post">' . "\n";
			$this->display .= '<table>' . "\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t\t" . '<th align="right">' . $this->langObj->get('USERNAME') . ":</th>\n";
			$this->display .= "\t\t<td><input type=\"text\" name=\"user\" value=\"$username\" /></td>\n";
			$this->display .= "\t</tr>\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t" . '<th align="right">' . $this->langObj->get('PASSWORD') . ":</th>\n";
			$this->display .= "\t<td><input type=\"password\" name=\"passwd\" value=\"\" /></td>\n";
			$this->display .= "\t</tr>\n";
			$this->display .= "\t<tr>\n";
			if (MAIL_TYPE == FALSE) {
				$this->display .= '<td>&nbsp;</td>';
			} else {
				$this->display .= '<td colspan="2"><p>' . $this->buildLink(BASE_URL . '?hint_me=1', $this->langObj->get('PASSWORD_HINT_LINK'), $this->langObj->get('PASSWORD_HINT_LINK')) . '</p></td>' . "\n";
			}
			$this->display .= "\t</tr>\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t\t" . '<td colspan="2"><input type="submit" name="login_req" value="' . $this->langObj->get('LOGIN') . '" />';
			$this->display .= '&nbsp;&nbsp;<input type="submit" name="cancel" value="' . $this->langObj->get('CANCEL') . '" /></td>' . "\n";
			$this->display .= "\t</tr>\n";
			$this->display .= '</table>' . "\n";
			$this->display .= '<form>' . "\n";
			$this->display .= '</div>' . "\n";
		}

		/**
		 * Request password hint by email display
		 */
		function setHintDisplay()
		{
			$this->display = '<div class="pb_entry">' . "\n";
			$this->display .= '<form action="' . BASE_URL . '" method="post">' . "\n";
			$this->display .= '<table>' . "\n";
			$this->display .= "\t<tr>\n";
			$this->display .= '<td colspan="2"><p>' . $this->langObj->get('PASSWORD_HINT_MSG') . '</p></td>' . "\n";
			$this->display .= "\t</tr>\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t\t" . '<th align="right">' . $this->langObj->get('USERNAME') . ":</th>\n";
			$this->display .= "\t\t<td><input type=\"text\" name=\"user\" value=\"\" /></td>\n";
			$this->display .= "\t</tr>\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t\t" . '<th align="right">&nbsp;</th>' . "\n";
			$this->display .= "\t</tr>\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t\t" . '<td colspan="2"><input type="submit" name="hint_req" value="' . $this->langObj->get('PASSWORD_HINT_BUTTON') . '" />';
			$this->display .= '&nbsp;&nbsp;<input type="submit" name="cancel" value="' . $this->langObj->get('CANCEL') . '" /></td>' . "\n";
			$this->display .= "\t</tr>\n";
			$this->display .= '</table>' . "\n";
			$this->display .= '<form>' . "\n";
			$this->display .= '</div>' . "\n";
		}

		/**
		 * Request password update display
		 */
		function setChangePasswordDisplay()
		{
			$this->display = '<div class="pb_entry">' . "\n";
			$this->display .= '<form action="' . BASE_URL . '" method="post" onsubmit="return pw_change(this,\'' . $this->langObj->get('FORM_ALL_FIELDS') . "','" . $this->langObj->get('PASSWORD_MISMATCH') . "');\">\n";
			$this->display .= '<table>' . "\n";
			$this->display .= "\t\t<tr>\n";
			$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('PASSWORD_NEW') . ":</td>\n";
			$this->display .= "\t\t" . ' <td><input type="password" size="30" name="password" value="" /></td>' . "\n";
			$this->display .= "\t\t</tr>\n";
			$this->display .= "\t\t<tr>\n";
			$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('PASSWORD_CONFIRM') . ":</td>\n";
			$this->display .= "\t\t" . ' <td><input type="password" size="30" name="password2" value="" /></td>' . "\n";
			$this->display .= "\t\t</tr>\n";

			$this->display .= "\t\t<tr>\n";
			$this->display .= "\t\t" . ' <td class="pb_input_text">' . $this->langObj->get('PASSWORD_HINT') . ":</td>\n";
			$this->display .= "\t\t" . ' <td><input type="text" name="hint" size="50" value="" /></td>' . "\n";
			$this->display .= "\t\t</tr>\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t\t" . '<th align="right">&nbsp;</th>' . "\n";
			$this->display .= "\t</tr>\n";
			$this->display .= "\t<tr>\n";
			$this->display .= "\t\t" . '<td colspan="2"><input type="submit" name="pw_update" value="' . $this->langObj->get('PASSWORD_UPDATE') . '" /></td>' . "\n";
			$this->display .= "\t</tr>\n";
			$this->display .= '</table>' . "\n";
			$this->display .= '<form>' . "\n";
			$this->display .= '</div>' . "\n";
		}

		//-----------------------------------------------
		//--- METHOD: setEntry
		//--- PURPOSE: add/edit entry
		//--- PARAMETERS: add/edit mode
		//---					entry id
		//---					entry title
		//---					entry topic id
		//---					entry add linebreaks?
		//---					entry display?
		//---					allow comments?
		//---					entry body text
		//---					entry more text
		//---					entry update timestamp
		//---					entry time adjust direction
		//---					entry time adjust hours
		//---					entry time adjust minutes
		//---					ping Ping-O-Matic!
		//---					trackback URL
		//---					trackback send
		//--- RETURNS: nothing
		//-----------------------------------------------
		function setEntry($mode, $eid, $title, $tid, $linebreak, $display, $allow_comments, $body, $more, $update_timestamp, $create_timestamp, $time_adj_dir, $time_adj_hour, $time_adj_min, $ping = FALSE, $tb_url = FALSE, $tb_send = FALSE)
		{
			//--- Build Entry Object
			$this->entries[0] = new PbEntry($this->getDbPrefix(), $eid);
			$this->entries[0]->setId($eid);
			$this->entries[0]->setTitle($title);
			$this->entries[0]->setTopicId($tid);
			$this->entries[0]->setLineBreak($linebreak);
			$this->entries[0]->setDisplay($display);
			$this->entries[0]->setAllowComments($allow_comments);
			$this->entries[0]->setBody($body);
			$this->entries[0]->setMore($more);
			$this->entries[0]->setAdjustTime($update_timestamp, $create_timestamp, $time_adj_dir, $time_adj_hour, $time_adj_min);
			$this->entries[0]->setPing($ping);
			$this->entries[0]->setTbUrl($tb_url);
			$this->entries[0]->setTbSend($tb_send);
			$this->entries[0]->setRblServices($this->getRblServices());

			//--- Add Entry
			if ($mode === 'insert') {
				$stat = $this->entries[0]->insertEntry();
				$this->entries[0]->setId($this->entries[0]->getId());
				$this->setNewEid($this->entries[0]->getId());
			} elseif($mode === 'update') {
				//--- Update Entry
				$stat = $this->entries[0]->updateEntry();
			}

			if (!$stat) {
				$this->setDisplayMsg($this->entries[0]->dbObj->getError());
			} else {
				$this->setDisplayMsg($this->entries[0]->getEntryMsg() . '.', 'ENTRY_STATUS');
			}

			return $stat;
		}

		/**
		 * Set the new entry id
		 *
		 * @param int $a
		 */
		function setNewEid($a)
		{
			$this->new_eid = (int) $a;
		}

		/**
		 * RBL services to use for spam checking.
		 *
		 * @param array $a
		 */
		function setRblServices($a)
		{
			$this->rbl_services = $a;
		}






		//-----------------------------------------------
		//-----------------------------------------------
		//--- GET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: getCommentIndent
		//--- PURPOSE: get comment indent value
		//--- PARAMETERS: none
		//--- RETURNS: comment indent value
		//-----------------------------------------------
		function getCommentIndent()
		{
			return $this->comment_indent;
		}

		/**
		 * Returns information on comments that have the display disabled.
		 *
		 * @return array
		 */
		function getCommentsNoDisplay()
		{
			if ($this->comments_no_display === NULL) {
				$this->setCommentsNoDisplay();
			}

			return $this->comments_no_display;
		}

		//-----------------------------------------------
		//--- METHOD: getDisplay
		//--- PURPOSE: get display output
		//--- PARAMETERS: none
		//--- RETURNS: XHTML display data
		//-----------------------------------------------
		function getDisplay()
		{
			return $this->display;
		}

		//-----------------------------------------------
		//--- METHOD: getAccessoryDisplay
		//--- PURPOSE: get calendar & header display output
		//--- PARAMETERS: none
		//--- RETURNS: XHTML calendar display data
		//-----------------------------------------------
		function getAccessoryDisplay()
		{
			return $this->accessory_display;
		}

		//-----------------------------------------------
		//--- METHOD: getEntryCnt
		//--- PURPOSE: get total entry count
		//--- PARAMETERS: none
		//--- RETURNS: (int) entry count
		//-----------------------------------------------
		function getEntryCnt()
		{
			return $this->entry_cnt;
		}

		/**
		 * Gets entry objects
		 *
		 * @return object
		 */
		function getEntries()
		{
			return $this->entries;
		}

		/**
		 * Get new entry id
		 *
		 * @return int
		 */
		function getNewEid()
		{
			return (int) $this->new_eid;
		}

		//-----------------------------------------------
		//--- METHOD: getWordCnt
		//--- PURPOSE: get string word count
		//--- PARAMETERS: (string) text string
		//--- RETURNS: (int) word count in string
		//-----------------------------------------------
		function getWordCnt($str)
		{
			$str = trim(strip_tags($str));
			$word_cnt = (strlen($str) == 0) ? 0 : count(preg_split('/\s+/', $str));

			return $word_cnt;
		}

		/**
		 * Gets an RBL object
		 * @return object
		 */
		function getRblObj()
		{
			if (!is_object($this->rblObj)) {
				 return $this->rblObj;
			}
			return FALSE;
		}

		/**
		 * Get RBL services to check for spam.
		 *
		 * @return array
		 */
		function getRblServices()
		{
			return $this->rbl_services;
		}


		//-----------------------------------------------
		//-----------------------------------------------
		//--- OTHER METHODS -----------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		//-----------------------------------------------
		//--- METHOD: buildEntry
		//--- PURPOSE: build entry div content
		//--- PARAMETERS: entry object reference
		//---					full entry display flag
		//---					comment mode flag (optional)
		//--- RETURNS: XHTML content
		//-----------------------------------------------
		function buildEntry($entryObj, $full_entry = FALSE, $footer_enabled = TRUE)
		{

			$tid = $entryObj->getTopicId();
			$content = '';

			//--- Date Separator
			if ((defined('DATE_SEPARATOR_FORMAT') || defined('I18N_DATE_SEPARATOR_FORMAT')) && $this->getEntryCnt() > 1) {
				$content .= $this->buildDateSeparator($entryObj->getUpdated());
			}

			//--- Header
			$content .= '<div class="pb_entry">' . "\n";
			$content .= "\t<div class=\"pb_topic_icon\">\n";
			$content .= "\t " . $this->buildTopicLink($tid) . "\n";
			$content .= "\t " . '&nbsp;&nbsp;<span class="pb_title">' . $entryObj->getTitle() . '</span>';
			if ($entryObj->getDisplay() && ($entryObj->getUpdated() > time())) {
				$content .= ' (<span style="color:#d50000; font-size:14px; font-weight:bold;">' . $this->langObj->get('DISPLAY_DISABLED_FUTURE') . '</span>)';
			}
			$content .= (!$entryObj->getDisplay()) ? ' (<span style="color:#d50000; font-size:14px; font-weight:bold;">' . $this->langObj->get('DISPLAY_DISABLED') . '</span>)<br />'  ."\n" : '<br />'  ."\n";
			$content .= "\t " . '<span class="pb_created">' . $entryObj->getCreated(TRUE) . "</span>\n";
			if ($entryObj->getCreated() != $entryObj->getUpdated()) {
				$content .= "\t <br />\n";
				$content .= "\t " . '<span class="pb_created">' . $this->langObj->get('UPDATED') . ': ' . $entryObj->getUpdated(TRUE) . "</span>\n";
			}
			if (defined('USE_POST_BY') && USE_POST_BY) {
				$content .= "\t <br />\n";
				$content .= "\t " . '<span class="pb_created">' . $this->langObj->get('POSTED_BY') . ' ' . $entryObj->getPostBy() . "</span>\n";
			}
			$content .= "\t</div>\n";
			$content .= "\t" . '<div class="pb_body">' . "\n";

			//--- Trackback Auto-Discovery
			if ((defined('USE_COMMENTS') && USE_COMMENTS) && (defined('USE_TRACKBACK') && USE_TRACKBACK) &&
				(defined('TB_AUTO_DISCOVERY') && TB_AUTO_DISCOVERY)) {
				$content .= $entryObj->buildTrackbackRdf($entryObj);
			}

			//--- Main Entry Content
			$content .= (defined('USE_GIGGLE') && USE_GIGGLE) ? $entryObj->getBody('GIGGLE') . "\n" : $entryObj->getBody() . "\n";

			//--- More Link
			if (!$full_entry && $entryObj->getMoreExists()) {
				$content .= '&nbsp;&nbsp;';
				$content .= $this->buildLink(BASE_URL . '?eid=' . $entryObj->getId(), $this->langObj->get('READ_MORE_OF') . ' ' . $entryObj->getTitle(), $this->langObj->get('READ_MORE') . '&hellip;') . "\n";
			} elseif ($full_entry && $entryObj->getMoreExists()) {
				//--- Include Full Entry
				$content .= '<br /><br />';
				if (defined('USE_GIGGLE') && USE_GIGGLE) {
					$content .= $entryObj->getMore('GIGGLE');
				} else {
					$content .= $entryObj->getMore();
				}

			}

			$content .= "\t</div>\n";

			if ($footer_enabled) {
				//--- Footer Links
				$link = $this->buildLink(BASE_URL . '?eid=' . $entryObj->getId(), $this->langObj->get('PERMALINK') . ': ' . $entryObj->getTitle(), $this->langObj->get('PERMALINK'));
				$content .= "\t<span class=\"pb_entry_footer\">\n";
				$content .= "\t [$link]\n";

				//--- Comment Links
				if ($entryObj->getCommentCnt() > 0) {
					$link = $this->buildLink(BASE_URL . '?eid=' . $entryObj->getId() . '#comments', $this->langObj->get('COMMENT_VIEW') . ' ' . $entryObj->getTitle(), $this->langObj->get('COMMENTS') . '  (' . $entryObj->getCommentCnt() . ')');
					$content .= "\t&nbsp;&nbsp;[$link]\n";
				}
				if (defined('USE_COMMENTS') && USE_COMMENTS && $entryObj->getAllowComments()) {
						if (!$this->getIpBlocked($this->getUserIp())) {
							$link = $this->buildLink(BASE_URL . '?eid=' . $entryObj->getId() . '&amp;pid=0#comment', $this->langObj->get('COMMENT_ON') . ' ' . $entryObj->getTitle(), $this->langObj->get('COMMENT_ADD'));
							$content .= "\t&nbsp;&nbsp;[$link]\n";
						}
				}

				//--- Trackback URL
				if ((defined('USE_COMMENTS') && USE_COMMENTS) && (defined('USE_TRACKBACK') && USE_TRACKBACK) && $entryObj->getAllowComments()) {
					if (!$this->getIpBlocked($this->getUserIp())) {
						$link = $this->buildLink(BASE_URL . 'trackback.php?eid=' . $entryObj->getId(), $this->langObj->get('TB_LINK_TEXT'), $this->langObj->get('TRACKBACK'));
						$content .= "\t&nbsp;&nbsp;[$link]\n";
					}
				}

				//--- Google Link
				if (defined('USE_GOOGLE_LINK') && USE_GOOGLE_LINK) {
					$google_topic = htmlentities(urlencode($entryObj->getTitle()), ENT_QUOTES, I18N_CHARSET);
					$google = $this->buildLink('http://www.google.com/search?q=' . $google_topic, $this->langObj->get('GOOGLE_FOR') . ': ' . $entryObj->getTitle(), $this->langObj->get('GOOGLE'));
					$content .= "\t&nbsp;&nbsp;[$google]\n";
				}

				//--- Word Count
				if (defined('USE_WORD_COUNT') && USE_WORD_COUNT) {
					$word_cnt = $this->getWordCnt($entryObj->getBody('DO_STUPEFY'));
					$word_cnt += $this->getWordCnt($entryObj->getMore('DO_STUPEFY'));
					$word_msg = ($word_cnt > 1) ? '(' . $word_cnt . ' ' . $this->langObj->get('WORDS') . ')' : '(' . $word_cnt . ' ' . $this->langObj->get('WORD') . ')';
					$word_msg = ($word_cnt == 0) ? '' : $word_msg;
					$content .= "\t&nbsp;&nbsp;$word_msg\n";
				}

				//--- Admin Links
				if ( (isset($_SESSION['login']) && $_SESSION['login']) && ( (isset($_SESSION['admin']) && $_SESSION['admin']) || (isset($_SESSION['uid']) && $_SESSION['uid'] == $entryObj->getUid()) ) ) {
					$content .= "<br />\n";
					$link = $this->buildLink(BASE_URL . '?eid=' . $entryObj->getId() . '&amp;admin=edit', $this->langObj->get('EDIT') . ': ' . $entryObj->getTitle(), $this->langObj->get('EDIT_ENTRY'));
					$content .= "\t&nbsp;&nbsp;&nbsp;[$link]\n";
					$url = "javascript:confirmDelete('" . $this->langObj->get('CONFIRM_DELETE_ENTRY') . "', '" . BASE_URL . '?eid=' . $entryObj->getId() . '&amp;admin=del' . "')";
					$link = $this->buildLink($url, 'Delete: ' . $entryObj->getTitle(), $this->langObj->get('DELETE_ENTRY'));
					$content .= "\t [$link]\n";
				}
				$content .= "\t</span>\n";
			}
			$content .= '</div>' . "\n";

			return $content;
		}

		/**
		 * Build comment display code.
		 * @param int $eid Entry ID
		 * @param object $commentObj Comment Object
		 * @param boolean $footer_enabled Flag indicating that navigation footer should be displayed.
		 * @param boolean $allow_reply Flag indicating that this entry allows comment replies
		 * @return string $content XHTML fragment of comment display
		 */
		function buildComment($eid, $commentObj, $footer_enabled = TRUE, $allow_reply = TRUE)
		{
			$content = '';
			$reply = '';
			$word_msg = '';
			$cid = $commentObj->getId();
			$name = $commentObj->getName();
			$url = $commentObj->getUrl();
			$timestamp = $commentObj->getTimestamp(TRUE);
			$ip = $commentObj->getIp();
			$ip_white = $commentObj->getIpWhitelisted();
			$comment = $commentObj->getComment();
			$link = $name;
			if (!empty($url)) {
				$url = (strpos($url, 'http://') === FALSE) ? 'http://' . $url : $url;
				$link = $this->buildLink($url, $this->langObj->get('WEBSITE_OF') . ' ' . $name, $name);
			}
			$permalink = $this->buildLink(BASE_URL . '?eid=' . $commentObj->getEid() . '#' . $cid, $this->langObj->get('PERMALINK_TO_COMMENT'), $this->langObj->get('PERMALINK'));
			$permalink = "[$permalink]";
			if ((defined('USE_COMMENTS') && USE_COMMENTS) && $allow_reply && !$this->getIpBlocked($this->getUserIp())) {
				$reply = $this->buildLink(BASE_URL . '?eid=' . $eid . '&amp;pid=' . $commentObj->getId() . '#reply', $this->langObj->get('REPLY_TO') . ' ' . $name . $this->langObj->get('REPLY_TO_NAMES_COMMENT') . '.', $this->langObj->get('REPLY'));
				$reply = "&nbsp;&nbsp;[$reply]";
			}
			if (defined('USE_WORD_COUNT') && USE_WORD_COUNT) {
				$word_cnt = $this->getWordCnt($commentObj->getComment('DO_STUPEFY'));
				$word_msg = '&nbsp;&nbsp;';
				$word_msg .= ($word_cnt > 1) ? '(' . $word_cnt . ' ' . $this->langObj->get('WORDS') . ')' : '(' . $word_cnt . ' ' . $this->langObj->get('WORD') . ')';
				$word_msg = ($word_cnt == 0) ? '' : '&nbsp;&nbsp;' . $word_msg;
			}

			//--- Admin Links
			$admin = '';
			if ( (isset($_SESSION['login']) && $_SESSION['login']) && (isset($_SESSION['admin']) && $_SESSION['admin']) ) {
				$edit_link = $this->buildLink(BASE_URL . '?cid=' . $cid . '&amp;admin=edit_comment', $this->langObj->get('EDIT_COMMENT_BY') . ' ' . $commentObj->getName(), $this->langObj->get('EDIT_COMMENT'));
				$url = "javascript:confirmDelete('" . $this->langObj->get('CONFIRM_DELETE_COMMENT') . "', '" . BASE_URL . '?cid=' . $cid . '&amp;admin=del_comment' . "')";
				$del_link = $this->buildLink($url, $this->langObj->get('DELETE_COMMENT_BY') . ' ' . $commentObj->getName(), $this->langObj->get('DELETE_COMMENT'));
				$admin = <<<ADMIN_COMMENT
				<br />
				&nbsp;&nbsp;&nbsp;[$edit_link]
				&nbsp;&nbsp;[$del_link]
ADMIN_COMMENT;
			}

			if (((isset($_SESSION['login']) && $_SESSION['login']) && (isset($_SESSION['admin']) && $_SESSION['admin'])) || $commentObj->getDisplay()) {
				$ip_display = '';
				if ((isset($_SESSION['login']) && $_SESSION['login']) && (isset($_SESSION['admin']) && $_SESSION['admin']) && $ip !== '000.000.000.000') {
					if (defined('RBL_IP_WHOIS') && $ip !== '000.000.000.000') {
						$ip_link = '<a href="' . str_replace('::IP::', $ip, RBL_IP_WHOIS) . '" title="' . $this->langObj->get('RBL_IP_LOOKUP') . '">'  . $ip . '</a>';
					} else {
						$ip_link = $comment['ip'];
					}
					$ip_display = '(' . $ip_link . ')&nbsp;&nbsp;';
					$url_wl = "javascript:confirmDelete('" . $this->langObj->get('CONFIRM_COMMENT_WL') . "', '" . BASE_URL . '?cid=' . $cid . '&amp;admin=rbl_wl' . "')";
					$url_bl = "javascript:confirmDelete('" . $this->langObj->get('CONFIRM_COMMENT_BL') . "', '" . BASE_URL . '?cid=' . $cid . '&amp;admin=rbl_bl' . "')";
					$ip_display .= ($ip_white) ? $this->buildLink($url_bl, $this->langObj->get('BLACKLIST_IP') . ' ' . $ip, $this->langObj->get('BLACKLIST_IP'))
											   : $this->buildLink($url_wl, $this->langObj->get('WHITELIST_IP') . ' ' . $ip, $this->langObj->get('WHITELIST_IP'));
				}
				$indent = $this->getCommentIndent();
				$comment_by = $this->langObj->get('COMMENT_BY');
				$disabled = (!$commentObj->getDisplay()) ? ' (<span style="color:#d50000; font-size:14px; font-weight:bold;">' . $this->langObj->get('DISPLAY_DISABLED') . '</span>)' : '';
				$content .= <<<COMMENT
				<div><a name="$cid" id="cid-$cid"></a></div>
				<div class="pb_entry" style="margin-left: {$indent}px;">
					<span class="pb_created">$timestamp</span>
					<br />
					<span class="pb_created">$comment_by $link $ip_display $disabled</span>
					<br />
					<br />
					<div class="pb_body">$comment</div>
COMMENT;

				if ($footer_enabled) {
					$content .= <<<COMMENT_FOOTER
					<span class="pb_entry_footer">
						&nbsp;&nbsp;$permalink$reply$word_msg
						$admin
					</span>
COMMENT_FOOTER;
				}
			}
			$content .= "\t\t\t\t</div>\n";

			return $content;
		}

		/**
		 * Build trackback display code.
		 *
		 * @param int $eid Entry ID
		 * @param object $commentObj Comment Object
		 * @param boolean $footer_enabled Flag indicating that navigation footer should be displayed.
		 * @param boolean $allow_reply Flag indicating that this entry allows comment replies
		 * @return string $content XHTML fragment of trackback display
		 */
		function buildTrackback($eid, $commentObj, $footer_enabled = TRUE, $allow_reply = TRUE)
		{
			$content = '';
			$reply = '';
			$word_msg = '';
			$cid = $commentObj->getId();
			$eid = $commentObj->getEid();
			$name = $commentObj->getName(); //the TBer's blog name
			$title = $commentObj->getEmail(); //TB entry title
			$url = $commentObj->getUrl();
			$timestamp = $commentObj->getTimestamp(TRUE);
			$ip = $commentObj->getIp();
			$ip_white = $commentObj->getIpWhitelisted();
			$comment = $commentObj->getComment();
			$link = $name;
			if (!empty($url)) {
				$url = (strpos($url, 'http://') === FALSE) ? 'http://' . $url : $url;
				$link = $this->buildLink($url, $this->langObj->get('TB_READ'), $name . ' - ' . $title);
			}
			$permalink = $this->buildLink(BASE_URL . '?eid=' . $commentObj->getEid() . '#' . $cid, $this->langObj->get('TB_PERMALINK'), $this->langObj->get('PERMALINK'));
			$permalink = "[$permalink]";
			if ((defined('USE_COMMENTS') && USE_COMMENTS) && $allow_reply && !$this->getIpBlocked($this->getUserIp())) {
				$reply = $this->buildLink(BASE_URL . '?eid=' . $eid . '&amp;pid=' . $commentObj->getId() . '#reply', $this->langObj->get('TB_REPLY'), $this->langObj->get('REPLY'));
				$reply = "&nbsp;&nbsp;[$reply]";
			}
			if (defined('USE_WORD_COUNT') && USE_WORD_COUNT) {
				$word_cnt = $this->getWordCnt($commentObj->getComment('DO_STUPEFY'));
				$word_msg = '&nbsp;&nbsp;';
				$word_msg .= ($word_cnt > 1) ? '(' . $word_cnt . ' ' . $this->langObj->get('WORDS') . ')' : '(' . $word_cnt . ' ' . $this->langObj->get('WORD') . ')';
				$word_msg = ($word_cnt == 0) ? '' : '&nbsp;&nbsp;' . $word_msg;
			}

			//--- Admin Links
			$admin = '';
			if ( (isset($_SESSION['login']) && $_SESSION['login']) && (isset($_SESSION['admin']) && $_SESSION['admin']) ) {
				if ($commentObj->getDisplay()) {
					$dl_text = $this->langObj->get('TB_DISPLAY_OFF');
					$dl_mode = 'tb_off';
				} else {
					$dl_text = $this->langObj->get('TB_DISPLAY_ON');
					$dl_mode = 'tb_on';
				}
				$display_link = $this->buildLink(BASE_URL . '?cid=' . $cid . '&eid=' . $eid . '&amp;admin=' . $dl_mode, $dl_text, $dl_text);
				$url = "javascript:confirmDelete('" . $this->langObj->get('TB_CONFIRM_DELETE') . "', '" . BASE_URL . '?cid=' . $cid . '&amp;admin=del_comment' . "')";
				$del_link = $this->buildLink($url, $this->langObj->get('TB_DELETE'), $this->langObj->get('TB_DELETE'));
				$admin = <<<ADMIN_COMMENT
				<br />
				&nbsp;&nbsp;&nbsp;[$display_link]
				&nbsp;&nbsp;[$del_link]
ADMIN_COMMENT;
			}

			if (((isset($_SESSION['login']) && $_SESSION['login']) && (isset($_SESSION['admin']) && $_SESSION['admin'])) || $commentObj->getDisplay()) {
				$ip_display = '';
				if ((isset($_SESSION['login']) && $_SESSION['login']) && (isset($_SESSION['admin']) && $_SESSION['admin']) && $ip !== '000.000.000.000') {
					if (defined('RBL_IP_WHOIS') && $ip !== '000.000.000.000') {
						$ip_link = '<a href="' . str_replace('::IP::', $ip, RBL_IP_WHOIS) . '" title="' . $this->langObj->get('RBL_IP_LOOKUP') . '">'  . $ip . '</a>';
					} else {
						$ip_link = $comment['ip'];
					}
					$ip_display = '(' . $ip_link . ')&nbsp;&nbsp;';
					$url_wl = "javascript:confirmDelete('" . $this->langObj->get('CONFIRM_COMMENT_WL') . "', '" . BASE_URL . '?cid=' . $cid . '&amp;admin=rbl_wl' . "')";
					$url_bl = "javascript:confirmDelete('" . $this->langObj->get('CONFIRM_COMMENT_BL') . "', '" . BASE_URL . '?cid=' . $cid . '&amp;admin=rbl_bl' . "')";
					$ip_display .= ($ip_white) ? $this->buildLink($url_bl, $this->langObj->get('BLACKLIST_IP') . ' ' . $ip, $this->langObj->get('BLACKLIST_IP'))
											   : $this->buildLink($url_wl, $this->langObj->get('WHITELIST_IP') . ' ' . $ip, $this->langObj->get('WHITELIST_IP'));
				}
				$indent = $this->getCommentIndent();
				$from = $this->langObj->get('TRACKBACK');
				$disabled = (!$commentObj->getDisplay()) ? ' (<span style="color:#d50000; font-size:14px; font-weight:bold;">' . $this->langObj->get('DISPLAY_DISABLED') . '</span>)' : '';
				$content .= <<<COMMENT
				<div><a name="$cid" id="cid-$cid"></a></div>
				<div class="pb_entry" style="margin-left: {$indent}px;">
					<span class="pb_created">$timestamp</span>
					<br />
					<span class="pb_created">$from: $link $ip_display $disabled</span>
					<br />
					<br />
					<div class="pb_body">$comment</div>
COMMENT;

				if ($footer_enabled) {
					$content .= <<<COMMENT_FOOTER
					<span class="pb_entry_footer">
						&nbsp;&nbsp;$permalink$reply$word_msg
						$admin
					</span>
COMMENT_FOOTER;
				}
			}
			$content .= "\t\t\t\t</div>\n";

			return $content;
		}

		//-----------------------------------------------
		//--- METHOD: buildComments
		//--- PURPOSE: build full block of entry comments
		//--- PARAMETERS: entry object reference
		//--- RETURNS: XHTML content
		//-----------------------------------------------
		function buildComments($entryObj)
		{
			$content = '';
			if ($entryObj->getCommentCnt() > 0) {
				$comments = $entryObj->getComments();
				$content .= $this->buildNestedComments($entryObj, $comments);
			}

			return $content;
		}

		//-----------------------------------------------
		//--- METHOD: buildNestedComments
		//--- PURPOSE: build comments content
		//--- PARAMETERS: entry object reference
		//---					comments id array reference
		//--- RETURNS: XHTML content
		//-----------------------------------------------
		function buildNestedComments($entryObj, $comments)
		{
			$content = '';
			$indent = COMMENT_INDENT_INCREMENT;
			if (empty($this->comment_indent)) {
				$this->setCommentIndent(COMMENT_INDENT);
			}
			$allow_reply = ($entryObj->getAllowComments() == '1') ? TRUE : FALSE;

			//--- SPL Support
			if ($this->getSpl()) {
				foreach (new ArrayObject($comments) as $commentObj) {
					//--- Trackback?
					if ($commentObj->getIsTrackback()) {
						$content .= $this->buildTrackback($entryObj->getId(), $commentObj, TRUE, $allow_reply);
					} else {
						$content .= $this->buildComment($entryObj->getId(), $commentObj, TRUE, $allow_reply);
					}

					//--- Nested Replies
					$replies = $entryObj->getComments($entryObj->getId(), $commentObj->getId());
					$current_indent = $this->getCommentIndent();
					if (count($replies) > 0) {
						$this->setCommentIndent($this->getCommentIndent() + $indent);
						$content .= $this->buildNestedComments($entryObj, $replies);
					}
					$this->setCommentIndent($current_indent);
				}
			} else {
				foreach ($comments as $commentObj) {
					//--- Trackback?
					if ($commentObj->getIsTrackback()) {
						$content .= $this->buildTrackback($entryObj->getId(), $commentObj, TRUE, $allow_reply);
					} else {
						$content .= $this->buildComment($entryObj->getId(), $commentObj, TRUE, $allow_reply);
					}

					//--- Nested Replies
					$replies = $entryObj->getComments($entryObj->getId(), $commentObj->getId());
					$current_indent = $this->getCommentIndent();
					if (count($replies) > 0) {
						$this->setCommentIndent($this->getCommentIndent() + $indent);
						$content .= $this->buildNestedComments($entryObj, $replies);
					}
					$this->setCommentIndent($current_indent);
				}
			}

			return $content;
		}

		//-----------------------------------------------
		//--- METHOD: buildDateSeparator
		//--- PURPOSE: build date separator display
		//--- PARAMETERS: entry date
		//--- RETURNS: XHTML content
		//-----------------------------------------------
		function buildDateSeparator($date)
		{
			$content = '';
			$build = FALSE;

			//--- Date Check
			$entry_date = (defined('I18N_DATE_SEPARATOR_FORMAT')) ? strftime(I18N_DATE_SEPARATOR_FORMAT, $date) :	date(DATE_SEPARATOR_FORMAT, $date);
			if (!isset($this->separator_date)) {
				$this->separator_date = $entry_date;
				$build = TRUE;
			} elseif (isset($this->separator_date) && $this->separator_date != $entry_date) {
				$this->separator_date = $entry_date;
				$build = TRUE;
			}

			//--- Build Content
			if ($build) {
				$content = "\t" . '<div class="pb_date_separator">' . "\n";
				$content .= "\t $entry_date\n";
				$content .= "\t</div>\n";
			}

			return $content;
		}

	/**
	 * Entry Index Calendar
	 *
	 * @param int $m The month to display
	 * @param int $y The year to display
	 **/
	function buildIndexCalendar($y, $m)
	{
		$this->index_mode = TRUE;
		$entries = $this->getCalendarEntries($y, $m, TRUE);
		$cal = new PbCalendar($y, $m);
		$cal->setFirstWeekDay(0);
		$prev = '&lt;&lt; ' . date('M', mktime(0, 0, 0, $m - 1, 1,  $y));
		$next = date('M', mktime(0, 0, 0, $m + 1, 1,  $y)) . ' &gt;&gt;';
		$cal->enableMonthNav(BASE_URL . '?index=-1', $prev, $next);
		$cal->enableDatePicker(2002, date('Y'), BASE_URL, FALSE, array('index' => '-1')); //2002 is when PWL was released

		//--- Set CSS
		$cal->cssMonthTable = 'idx_month';
		$cal->cssMonthTitle = 'idx_monthname';
		$cal->cssMonthNav = 'idx_monthnavigation';
		$cal->cssPickerMonth = 'idx_monthpicker';
		$cal->cssYearTable = 'idx_year';
		$cal->cssYearTitle = 'idx_yeartitle';
		$cal->cssYearNav = 'idx_yearnavigation';
		$cal->cssPickerYear = 'idx_yearpicker';
		$cal->cssPickerYear = 'idx_yearpicker';
		$cal->cssPicker = 'idx_datepicker';
		$cal->cssPickerForm = 'idx_datepickerform';
		$cal->cssPickerButton = 'idx_pickerbutton';
		$cal->cssWeekDay = 'idx_dayname';
		$cal->cssNoMonthDay = 'idx_nomonthday';
		$cal->cssMonthDay = 'idx_monthday';
		$cal->cssPrefixTodayEvent = 'idx_today';
		$cal->cssSunday = 'idx_sunday';
		$cal->cssSaturday = 'idx_saturday';
		$cal->cssSelecDay = 'idx_selectedday';
		$cal->cssEvent = 'idx_event';
		$cal->cssPrefixSelecEvent = 'idx_selectedevent';
		$cal->cssPrefixTodayEvent = 'idx_todayevent';
		$cal->cssEventContent = 'idx_eventcontent';
		//$cal->cssPickerButton = 'idx_weeklycontent';
		//$cal->cssPickerButton = 'idx_emptycontent';

		//--- Set Entries
		foreach ($entries as $eid => $data) {
			$topic = '';
			if (isset($this->topics[$data['tid']])) {
				$topicObj = $this->topics[$data['tid']];
				$topic = $topicObj->getName();
			}
			$cal->setEventContent($y, $m, $data['day'], $data['title'], array(BASE_URL . '?eid=' . $eid, $topic . ': ' . $data['title'] . ' (' . $data['date'] . ')'), 'idx_eventcontent');
		}
		$this->display = '<div id="PbIdxCalendar" class="pb_nav-bar">' . "\n";
		$this->display .= '<div class="idx_arc_link">' . $this->buildLink('?index=1', $this->langObj->get('ARCHIVE_LINK'), '[' . $this->langObj->get('ARCHIVE_LINK') . ']') . "</div>\n";
		$this->display .= $cal->showMonth();
		$this->display .= "</div>\n";
	}

		//-----------------------------------------------
		//--- METHOD: buildIndex
		//--- PURPOSE: build index display
		//--- PARAMETERS: none
		//--- RETURNS: XHTML content
		//-----------------------------------------------
		function buildIndex()
		{
			$date_hdr = '';
			$topic = '';
			$content = '<div class="pb_entry">' . "\n";
			if (defined('USE_ARCHIVE_CAL') && USE_ARCHIVE_CAL) {
				$content .= '<p>' . $this->buildLink('?index=-1', $this->langObj->get('ARCHIVE_LINK_CAL'), '[' . $this->langObj->get('ARCHIVE_LINK_CAL') . ']') . "</p>\n";
			}
			//--- SPL Support
			if ($this->getSpl()) {
				foreach (new ArrayObject($this->entries) as $entryObj) {
					if (isset($this->topics[$entryObj->getTopicId()])) {
						$topicObj = $this->topics[$entryObj->getTopicId()];
						$topic = $topicObj->getName();
					}
					$entry_date = strftime('%B %Y', $entryObj->getUpdated());
					if ($date_hdr != $entry_date && empty($date_hdr)) {
						$date_hdr = $entry_date;
						$content .= <<< DATE_HDR_START
						<span class="pb_title">$entry_date</span>
						<br />
						<ul>
DATE_HDR_START;
					} elseif ($date_hdr != $entry_date) {
						$date_hdr = $entry_date;
						$content .= <<< DATE_HDR_CONT
						</ul>
						<br />
						<span class="pb_title">$entry_date</span>
						<br />
						<ul>
DATE_HDR_CONT;
					}
					$content .= '<li>' . $this->buildLink(BASE_URL . '?eid=' . $entryObj->getId(), $topic . ': ' . $entryObj->getTitle() . ' (' . $entryObj->getUpdated(TRUE) . ')', $entryObj->getTitle()) . '</li>' . "\n";
				}
			} else {
				foreach ($this->entries as $entryObj) {
					if (isset($this->topics[$entryObj->getTopicId()])) {
						$topicObj = $this->topics[$entryObj->getTopicId()];
						$topic = $topicObj->getName();
					}
					$entry_date = strftime('%B %Y', $entryObj->getUpdated());
					if ($date_hdr != $entry_date && empty($date_hdr)) {
						$date_hdr = $entry_date;
						$content .= <<< DATE_HDR_START
						<span class="pb_title">$entry_date</span>
						<br />
						<ul>
DATE_HDR_START;
					} elseif ($date_hdr != $entry_date) {
						$date_hdr = $entry_date;
						$content .= <<< DATE_HDR_CONT
						</ul>
						<br />
						<span class="pb_title">$entry_date</span>
						<br />
						<ul>
DATE_HDR_CONT;
					}
					$content .= '<li>' . $this->buildLink(BASE_URL . '?eid=' . $entryObj->getId(), $topic . ': ' . $entryObj->getTitle() . ' (' . $entryObj->getUpdated(TRUE) . ')', $entryObj->getTitle()) . '</li>' . "\n";
				}
			}
			$content .= '</ul>' . "\n";
			$content .= '</div>' . "\n";
			unset($this->entries);

			return $content;
		}

		//-----------------------------------------------
		//--- METHOD: buildTopicLink
		//--- PURPOSE: build entry topic header
		//--- PARAMETERS: topic Id
		//--- RETURNS: XHTML content
		//-----------------------------------------------
		function buildTopicLink($tid)
		{
			$link = '';

			if (isset($this->topics[$tid])) {
				$icon = $this->topics[$tid]->getIcon();
				if (!empty($icon)) {
					$icon = $this->buildTopicIcon($icon, $this->topics[$tid]->getName());
					$link = $this->buildLink(BASE_URL . '?tid=' . $tid, $this->langObj->get('TOPIC') . ': ' . $this->topics[$tid]->getName(), $icon);
				}
			}

			return $link;
		}

		//-----------------------------------------------
		//--- METHOD: buildTopicIcon
		//--- PURPOSE: build a topic icon image tag
		//--- PARAMETERS: topic src
		//--- 				topic alt
		//--- RETURNS: XHTML hyperlink
		//-----------------------------------------------
		function buildTopicIcon($src, $topic, $class = 'pb_topic_icon_image')
		{
			$topic_size = '';
			if (defined('TOPIC_ICON_WIDTH') && defined('TOPIC_ICON_HEIGHT')) {
				$topic_size =  ' width="' . TOPIC_ICON_WIDTH . '" height="' . TOPIC_ICON_HEIGHT .'"';
			}
			return '<img class="' . $class . '" src="' . $src . '" alt="' . $this->langObj->get('TOPIC') . ': ' . $topic . '"' . $topic_size . ' />';
		}

		//-----------------------------------------------
		//--- METHOD: buildNavBar
		//--- PURPOSE: build the navigation bar
		//--- PARAMETERS: display start position
		//--- 				search string
		//--- 				topic id
		//--- RETURNS: XHTML content
		//-----------------------------------------------
		function buildNavBar($start, $search, $tid = FALSE)
		{
			$param = ($this->index_mode) ? 'index' : 'start';

			$nav_bar = '<div class="pb_content">' . "\n";
			$nav_bar .= "\t<table>\n";
			$nav_bar .= "\t\t<tr>\n";
			$nav_bar .= "\t\t <td colspan=\"3\">\n";
			$nav_bar .= "\t\t <form action=\"" . BASE_URL . "\" method=\"post\">\n";
			$nav_bar .= "\t\t <p>";
			$search = htmlentities(stripslashes($search), ENT_QUOTES, I18N_CHARSET);
			$nav_bar .= "\t\t <input type=\"text\" name=\"search\" value=\"{$search}\" size=\"50\" />\n";
			$nav_bar .= "\t\t " . '&nbsp;<input type="submit" name="search_btn" value="' . $this->langObj->get('SEARCH') . '" />' . "\n";
			$nav_bar .= "\t\t </p></form>\n";
			$nav_bar .= "\t\t </td>\n";
			$nav_bar .= "\t\t</tr>\n";
			$nav_bar .= "\t\t<tr>\n";
			$nav_bar .= "\t\t <th class=\"pb_align_left\">\n";

			if ($start > 0) {
				$prev_start = $start - $this->display_cnt;
				$nav_bar .= "\t\t  <a href=\"" . BASE_URL . "?$param=$prev_start";
				$nav_bar .= ($tid) ? "&amp;tid=$tid\">" : '">';
				$nav_bar .= '<img class="pb_no_border" src="' . PREV_ICON . '" alt="' . $this->langObj->get('PREV_ENTRIES') . '" />';
				$nav_bar .= (PREV_TEXT) ? '&nbsp;' . $this->langObj->get('PREVIOUS') . '&nbsp;' : '';
				$nav_bar .= '</a>';
			} else {
				$nav_bar .= "<img src=\"gfx/blank.png\" alt=\"\" />";
			}
			$nav_bar .= "\n\t\t </th>";
			$nav_bar .= "\n\t\t <th>";
			$nav_bar .= "\t\t <form action=\"" . BASE_URL . "\" method=\"get\">\n";
			$nav_bar .= '<p>&nbsp;<input type="submit" name="recent" value="' . $this->langObj->get('DISPLAY_RECENT_ENTRIES') . '" />' . "\n";
			$nav_bar .= "\t\t </p></form>\n";
			$nav_bar .= "\n\t\t </th>";
			$nav_bar .= "\t\t <th class=\"pb_align_right\">\n";

			if (empty($search) && $start <= $this->getEntryCnt() && $start + $this->display_cnt <= $this->getEntryCnt()) {
				$next_start = $start + $this->display_cnt;
				$nav_bar .= "\t\t  <a href=\"" . BASE_URL . "?$param=$next_start";
				$nav_bar .= ($tid) ? "&amp;tid=$tid\">" : '">';
				$nav_bar .= (NEXT_TEXT) ? '&nbsp;' . $this->langObj->get('NEXT') . '&nbsp;' : '';
				$nav_bar .= '<img class="pb_no_border" src="' . NEXT_ICON . '" alt="' . $this->langObj->get('NEXT_ENTRIES') . '" />';
				$nav_bar .= '</a>';
			} else {
				$nav_bar .= (NEXT_TEXT) ? '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' : '';
			}
			$nav_bar .= "\n\t\t </th>\n";
			$nav_bar .= "\t\t</tr>\n";
			$nav_bar .= "\t</table>\n";
			$nav_bar .= "</div>\n";

			return $nav_bar;
		}

		//-----------------------------------------------
		//--- METHOD: searchEntries
		//--- PURPOSE: set entry ID that match search criteria
		//--- PARAMETERS: search text
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function searchEntries($search)
		{
			$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? "AND display='1'" : '';
			$post_dated = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? ' AND updated<=UNIX_TIMESTAMP()' : '';
			$boolean_search = (defined('BOOLEAN_SEARCH') && BOOLEAN_SEARCH) ? ' IN BOOLEAN MODE' : '';
			$query = "SELECT
							eid
						 FROM
						 	{$this->prefix}entries
						 WHERE
						 	MATCH (title,body,more)
						 	AGAINST ('{%s}'{$boolean_search})
						 $display_flag
						 $post_dated";
			$query_data = array($search);

			//--- Issue Query
			if ($this->dbObj->safeQuery($query, $query_data)) {
				//--- Check for results
				if ($this->dbObj->getResultCnt() > 0) {
					// Hack to prevent running out of memory because of poor architectual decision.  :(
					$this->entry_cnt = ($this->dbObj->getResultCnt() > 50) ? 50 : (int) $this->dbObj->getResultCnt();

					//--- Retrieve Results
					$i = 0;
					while ($this->dbObj->getRow()) {
						$eid = (int) $this->dbObj->row['eid'];
						$this->entries[$i] = new PbEntry($this->getDbPrefix(), $eid);
						$i++;
					}
				}

				$stat = TRUE;

			} else {

				//--- Query Error
				echo $this->dbObj->getError();
				$stat = FALSE;
			}

			return $stat;
		}




		//-----------------------------------------------
		//-----------------------------------------------
		//--- Calendar Methods -------------------
		//-----------------------------------------------
		//-----------------------------------------------


	/**
	 * Build entries for the navigation calendar
	 *
	 * @param int $m The month to display
	 * @param int $y The year to display
	 * @param bool $ext Retrieve extended information (for index display)
	 * @return array Associative array with the day as the key and an entry count of for each day.
	 **/
 	function getCalendarEntries($y, $m, $ext = FALSE)
 	{
		$days = array();

		//--- Look for blog entries on this day
		$start_day = mktime(0, 0, 0, $m, 1, $y);
		$days_in_month = cal_days_in_month(CAL_GREGORIAN, $m, $y);
		$end_day = mktime(23, 59 ,59 ,$m ,$days_in_month, $y);
		$display_flag = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? "AND display='1'" : '';
		$post_dated = (!(isset($_SESSION['login']) && $_SESSION['login'])) ? ' AND updated<=UNIX_TIMESTAMP()' : '';
		$cols = ($ext) ? ', eid, title, tid, updated, SUBSTRING(body,1,50) AS snippet' : '';
		$query = "SELECT
					FROM_UNIXTIME(updated, '%d') AS day
	   				$cols
	   			 FROM
	   			 	{$this->prefix}entries
	   			 WHERE
	   			 	(updated >='%i'
	   			 		AND
	   			 	updated <= '%i')
	   			 $display_flag
	   			 $post_dated";
	   	$query_data = array($start_day, $end_day);
		//--- Issue Query
		if ($this->dbObj->safeQuery($query, $query_data) && $this->dbObj->getResultCnt() > 0)  {
			//--- Retrieve Results
			while ($this->dbObj->getRow()) {
				if ($ext) {
					$date = (defined('I18N_DATE_FORMAT')) ? strftime(I18N_DATE_FORMAT, $this->dbObj->row['updated'] + TIME_ADJUST) : date(DATE_FORMAT, $this->dbObj->row['updated'] + TIME_ADJUST);
					$days[$this->dbObj->row['eid']] = array(
													'tid'		=> $this->dbObj->row['tid'],
													'day'		=> $this->dbObj->row['day'],
													'date'		=> $date,
													'title'		=> $this->dbObj->row['title'],
													'snippet'	=> $this->dbObj->row['snippet']
											 	     );
				} else {
					$day = (int) $this->dbObj->row['day'];
					if (isset($days[$day])) {
						$days[$day]++;
					} else {
						$days[$day] = 1;
					}
				}
			}
		} else {
			//--- Query Error
			echo $this->dbObj->getError();
		}

		return $days;
	}

	/**
	 * Navigation Calendar
	 *
	 * @param int $m The month to display
	 * @param int $y The year to display
	 * @return string Calendar HTML
	 **/
	function buildCalendar($y, $m)
	{
		$days = $this->getCalendarEntries($y, $m);
		$cal = new PbCalendar($y, $m);
		$cal->setFirstWeekDay(0);
		$cal->enableDatePicker(2002, date('Y'), BASE_URL); //2002 is when PWL was released

		//--- Set Entries
		$today = date('Ymd', time());
		foreach ($days as $day => $cnt)  {
			$event = BASE_URL . '?date=' . $y . sprintf("%02d", $m) . sprintf("%02d", $day) . '&amp;y=' . $y . '&amp;m=' . $m;
	   	   	$showdate = $y . sprintf("%02d", $m) . sprintf("%02d", $day);
	   	  	$link_style = ($showdate == $today) ? '_pb_cal_link' : 'pb_cal_link';
	   	  	$entry_cnt = ($cnt > 1) ? $cnt . ' ' . $this->langObj->get('ENTRIES') : $cnt . ' ' . $this->langObj->get('ENTRY');
			$cal->setEvent($y, $m, $day, $link_style, $event, $entry_cnt);
		}

		$return = $cal->showMonth();

		#--- determine next and previous month and year
		$prev_month_txt = $this->get_month($m, -1);
		$next_month_txt = $this->get_month($m, 1);

		if(($m - 1) < 1) {
			$prev_month = 12;
			$prev_year = $y - 1;
		} else {
			$prev_month = $m - 1;
			$prev_year = $y;
		}
		if(($m + 1) > 12) {
			$next_month = 1;
			$next_year = $y + 1;
		} else {
			$next_month = $m + 1;
			$next_year = $y;
		}

		$prev_month = sprintf("%02d", $prev_month);
		$next_month = sprintf("%02d", $next_month);

		$base_url = BASE_URL;
		$return .= <<<CAL_NAV
		 <table width="100%">
		 <tr>
		  <th align="left"><a href="$base_url?m=$prev_month&amp;y=$prev_year">&lt; $prev_month_txt</a></th>
		  <td>&nbsp;</td>
		  <th align="right"><a href="$base_url?m=$next_month&amp;y=$next_year">$next_month_txt &gt;</a></th>
		 </tr>
		</table>
CAL_NAV;

		return $return;
		}

		/*== get previous/current/next month abreviation ==*/
		function get_month($month, $pos = 0) {

			//--- get a timestamp from the middle of the month (but not from <= 1970!)
			$timestamp = mktime(12, 0, 0, 1, 15, 1980);
			$month_offset = 2592000; // 30 days

			//--- Remove leading zero if present
			$month = trim(sprintf('%2d', $month));
			$month--;

			if ($pos == -1 || $pos == 1) {
				$month += $pos;
			}
			$timestamp += $month * $month_offset;

			return strftime('%h', $timestamp);
		}

		/*== get the last day of the month ==*/
		function get_last_month_day($mon,$year)
		{
		    for ($tday=28; $tday <= 31; $tday++)  {
	        $tdate = getdate(mktime(0,0,0,$mon,$tday,$year));
	        if ($tdate["mon"] != $mon) { break; }

		    }
		    $tday--;

		    return $tday;
		}

		//-----------------------------------------------
		//--- METHOD: checkLogin
		//--- PURPOSE: verify login
		//--- PARAMETERS: username
		//--- 				password
		//--- RETURNS: TRUE/FALSE
		//-----------------------------------------------
		function checkLogin($user, $passwd, $reverify = FALSE)
		{
			$magic = sha1($_SERVER['HTTP_USER_AGENT'] . DB_PASSWD);

			if ($reverify) {
				if ($passwd === $magic) {
					$stat = TRUE;
				} else {
					$stat= FALSE;
					$_SESSION['login'] = FALSE;
					$_SESSION['magic'] = FALSE;
				}
			} else {
				$passwd = md5(md5($passwd));
				$query = "SELECT
								id,
								email,
								post_by,
								post_entry,
								post_comment,
								admin
							 FROM
						 		{$this->prefix}users
						 	 WHERE
								user='%s'
							 AND
								password='%s'
							 LIMIT 1";
				$query_data = array($user, $passwd);
				//--- Issue Query
				if ($this->dbObj->safeQuery($query, $query_data) && $this->dbObj->getResultCnt() > 0) {
					//--- Retrieve Results
					$row = $this->dbObj->getRow();
					$_SESSION['login'] = TRUE;
					$_SESSION['magic'] = $magic;
					$_SESSION['post_by'] = $row['post_by'];
					$_SESSION['uid'] = (int) $row['id'];
					$_SESSION['post_entry'] = ((int) $row['post_entry'] == 1) ? TRUE : FALSE;
					$_SESSION['post_comment'] = ((int) $row['post_comment'] == 1) ? TRUE : FALSE;
					$_SESSION['admin'] = ((int) $row['admin'] == 1) ? TRUE : FALSE;
					$stat = TRUE;
	 			} else {
					//--- Query Error
					echo $this->dbObj->getError();
					$_SESSION['login'] = FALSE;
					$_SESSION['magic'] = FALSE;
					$stat = FALSE;
				}
			}

			return $stat;
		}

		/**
		 * Email password hint to a user if his email address exists.
		 *
		 * @param string $user Username
		 * @return boolean
		 */
		function emailHint($user)
		{
			$stat = FALSE;
			$query = "SELECT
							email,
							post_by,
							hint
						 FROM
					 		{$this->prefix}users
					 	 WHERE
							user='%s'
						 LIMIT 1";
			$query_data = array($user);

			if (MAIL_TYPE != FALSE && $this->dbObj->safeQuery($query, $query_data) && $this->dbObj->getResultCnt() > 0) {
				$row = $this->dbObj->getRow();
				$email = trim($row['email']);

				if ($email) {
					require_once(LIB_PATH . 'class.phpmailer.php');
					$blog_name = (defined('BLOG_NAME')) ? BLOG_NAME : 'PolarBlog';
					$mail = new PHPMailer();
					if (defined('MAIL_HOST')) {
						$mail->Host = MAIL_HOST;
						$mail_debug_msg = $mail->ErrorInfo . '<br />';
					}
					$mail->CharSet = I18N_CHARSET;
					$mail->Mailer = MAIL_TYPE;
					$mail_debug_msg .= $mail->ErrorInfo . '<br />';
					$mail->From = MAIL_FROM;
					$mail_debug_msg .= $mail->ErrorInfo . '<br />';
					$mail->FromName = MAIL_FROM_NAME;
					$mail_debug_msg .= $mail->ErrorInfo . '<br />';
					$mail->Subject = $blog_name . ' ' . $this->langObj->get('PASSWORD_HINT');
					$mail_debug_msg .= $mail->ErrorInfo . '<br />';

					if (strlen($row['email']) > 0) {
						$mail->AddAddress($email, $row['post_by']);
						$mail_debug_msg .= $mail->ErrorInfo . '<br />';
						$stat = TRUE;
					}

					$mail->Body = $this->langObj->get('PASSWORD_EMAIL_TEXT') . "\r\n\r\n";
					$mail->Body .= $row['hint']  . "\r\n\r\n\r\n";
					$for = $blog_name;
					$for .= ' (' . BASE_URL . ')';
					$for = stripslashes(strip_tags($for));
					$mail->Body .= $this->langObj->get('MAIL_TAG') . " $for.\r\n";
					$mail_debug_msg .= $mail->ErrorInfo . '<br />';

					if ($stat) {
						$stat = $mail->Send();
					}

					if (defined('MAIL_DEBUG') && MAIL_DEBUG) {
						echo 'Mail Debug:<br />' . $mail_debug_msg;
					}
				}
			}

			return $stat;
		}

		/**
		 * Set the RBL Adminstration Display
		 */
		function setRblAdminDisplay()
		{
			$this->setRblObj();

			//--- Set Headers
			if (defined('TOP_HEADER')) {
				$this->accessory_display .= '<div id="PbTopHeader" class="pb_nav-bar">';
				$this->accessory_display .= TOP_HEADER;
				$this->accessory_display .= '</div>';
			}
			$this->accessory_display .= '<div id="PbSubHeader" class="pb_sh_h1">';
			$this->accessory_display .= $this->langObj->get('RBL_ADMIN_HDR');
			$this->accessory_display .= '&nbsp;&nbsp;' . $this->buildLink(BASE_URL, $this->langObj->get('EXIT_RBL'), '[' . $this->langObj->get('EXIT_RBL') . ']');
			$this->accessory_display .= '</div>';

			//--- Comment Display Disabled
			$this->display = '<div class="pb_edit">';
			$this->display .= '<form action="' . BASE_URL . '" method="post">' . "\n";
			$this->display .= '<table class="pb_rbl_table">';
			$this->display .= '<caption>' . $this->langObj->get('COMMENT_DISABLED') . '</caption>';
			$this->display .= '<thead>';
			$this->display .= '<tr>';
			$this->display .= '<th>' . $this->langObj->get('ACTION') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('IP_ADDRESS');
			$this->display .= '<input type="hidden" name="admin" value="rbl_admin" />';
			$this->display .= '<input type="hidden" name="rbl_admin_action" value="comment_action" />';
			$this->display .= "</th>\n";
			$this->display .= '<th>' . $this->langObj->get('POSTED') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('FROM') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('EXCERPT') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('TRACKBACK') . '</th>';
			$this->display .= '</tr>';
			$this->display .= '</thead>';
			$this->display .= '<tfoot class="ip_tfoot">';
			$this->display .= '<tr><td>&nbsp;</td></tr>';
			$this->display .= '</tfoot>';
			$this->display .= '<tbody class="ip_tbody">';
			$cnt = 0;

			$no_display_comments = $this->getCommentsNoDisplay();
			if (count($no_display_comments) === 0) {
				$this->display .= '<tr class="pb_rbl_row1">';
				$this->display .= '<td colspan="6">';
				$this->display .= $this->langObj->get('COMMENTS_NO_DISABLED');
				$this->display .= '</td>';
				$this->display .= '</tr>';
			} else {
				//--- SPL Support
				if ($this->getSpl()) {
					foreach (new ArrayObject($no_display_comments) as $comment) {
						$row_style = ($cnt % 2) ? 'pb_rbl_row2' : 'pb_rbl_row1';
						$tb = ($comment['trackback'] === '1') ? $this->langObj->get('YES') : $this->langObj->get('NO');
						$ref_link = $this->langObj->get('NONE');
						if (!empty($comment['referer'])) {
							$x = htmlentities($comment['referer'], ENT_QUOTES, I18N_CHARSET);
							$ref_link = $this->buildLink(BASE_URL . '?eid=' . $comment['eid'] . '#' . $comment['cid'], $this->langObj->get('REF_FROM') . ': ' . $x, $this->langObj->get('VIEW'));
						}
						if (defined('RBL_IP_WHOIS')) {
							$ip = '<a href="' . str_replace('::IP::', $comment['ip'], RBL_IP_WHOIS) . '" title="' . $this->langObj->get('RBL_IP_LOOKUP') . '">'  . $comment['ip'] . '</a>';
						} else {
							$ip = $comment['ip'];
						}
						$e_url = '<a href="' . BASE_URL . '?eid=' . $comment['eid'] . '#' . $comment['cid'] . '" title="' . $this->langObj->get('COMMENT_VIEW_FULL') . '">';
						$this->display .= '<tr class="' . $row_style . '">';
						$this->display .= '<td>';
						$this->display .= '<input type="checkbox" value="' . $comment['cid'] . '" name="cids[' . $comment['cid'] . ']" />';
						$this->display .= '</td>';
						$this->display .= '<td >' . $ip . '</td>';
						$this->display .= '<td>' . substr($comment['timestamp'], 0, -3) . '</td>';
						$this->display .= '<td align="right">' . $comment['name'] . '</td>';
						$this->display .= '<td>' . $e_url . $comment['excerpt'] . '</a></td>';
						$this->display .= '<td>' . $tb . '</td>';
						$this->display .= '</tr>';
						$cnt++;
					}
				} else {
					foreach ($no_display_comments as $comment) {
						$row_style = ($cnt % 2) ? 'pb_rbl_row2' : 'pb_rbl_row1';
						$tb = ($comment['trackback'] === '1') ? $this->langObj->get('YES') : $this->langObj->get('NO');
						$ref_link = $this->langObj->get('NONE');
						if (!empty($comment['referer'])) {
							$x = htmlentities($comment['referer'], ENT_QUOTES, I18N_CHARSET);
							$ref_link = $this->buildLink(BASE_URL . '?eid=' . $comment['eid'] . '#' . $comment['cid'], $this->langObj->get('REF_FROM') . ': ' . $x, $this->langObj->get('VIEW'));
						}
						if (defined('RBL_IP_WHOIS')) {
							$ip = '<a href="' . str_replace('::IP::', $comment['ip'], RBL_IP_WHOIS) . '" title="' . $this->langObj->get('RBL_IP_LOOKUP') . '">'  . $comment['ip'] . '</a>';
						} else {
							$ip = $comment['ip'];
						}
						$e_url = '<a href="' . BASE_URL . '?eid=' . $comment['eid'] . '#' . $comment['cid'] . '" title="' . $this->langObj->get('COMMENT_VIEW_FULL') . '">';
						$this->display .= '<tr class="' . $row_style . '">';
						$this->display .= '<td>';
						$this->display .= '<input type="checkbox" value="' . $comment['cid'] . '" name="cids[' . $comment['cid'] . ']" />';
						$this->display .= '</td>';
						$this->display .= '<td >' . $ip . '</td>';
						$this->display .= '<td>' . substr($comment['timestamp'], 0, -3) . '</td>';
						$this->display .= '<td align="right">' . $comment['name'] . '</td>';
						$this->display .= '<td>' . $e_url . $comment['excerpt'] . '</a></td>';
						$this->display .= '<td>' . $tb . '</td>';
						$this->display .= '</tr>';
						$cnt++;
					}
				}
			}
			$this->display .= '</tbody>';
			$this->display .= '</table>';
			if (count($no_display_comments) !== 0) {
				$this->display .= '<p><input type="checkbox" onclick="if (this.checked) { checkAll(); } else { uncheckAll(); }" />' . $this->langObj->get('ALL');
				$this->display .= '&nbsp;<input type="submit" value="' . $this->langObj->get('DISPLAY') . '" name="comment_display" />&nbsp;';
				$this->display .= '<input type="submit" value="' . $this->langObj->get('DELETE') . '" name="comment_delete" />&nbsp;&nbsp;';
				$this->display .= '(' . count($no_display_comments) . ' ' . $this->langObj->get('ENTRIES') . ')&nbsp;';
			}
			$this->display .= "</form>\n";
			$this->display .= '</div>' . "\n";

			//--- Set IP Display
			$this->display .= '<div class="pb_edit">';
			$this->display .= '<form action="' . BASE_URL . '" method="post">' . "\n";
			$this->display .= '<table class="pb_rbl_table">';
			$this->display .= '<caption>' . $this->langObj->get('LISTED_IPS') . '</caption>';
			$this->display .= '<thead>';
			$this->display .= '<tr>';
			$this->display .= '<th>' . $this->langObj->get('IP_ADDRESS');
			$this->display .= '<input type="hidden" name="admin" value="rbl_admin" />';
			$this->display .= '<input type="hidden" name="rbl_admin_action" value="ip_edit" />';
			$this->display .= "</th>\n";
			$this->display .= '<th>' . $this->langObj->get('LAST_VISIT') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('VISITS') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('RBLS') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('REFERRER') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('WHITELISTED') . '</th>';
			$this->display .= '<th>&nbsp;&nbsp;' . $this->langObj->get('ACTION') . '&nbsp;&nbsp;</th>';
			$this->display .= '</tr>';
			$this->display .= '</thead>';
			$this->display .= '<tfoot class="ip_tfoot">';
			$this->display .= '<tr><td>&nbsp;</td></tr>';
			$this->display .= '</tfoot>';
			$this->display .= '<tbody class="ip_tbody">';
			$cnt = 0;
			//--- SPL Support
			if ($this->getSpl()) {
				foreach (new ArrayObject($this->rblObj->getLocalRbls()) as $rbl) {
					$row_style = ($cnt % 2) ? 'pb_rbl_row2' : 'pb_rbl_row1';
					$wl = ($rbl['whitelist']) ? $this->langObj->get('YES') : $this->langObj->get('NO');
					$ref_link = $this->langObj->get('NONE');
					if (!empty($rbl['referer'])) {
						$x = htmlentities($rbl['referer'], ENT_QUOTES, I18N_CHARSET);
						$ref_link = $this->buildLink($x, $this->langObj->get('REF_FROM') . ': ' . $x, $this->langObj->get('VIEW'));
					}
					if (defined('RBL_IP_WHOIS')) {
						$ip = '<a href="' . str_replace('::IP::', $rbl['ip'], RBL_IP_WHOIS) . '" title="' . $this->langObj->get('RBL_IP_LOOKUP') . '">'  . $rbl['ip'] . '</a>';
					} else {
						$ip = $rbl['ip'];
					}
					$this->display .= '<tr class="' . $row_style . '">';
					$this->display .= '<td >' . $ip . '</td>';
					$this->display .= '<td>' . substr($rbl['lastseen'], 0, -3) . '</td>';
					$this->display .= '<td align="right">' . $rbl['visits'] . '</td>';
					$this->display .= '<td>' . $rbl['services'] . '</td>';
					$this->display .= '<td>' . $ref_link . '</td>';
					$this->display .= '<td>' . $wl . '</td>';
					$this->display .= '<td>&nbsp;&nbsp;';
					$this->display .= '<input type="submit" value="' . $this->langObj->get('DELETE') . '" title="' . $this->langObj->get('DELETE') . '" name="del-' . $rbl['id'] . '" />&nbsp;';
					if ($rbl['whitelist']) {
						$this->display .= '<input type="submit" value="BL" title="' . $this->langObj->get('BLACKLIST_IP') . '" name="bl-' . $rbl['id'] . '" />';
					} else {
						$this->display .= '<input type="submit" value="WL" title="' . $this->langObj->get('WHITELIST_IP') . '" name="wl-' . $rbl['id'] . '" />';
					}
					$this->display .= '&nbsp;&nbsp;</td>';
					$this->display .= '</tr>';
					$cnt++;
				}
			} else {
				foreach ($this->rblObj->getLocalRbls() as $rbl) {
					$row_style = ($cnt % 2) ? 'pb_rbl_row2' : 'pb_rbl_row1';
					$wl = ($rbl['whitelist']) ? $this->langObj->get('YES') : $this->langObj->get('NO');
					$ref_link = '';
					if (!empty($rbl['referer'])) {
						$x = htmlentities($rbl['referer'], ENT_QUOTES, I18N_CHARSET);
						$ref_link = $this->buildLink($x, $this->langObj->get('REF_FROM') . ': ' . $x, $x);
					}
					if (defined('RBL_IP_WHOIS')) {
						$ip = '<a href="' . str_replace('::IP::', $rbl['ip'], RBL_IP_WHOIS) . '" title="' . $this->langObj->get('RBL_IP_LOOKUP') . '">'  . $rbl['ip'] . '</a>';
					} else {
						$ip = $rbl['ip'];
					}
					$this->display .= '<tr class="' . $row_style . '">';
					$this->display .= '<td >' . $ip . '</td>';
					$this->display .= '<td>' . substr($rbl['lastseen'], 0, -3) . '</td>';
					$this->display .= '<td align="right">' . $rbl['visits'] . '</td>';
					$this->display .= '<td>' . $rbl['services'] . '</td>';
					$this->display .= '<td>' . $ref_link . '</td>';
					$this->display .= '<td>' . $wl . '</td>';
					$this->display .= '<td>&nbsp;&nbsp;';
					$this->display .= '<input type="submit" value="' . $this->langObj->get('DELETE') . '" title="' . $this->langObj->get('DELETE') . '" name="del-' . $rbl['id'] . '" />&nbsp;';
					if ($rbl['whitelist']) {
						$this->display .= '<input type="submit" value="BL" title="' . $this->langObj->get('BLACKLIST_IP') . '" name="bl-' . $rbl['id'] . '" />';
					} else {
						$this->display .= '<input type="submit" value="WL" title="' . $this->langObj->get('WHITELIST_IP') . '" name="wl-' . $rbl['id'] . '" />';
					}
					$this->display .= '&nbsp;&nbsp;</td>';
					$this->display .= '</tr>';
					$cnt++;
				}
			}

			$this->display .= '</tbody>';
			$this->display .= '</table>';
			$this->display .= '<p><input type="text" value="" name="rbl_ip" />&nbsp;';
			$this->display .= '<input type="submit" value="' . $this->langObj->get('ADD') . '" name="add" />&nbsp;&nbsp;';
			if (count($this->rblObj->getLocalRbls()) > 0) {
				$this->display .= '(' . count($this->rblObj->getLocalRbls()) . ' ' . $this->langObj->get('ENTRIES') . ')</p>';
			} else {
				$this->display .= '</p>';
			}
			$this->display .= "</form>\n";
			$this->display .= '</div>' . "\n";

			//--- Set Keyword Display
			$this->display .= '<div class="pb_edit">';
			$this->display .= '<form action="' . BASE_URL . '" method="post">' . "\n";
			$this->display .= '<table class="pb_rbl_table">';
			$this->display .= '<caption>' . $this->langObj->get('LISTED_KEYWORDS') . '</caption>';
			$this->display .= '<thead>';
			$this->display .= '<tr>';
			$this->display .= '<th>' . $this->langObj->get('KEYWORD');
			$this->display .= '<input type="hidden" name="admin" value="rbl_admin" />';
			$this->display .= '<input type="hidden" name="rbl_admin_action" value="keyword_edit" />';
			$this->display .= "</th>\n";
			$this->display .= '<th>' . $this->langObj->get('OCCURRENCES') . '</th>';
			$this->display .= '<th>' . $this->langObj->get('LAST_VISIT') . '</th>';
			$this->display .= '<th>&nbsp;&nbsp;' . $this->langObj->get('ACTION') . '&nbsp;&nbsp;</th>';
			$this->display .= '</tr>';
			$this->display .= '</thead>';
			$this->display .= '<tfoot class="ip_tfoot">';
			$this->display .= '<tr><td>&nbsp;</td></tr>';
			$this->display .= '</tfoot>';
			$this->display .= '<tbody class="ip_tbody">';
			//--- SPL Support
			if ($this->getSpl()) {
				foreach (new ArrayObject($this->rblObj->getAllBlockedKeywords()) as $rbl) {
					$row_style = ($cnt % 2) ? 'pb_rbl_row2' : 'pb_rbl_row1';
					$this->display .= '<tr class="' . $row_style . '">';
					$this->display .= '<td>' . $rbl['keyword'] . '</td>';
					$this->display .= '<td align="right">' . $rbl['occurances'] . '</td>';
					$this->display .= '<td>' . substr($rbl['lastseen'], 0, -3) . '</td>';
					$this->display .= '<td>&nbsp;&nbsp;';
					$this->display .= '<input type="submit" value="' . $this->langObj->get('DELETE') . '" title="' . $this->langObj->get('DELETE') . '" name="del-' . $rbl['id'] . '" />&nbsp;';
					$this->display .= '&nbsp;&nbsp;</td>';
					$this->display .= '</tr>';
					$cnt++;
				}
			} else {
				foreach ($this->rblObj->getAllBlockedKeywords() as $rbl) {
					$row_style = ($cnt % 2) ? 'pb_rbl_row2' : 'pb_rbl_row1';
					$this->display .= '<tr class="' . $row_style . '">';
					$this->display .= '<td>' . $rbl['keyword'] . '</td>';
					$this->display .= '<td align="right">' . $rbl['occurances'] . '</td>';
					$this->display .= '<td>' . substr($rbl['lastseen'], 0, -3) . '</td>';
					$this->display .= '<td>&nbsp;&nbsp;';
					$this->display .= '<input type="submit" value="' . $this->langObj->get('DELETE') . '" title="' . $this->langObj->get('DELETE') . '" name="del-' . $rbl['id'] . '" />&nbsp;';
					$this->display .= '&nbsp;&nbsp;</td>';
					$this->display .= '</tr>';
					$cnt++;
				}
			}

			$this->display .= '</tbody>';
			$this->display .= '</table>';
			$this->display .= '<p><input type="text" value="" name="rbl_keyword" />&nbsp;';
			$this->display .= '<input type="submit" value="' . $this->langObj->get('ADD') . '" name="add" />&nbsp;&nbsp;';
			if (count($this->rblObj->getAllBlockedKeywords()) > 0) {
				$this->display .= '(' . count($this->rblObj->getAllBlockedKeywords()) . ' ' . $this->langObj->get('ENTRIES') . ')</p>';
			} else {
				$this->display .= '</p>';
			}
			$this->display .= "</form>\n";
			$this->display .= '</div>' . "\n";
		}

		/**
		 * Sets an RBL object
		 * @return void
		 */
		function setRblObj()
		{
			require_once(LIB_PATH . 'PbRbl_class.php');
			if (!is_object($this->rblObj)) {
				$this->rblObj = new PbRbl($this->prefix);
			}
		}
	} //end of class
?>
