<?php

// This program is distributed under the terms of the GNU General Public License.
// See the README & LICENSE files for more information.
// Copyright (C) 2001-2005  Kevin L. Papendick <kevinp@polarlava.com>

//--- REVISIONS
// 20031030 - klp
//	Fixed syntax error in USAGE example!
//	Added support for those lacking BC Math support with
//	Added rounding of millisecond time display to 3 digit precision in getPrettyTime()
//	BUG FIX: In setPageTime() calling getEndTime() as "getEndTime"! 20051109 -klp



//------------------------------------------------------------------
//--- CLASS: Timer
//--- PURPOSE: Calculate page generation time
//---
//--- USAGE:
//---
/*
			<?php
				require('Timer.php');
				$page_timer = new Timer();
				//$page_timer = new Timer('NO_BC_MATH');
			?>

			[ --- Page Content --- ]

			<?php
				$ptime = $page_timer->getPageTime();
				echo 'This page took ' . $ptime . ' to generate.';
			?>

*/
//------------------------------------------------------------------
class Timer {

	//--- Class Variables
	var $start_time;		//time page started
	var $end_time;			//time page ended
	var $page_time;		//page generation time
	var $no_bc_math;		//don't use BC Math Functions


	//--------------------------------------------------
	//--- CONSTRUCTOR
	//--- PURPOSE:
	//---   Create a Timer object & start timing
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//--------------------------------------------------
	function Timer($no_bc_math = '')
	{
		//--- Don't use BC Math?
		$this->no_bc_math = ($no_bc_math === 'NO_BC_MATH') ? true : false;

		//--- Start Timer
		$this->setStartTime();
	}


	//-----------------------------------------------
	//--- METHOD: setStartTime
	//--- PURPOSE: create page start time mark
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setStartTime()
	{
		$time = $this->getTime();
		$this->start_time = $time[1] . substr($time[0], 1);
	}


	//-----------------------------------------------
	//--- METHOD: setEndTime
	//--- PURPOSE: create page end time mark
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setEndTime()
	{
		$time = $this->getTime();
		$this->end_time = $time[1] . substr($time[0], 1);
	}


	//-----------------------------------------------
	//--- METHOD: setPageTime
	//--- PURPOSE: calculate & set page generation time
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setPageTime()
	{
		//--- Set end time
		if ($this->getEndTime() == '') {
			$this->setEndTime();
		}

		//--- Calculate page time
		if ($this->no_bc_math) {
			$this->page_time = (float) $this->getEndTime() - (float) $this->getStartTime();
		} else {
			$this->page_time = bcsub($this->getEndTime(), $this->getStartTime(), 5);
		}
 	}




	//-----------------------------------------------
	//--- METHOD: getTime
	//--- PURPOSE: create microtime array
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function getTime()
	{
		$t = explode(" ", microtime());

		return $t;
	}


	//-----------------------------------------------
	//--- METHOD: getStartTime
	//--- PURPOSE: get page start time
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function getStartTime()
	{
		return $this->start_time;
	}


	//-----------------------------------------------
	//--- METHOD: getEndTime
	//--- PURPOSE: get page end time
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function getEndTime()
	{
		return $this->end_time;
	}


	//-----------------------------------------------
	//--- METHOD: getPageTime
	//--- PURPOSE: create
	//--- PARAMETERS: format
	//--- 				raw (secs) or pretty (sec/msec string)
	//--- RETURNS: nothing
	//-----------------------------------------------
	function getPageTime($format = 'pretty')
	{
		//--- Calculate Page Time
		if ($this->page_time == '') {
			$this->setPageTime();
		}


		if ($format == 'pretty') {
			return $this->getPrettyTime();
		} else {
			return $this->page_time;
		}
	}


	//-----------------------------------------------
	//--- METHOD: getPrettyTime
	//--- PURPOSE: format page time string
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function getPrettyTime()
	{
		if ($this->page_time < 1) {
			$t = round($this->page_time * 1000, 3);
			$pretty = $t . ' msecs';
		} else {
			$pretty = round($this->page_time, 2) . ' Secs';
		}

		return $pretty;
	}
}
?>