<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-200  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require('config/config.php');
	require(LIB_PATH . 'PolarBlog_class.php');
	require(LIB_PATH . 'feedcreator.class.php');

	if (DEFINED('USE_RSS') && !USE_RSS) {
		header('Location: ' . BASE_URL);
		exit;
	}

	//--- All or Topic?
	$clean['tid'] = FALSE;
	$rss_file = 'rss20.xml';
	if (isset($_GET['tid']) && preg_match('/^\d+$/', $_GET['tid'])) {
		$clean['tid'] = (int) $_GET['tid'];
		$rss_file = 'rss20_tid_' . $clean['tid'] . '.xml';
	}

	//--- Create Blog Object
	$blog = new PolarBlog(DB_PREFIX);
	$blog->setDisplayCnt(1);
	$blog->setEntries(NULL, FALSE, $clean['tid'], TRUE);

	//--- Empty/Invalid Topic
	if (count($blog->getEntries()) == 0) {
		echo 'No Data Found.';
		exit;
	}

	//--- Check Last Updated
	$last_update = 0;
	//--- SPL Support
	if ($blog->getSpl()) {
		foreach (new ArrayObject($blog->getEntries()) as $entry) {
			$last_update = $entry->getUpdated();
		}
	} else {
		foreach ($blog->getEntries() as $entry) {
			$last_update = $entry->getUpdated();
		}
	}


	//--- Headers & Redirect
	$now = time();
	$ttl = (isset($config['rss']['optional']['ttl'])) ? $config['rss']['optional']['ttl'] * 60 : 3600;
	if (file_exists($rss_file)) {
		$stat = stat($rss_file);
		$rss_created = $stat['mtime'];
		$last_modified = gmdate('D, d M Y H:i:s', $rss_created) . ' GMT';
		$etag = '"' . md5($last_modified) . '"';
		$rss_expires = $rss_created + $ttl;
		$new_entry = ($last_update > $rss_created) ? TRUE : FALSE;

		// RSS still valid
		header('Expires: ' . gmdate('D, d M Y H:i:s', $now + $rss_expires) . ' GMT');
		header('Last-Modified: ' . $last_modified);
		header('ETag: ' . $etag);
		if (!$new_entry) {
				header('Location: ' . BASE_URL . $rss_file);
				exit;
		} elseif (!$new_entry && ($rss_expires > $now)) {
			//REF: http://simon.incutio.com/archive/2003/04/23/conditionalGet
			// See if the client has provided the required headers
    		$if_modified_since = (isset($_SERVER['HTTP_IF_MODIFIED_SINCE'])) ? stripslashes($_SERVER['HTTP_IF_MODIFIED_SINCE']) : FALSE;
    		$if_none_match = (isset($_SERVER['HTTP_IF_NONE_MATCH'])) ? stripslashes($_SERVER['HTTP_IF_NONE_MATCH']) : FALSE;

			if (
    			(!$if_modified_since && !$if_none_match)  ||
    			($if_none_match && $if_none_match != $etag) ||
    			($if_modified_since && $if_modified_since != $last_modified)
			   ) {
				/*
				 1) Missing required parameters
				 2) At least one of the headers is there - check them
				 3) if-modified-since is there but doesn't match
				*/
				header('Location: ' . BASE_URL . $rss_file);
				exit;
    		} else {
    			// Nothing has changed since their last request - serve a 304 and exit
   				header('HTTP/1.0 304 Not Modified');
   				exit;
    		}
		}
	}


	//--- Outdated or Non-Existent RSS File

	//--- Reset Blog Object
	$feed_cnt = (!empty($config['rss']['entry_count'])) ? $config['rss']['entry_count'] : DEFAULT_DISPLAY_CNT;
	$blog->setDisplayCnt($feed_cnt);
	$blog->setEntries(NULL, FALSE, $clean['tid'], TRUE);

	//--- Build Feed
	//REF: http://www.bitfolge.de/rsscreator-en.html
	$rss = new UniversalFeedCreator();
	$rss->title = $config['rss']['required']['title'];
	$rss->description = $config['rss']['required']['description'];
	$rss->link = $config['rss']['required']['link'];
	$rss->syndicationURL = $config['rss']['required']['link'] . 'rss.php';
	if ($clean['tid']) {
		$blog->setTopics();
		$rss->title .= ' - ' . $blog->topics[$clean['tid']]->getName();
		$rss->description .=  ' - ' . $blog->topics[$clean['tid']]->getName();
		$rss->link .= '?tid=' . $clean['tid'];
		$rss->syndicationURL .= '?tid=' . $clean['tid'];
	}
	if (isset($config['rss']['required']['contentType'])) {
		$rss->contentType = $config['rss']['required']['contentType'];
	}

	//--- Optionals
	if (isset($config['rss']['optional']['copyright'])) {
		$rss->copyright = str_replace('::YEAR::', gmdate('Y'), $config['rss']['optional']['copyright']);
	}
	if (isset($config['rss']['optional']['docs'])) {
		$rss->docs = $config['rss']['optional']['docs'];
	}
	if (isset($config['rss']['optional']['webMaster'])) {
		$rss->webmaster = $config['rss']['optional']['webMaster'];
	}
	if (isset($config['rss']['optional']['managingEditor'])) {
		$rss->editor = $config['rss']['optional']['managingEditor'];
	}
	if (isset($config['rss']['optional']['language'])) {
		$rss->language = $config['rss']['optional']['language'];
	}
	if (isset($config['rss']['feed']['encoding'])) {
		$rss->encoding = $config['rss']['feed']['encoding'];
	}

	//--- Build Entries

	//--- SPL Support
	if ($blog->getSpl()) {
		foreach (new ArrayObject($blog->entries) as $entry) {
			$item = new FeedItem();
			$item->title = $entry->getTitle();
			$item->link = $config['rss']['required']['link'] . '?eid=' . $entry->getId();
			$item->description = $entry->getBody();
			if ($entry->getMoreExists()) {
				$item->description .= '<br /><br />';
				$item->description .= $entry->getMore();
			}
			$item->date = gmdate('D, d M Y H:i:s \G\M\T', $entry->getUpdated());
			$item->source = $config['rss']['required']['link'];
			$rss->addItem($item);
		}
	} else {
		foreach ($blog->entries as $entry) {
			$item = new FeedItem();
			$item->title = $entry->getTitle();
			$item->link = $config['rss']['required']['link'] . '?eid=' . $entry->getId();
			$item->description = $entry->getBody();
			if ($entry->getMoreExists()) {
				$item->description .= '<br /><br />';
				$item->description .= $entry->getMore();
			}
			$item->date = gmdate('D, d M Y H:i:s \G\M\T', $entry->getUpdated());
			$item->source = $config['rss']['required']['link'];
			$rss->addItem($item);
		}
	}

	$rss->saveFeed('RSS2.0', $rss_file);
	$stat = stat($rss_file);
	$rss_created = $stat['mtime'];
	$rss_expires = $rss_created + $ttl;
	header('Expires: ' . gmdate('D, d M Y H:i:s', $rss_expires) . ' GMT');
	header('Last-Modified: ' . gmdate('D, d M Y H:i:s', $rss_created) . ' GMT');
	echo $rss_file;
?>
